#include "urbi.h"
#include "captureimage.h"


ConnectAibo::ConnectAibo(QWidget *parent)
        : QWidget(parent)
{
     robotI = NULL;  
     buffer = NULL;
     bufferRGB = NULL;   

     traitementEnCours=0; 
      visible = 0;
     capture=0;
}

UCallbackAction ConnectAibo::command(const UMessage &msg) 
{
  if (msg.type != MESSAGE_DATA || msg.value->type != urbi::DATA_BINARY || msg.value->binary->type != urbi::BINARY_IMAGE)
    return URBI_CONTINUE;


 if(traitementEnCours == 0)
 {
      traitementEnCours=1;

	  urbi::UImage& img = msg.value->binary->image;

      int width = img.width;
	  int height = img.height;
      int size3b = width*height*3;


      if (buffer == NULL && bufferRGB == NULL)
      {
    		       buffer = new unsigned char[img.width * img.height * 3 * 5]; // 24 bits
     		       bufferRGB = new unsigned char[img.width * img.height * 4 * 5];  // 32 bits
      }



      // conversion de l'image en RGB
      if( img.imageFormat == urbi::IMAGE_JPEG )  // cas ou l'image est un jpeg
          convertJPEGtoRGB( (const byte *) img.data, img.size, (byte *) buffer, size3b);
      else if ( img.imageFormat == urbi::IMAGE_YCbCr)
          convertYCrCbtoRGB( (const byte *) img.data, img.size, (byte*) buffer );
      else if ( img.imageFormat == urbi::IMAGE_RGB)
            memcpy(  buffer, img.data, img.size );
      else
            memcpy(  buffer, img.data, img.size );



      if (visible == 1)
      {

         int positionx =  (int) ((posx + 0.5) * width);
         int positiony =  (int) ((posy + 0.5) * height);

         positionx =  width - positionx;
         positiony =  height - positiony;

           for(int i=0; i<=width; i++)
           {
                      buffer[(3*i) + (positiony*width*3) + 2] = 255;  // bleu
                      buffer[(3*i) + (positiony*width*3) + 1] = 255;  //vert
                      buffer[(3*i) + (positiony*width*3) + 0] = 255;  // rouge
           }                      
                      
           for(int j=0; j<=height; j++)
           {
                      buffer[(3*positionx) + (j*width*3) + 2] = 255;  // bleu
                      buffer[(3*positionx) + (j*width*3) + 1] = 255;  //vert
                      buffer[(3*positionx) + (j*width*3) + 0] = 255;  // rouge
           }                      
      }


      // conversion de l'image RGB 24 en RGB 32
       for(int i=0; i<=width; i++)
       {
             for(int j=0; j<=height; j++)
             {
                  bufferRGB[(4*i) + (j*width*4) + 0] = buffer[(3*i) + (j*width*3) + 2];  // bleu
                  bufferRGB[(4*i) + (j*width*4) + 1] = buffer[(3*i) + (j*width*3) + 1];  //vert
                  bufferRGB[(4*i) + (j*width*4) + 2] = buffer[(3*i) + (j*width*3) + 0];  // rouge
                  bufferRGB[(4*i) + (j*width*4) + 3] = 0;        // vide
             }
       }    

/*
// dans le cas d'un mac os x, la conversion est differente
       for(int i=0; i<=width; i++)
       {
             for(int j=0; j<=height; j++)
             {
                  bufferRGB[(4*i) + (j*width*4) + 0] = 0;//  // vide
                  bufferRGB[(4*i) + (j*width*4) + 1] = buffer[(3*i) + (j*width*3) + 0];  // ROUGE
                  bufferRGB[(4*i) + (j*width*4) + 2] = buffer[(3*i) + (j*width*3) + 1];  // VERT
                  bufferRGB[(4*i) + (j*width*4) + 3] = buffer[(3*i) + (j*width*3) + 2];  // BLEU
             }
       }  

*/


       // copie de l'image dans QImage
      imageQT=QImage( bufferRGB, width, height, QImage::Format_RGB32 ); // on stocke l'image

      // on recupere la largeur et la hauteur
      int widthLabel = pere->ui.label_2->width();
      int heightLabel  = pere->ui.label_2->height();

      if( heightLabel < (widthLabel * height / width) )
          imageQT2 = imageQT.scaledToHeight ( heightLabel - 2 );          // on double la taille
      else if( widthLabel < (heightLabel * width / height) )
          imageQT2 = imageQT.scaledToWidth ( widthLabel - 2 );          // on double la taille
      else 


          imageQT2 = imageQT;
  
       e1 = new QEvent((QEvent::Type)(1001));
       QCoreApplication::postEvent(this,e1);
    
      traitementEnCours=0;

 }



  return URBI_CONTINUE;
}


UCallbackAction ConnectAibo::recupx(const UMessage &msg) 
{
  if (msg.type != MESSAGE_DATA || msg.value->type != urbi::DATA_DOUBLE)
    return URBI_CONTINUE;

  double value = msg.value->val;
  
  if( value == -1)
  {
      visible = 0;
      posx = value;
  }

  else
  {
      visible = 1;
      posx = value;
  }

  return URBI_CONTINUE;
}


UCallbackAction ConnectAibo::recupy(const UMessage &msg) 
{
  if (msg.type != MESSAGE_DATA || msg.value->type != urbi::DATA_DOUBLE)
    return URBI_CONTINUE;

  double value = msg.value->val;

  if( value == -1)
  {
      visible = 0;
      posy = 0;
  }

  else
  {
      visible = 1;
      posy = value;
  }


  return URBI_CONTINUE;
}


///////////////////////////////////////////
// se declenche quand des evenements sont recus
bool ConnectAibo::event(QEvent *ev)
{

 if( ev->type() == (QEvent::Type)(1001))
 {
      pere->ui.label_2->setPixmap(QPixmap::fromImage(imageQT2));     // on affiche
 }

 return 1;
}
