#include <urbi/uclient.hh>
# include "monitor.h"

class BallTrackingHead
{
 public:
  BallTrackingHead(const char * robotName);

  //callback functions
  urbi::UCallbackAction getImage(const urbi::UMessage &msg);

  Monitor* mon;

 private:
  urbi::UClient robotC;
  unsigned char *image;  //uncompressed image
};

urbi::UCallbackAction BallTrackingHead::getImage(const urbi::UMessage &msg)
{

  if (msg.type != urbi::MESSAGE_DATA
      || msg.value->type != urbi::DATA_BINARY
      || msg.value->binary->type != urbi::BINARY_IMAGE)
    return urbi::URBI_CONTINUE;

  urbi::UImage& img = msg.value->binary->image;

  int imgsize = img.width * img.height * 3;

   if (image == NULL)
        image = new unsigned char[img.width * img.height * 3]; // 24 bits

      // conversion de l'image en RGB
      if( img.imageFormat == urbi::IMAGE_JPEG )  // cas ou l'image est un jpeg
          urbi::convertJPEGtoRGB( (const byte *) img.data, img.size, (byte *) image, imgsize);
      else if ( img.imageFormat == urbi::IMAGE_YCbCr)
          urbi::convertYCrCbtoRGB( (const byte *) img.data, img.size, (byte*) image );
      else if ( img.imageFormat == urbi::IMAGE_RGB)
            memcpy(  image, img.data, img.size );
      else
            memcpy(  image, img.data, img.size );

  if (!mon)
    mon = new Monitor(img.width, img.height, "Image");

  mon->setImage((bits8*)image, img.width*img.height*3);

  return urbi::URBI_CONTINUE;
}


BallTrackingHead::BallTrackingHead(const char * robotname) :  robotC (robotname)
{
  robotC.start();
  if (robotC.error())
    urbi::exit(1);

  image = NULL;
  mon = NULL;

  robotC.setCallback(*this, &BallTrackingHead::getImage,"cam");
  robotC.send("loop cam: camera.val, ");

}
