#include "urbi.h"
#include "tele.h"

ConnectAibo::ConnectAibo(QWidget *parent)
        : QWidget(parent)
{
     robotC = NULL;                           
     initialized = false;
     errorfmod = false;
}



UCallbackAction ConnectAibo::getSound(const UMessage &msg) 
{

  // si ce n'est pas du son, alors on quitte
  if (msg.type != MESSAGE_DATA || msg.value->type != urbi::DATA_BINARY || msg.value->binary->type != urbi::BINARY_SOUND)
    return URBI_CONTINUE;


  if ( initialized == false ) 
  {
        initialized = true;
    
    	channel=0;
    
    	result = FMOD_System_Create(&system);
    	if ( ERRCHECK(result) == -1 )
        {
          errorfmod = true;
          QMessageBox::information(this, "Error", "FMOD_System_Create Fail");
          return URBI_CONTINUE;
        }
            
    	result = FMOD_System_GetVersion(system, &version);
    	if ( ERRCHECK(result) == -1 )
        {
          errorfmod = true;
          QMessageBox::information(this, "Error", "FMOD_System_GetVersion Fail");
          return URBI_CONTINUE;
        }
            
    	if (version < FMOD_VERSION)
        {
          errorfmod = true;
          QMessageBox::information(this, "Error", "Error!  You are using an old version of FMOD.");
          return URBI_CONTINUE;
        }
    
            
    	result = FMOD_System_Init(system, 32, FMOD_INIT_NORMAL, NULL);
    	if ( ERRCHECK(result) == -1 )
        {
          errorfmod = true;
          QMessageBox::information(this, "Error", "FMOD_INIT_NORMAL Fail");
          return URBI_CONTINUE;
        }

  }

 
     if(errorfmod == true)
      return URBI_CONTINUE;

    urbi::USound& soundaudio = msg.value->binary->sound;
 
           if ( soundaudio.soundFormat == SOUND_RAW )
           {
                USound tmpSnd;
                tmpSnd.soundFormat = SOUND_WAV;
                tmpSnd.rate = soundaudio.rate;
                tmpSnd.sampleSize = soundaudio.sampleSize;
                tmpSnd.sampleFormat = soundaudio.sampleFormat;
                tmpSnd.channels = soundaudio.channels;
                tmpSnd.data = 0;
                tmpSnd.size = 0;
                convert( soundaudio, tmpSnd ); 

               file.setFileName("soundFile");  // ouverture du fichier
               file.open(QIODevice::WriteOnly );
//               outfile = new  QDataStream(&file);
//               outfile->writeRawData ( tmpSnd.data, tmpSnd.size );
               file.write ( tmpSnd.data, tmpSnd.size ); // on copie le contenu de texte dans le fichier
//               delete outfile;
               file.close();  // on ferme le fichier
           }
           else if ( soundaudio.soundFormat == SOUND_WAV )
           {
               file.setFileName("soundFile");  // ouverture du fichier
               file.open(QIODevice::WriteOnly );
//               outfile = new  QDataStream(&file);
//               outfile->writeRawData ( soundaudio.data, soundaudio.size );
               file.write ( soundaudio.data, soundaudio.size ); // on copie le contenu de texte dans le fichier
//               delete outfile;
               file.close();  // on ferme le fichier
           }
        
        
            result = FMOD_System_CreateSound(system, "soundFile", FMOD_LOOP_OFF | FMOD_2D | FMOD_SOFTWARE, 0, &sound1);
            if ( ERRCHECK(result) == -1 )
            {
               errorfmod = true;
               QMessageBox::information(this, "Error", "FMOD_System_CreateSound Fail");
               return URBI_CONTINUE;
            }
        
            result = FMOD_System_PlaySound(system, FMOD_CHANNEL_FREE, sound1, false, &channel); 
            if ( ERRCHECK(result) == -1 )
            {
               errorfmod = true;
               QMessageBox::information(this, "Error", "FMOD_System_PlaySound Fail");
               return URBI_CONTINUE;
            }
        
            FMOD_System_Update(system);
        
   return URBI_CONTINUE;

}


/////////////////////////////////////////////
// ERRCHECK
/////////////////////////////////////////////
int ConnectAibo::ERRCHECK(FMOD_RESULT result)
{
    if (result != FMOD_OK)
    {
//        printf("FMOD error! (%d) %s\n", result, FMOD_ErrorString(result));
        return -1;
    }
    return 0;
}
/////////////////////////////////////////////
// ERRCHECK
/////////////////////////////////////////////
