#include "urbi.h"
#include "tele.h"

ConnectAibo::ConnectAibo(QWidget *parent)
        : QWidget(parent)
{
     robotC = NULL;                           

     // page 6
     robotI = NULL; 
     traitementEnCours=0;
     buffer = NULL;
     bufferRGB=NULL;
     power=100;

     // son
     robotS = NULL; 
     initialized = false;
     errorfmod = false;

      reception = 0;
}





/////////////////////////////////////////////////
// Recuperer les images
//////////////////////////////////////////////////
UCallbackAction ConnectAibo::allrecupimg(const UMessage &msg) 
{ 

  if (msg.type != MESSAGE_DATA && msg.value->type != urbi::DATA_BINARY) 
    return URBI_CONTINUE;

  if (msg.value->binary->type != urbi::BINARY_IMAGE)
    return URBI_CONTINUE;


 if(traitementEnCours == 0)
 {
    traitementEnCours=1;

	urbi::UImage& img = msg.value->binary->image;

     int width = img.width;
     int height = img.height;


      if (buffer == NULL && bufferRGB == NULL)
      {
    		       buffer = new unsigned char[img.width * img.height * 3 * 5]; // 24 bits
     		       bufferRGB = new unsigned char[img.width * img.height * 4 * 5];  // 32 bits
      }


    		    if( img.imageFormat == urbi::IMAGE_JPEG )  // cas ou l'image est un jpeg
               {
    		        imageQT.loadFromData ( (const uchar *) img.data, img.size, "JPEG" );
              }

     		   else if ( img.imageFormat == urbi::IMAGE_YCbCr)
     		   {
     		           convertYCrCbtoRGB( (const byte *) img.data, img.size, (byte*) buffer );
        
              // conversion de l'image RGB 24 en RGB 32
               for(int i=0; i<=width; i++)
               {
                     for(int j=0; j<=height; j++)
                     {
                          bufferRGB[(4*i) + (j*width*4) + 0] = buffer[(3*i) + (j*width*3) + 2];  // bleu
                          bufferRGB[(4*i) + (j*width*4) + 1] = buffer[(3*i) + (j*width*3) + 1];  //vert
                          bufferRGB[(4*i) + (j*width*4) + 2] = buffer[(3*i) + (j*width*3) + 0];  // rouge
                          bufferRGB[(4*i) + (j*width*4) + 3] = 0;        // vide
                     }
               }    
        
               // copie de l'image dans QImage
              imageQT=QImage( bufferRGB, width, height, QImage::Format_RGB32 ); // on stocke l'image
        }

      // on recupere la largeur et la hauteur
      int widthLabel = pere->ui.label_7->width();
      int heightLabel  = pere->ui.label_7->height();

      if( heightLabel < (widthLabel * height / width) )
          imageQT2 = imageQT.scaledToHeight ( heightLabel - 4 );          // on double la taille
      else if( widthLabel < (heightLabel * width / height) )
          imageQT2 = imageQT.scaledToWidth ( widthLabel - 4 );          // on double la taille
      else 
          imageQT2 = imageQT;

//     imageQT2 = imageQT.scaledToHeight ( 320 );          // on double la taille


      e4 = new QEvent((QEvent::Type)(1003));
      QCoreApplication::postEvent(this,e4);  // on affiche l'image

      
      traitementEnCours=0;

  }


  return URBI_CONTINUE;

}




/////////////////////////////////////////////////
// Recuperer le niveau d'energie
//////////////////////////////////////////////////
UCallbackAction ConnectAibo::allrecupenergie(const UMessage &msg) 
{ 

  if (msg.type != MESSAGE_DATA || msg.value->type != urbi::DATA_DOUBLE)
    return URBI_CONTINUE;

  double value = msg.value->val * 100;
  power = (int) value;

   e4 = new QEvent((QEvent::Type)(1004));
   QCoreApplication::postEvent(this,e4);  // on affiche l'image

  return URBI_CONTINUE;

}


UCallbackAction ConnectAibo::getSoundFMOD(const UMessage &msg) 
{

  // si ce n'est pas du son, alors on quitte
  if (msg.type != MESSAGE_DATA || msg.value->type != urbi::DATA_BINARY || msg.value->binary->type != urbi::BINARY_SOUND)
    return URBI_CONTINUE;


  if ( initialized == false ) 
  {
        initialized = true;
    
    	channel=0;
    
    	result = FMOD_System_Create(&system);
    	if ( ERRCHECK(result) == -1 )
        {
          errorfmod = true;
          QMessageBox::information(this, "Error", "FMOD_System_Create Fail");
          return URBI_CONTINUE;
        }
            
    	result = FMOD_System_GetVersion(system, &version);
    	if ( ERRCHECK(result) == -1 )
        {
          errorfmod = true;
          QMessageBox::information(this, "Error", "FMOD_System_GetVersion Fail");
          return URBI_CONTINUE;
        }
            
    	if (version < FMOD_VERSION)
        {
          errorfmod = true;
          QMessageBox::information(this, "Error", "Error!  You are using an old version of FMOD.");
          return URBI_CONTINUE;
        }
    
            
    	result = FMOD_System_Init(system, 32, FMOD_INIT_NORMAL, NULL);
    	if ( ERRCHECK(result) == -1 )
        {
          errorfmod = true;
          QMessageBox::information(this, "Error", "FMOD_INIT_NORMAL Fail");
          return URBI_CONTINUE;
        }

  }

 
     if(errorfmod == true)
      return URBI_CONTINUE;

    urbi::USound& soundaudio = msg.value->binary->sound;
 
           if ( soundaudio.soundFormat == SOUND_RAW )
           {
                USound tmpSnd;
                tmpSnd.soundFormat = SOUND_WAV;
                tmpSnd.rate = soundaudio.rate;
                tmpSnd.sampleSize = soundaudio.sampleSize;
                tmpSnd.sampleFormat = soundaudio.sampleFormat;
                tmpSnd.channels = soundaudio.channels;
                tmpSnd.data = 0;
                tmpSnd.size = 0;
                convert( soundaudio, tmpSnd ); 

               fileFMOD.setFileName("soundFileFMOD.wav");  // ouverture du fichier
               fileFMOD.open(QIODevice::WriteOnly );
//               outfile = new  QDataStream(&file);
//               outfile->writeRawData ( tmpSnd.data, tmpSnd.size );
               fileFMOD.write ( tmpSnd.data, tmpSnd.size ); // on copie le contenu de texte dans le fichier
//               delete outfile;
               fileFMOD.close();  // on ferme le fichier
           }
        
        
            result = FMOD_System_CreateSound(system, "soundFileFMOD.wav", FMOD_LOOP_OFF | FMOD_2D | FMOD_SOFTWARE, 0, &sound1);
            if ( ERRCHECK(result) == -1 )
            {
               errorfmod = true;
               QMessageBox::information(this, "Error", "FMOD_System_CreateSound Fail");
               return URBI_CONTINUE;
            }
        
            result = FMOD_System_PlaySound(system, FMOD_CHANNEL_FREE, sound1, false, &channel); 
            if ( ERRCHECK(result) == -1 )
            {
               errorfmod = true;
               QMessageBox::information(this, "Error", "FMOD_System_PlaySound Fail");
               return URBI_CONTINUE;
            }
        
            FMOD_System_Update(system);
        
   return URBI_CONTINUE;

}


/////////////////////////////////////////////
// ERRCHECK
/////////////////////////////////////////////
int ConnectAibo::ERRCHECK(FMOD_RESULT result)
{
    if (result != FMOD_OK)
    {
//        printf("FMOD error! (%d) %s\n", result, FMOD_ErrorString(result));
        return -1;
    }
    return 0;
}
/////////////////////////////////////////////
// ERRCHECK
/////////////////////////////////////////////




///////////////////////////////////////////
// se declenche quand des evenements sont recus
bool ConnectAibo::event(QEvent *ev)
{
 
 if( ev->type() == (QEvent::Type)(1003))
 {
        pere->ui.label_7->setPixmap(QPixmap::fromImage(imageQT2));     // on affiche   

       // creation de la couleur rouge
       QColor couleurR ( 127, 127, 127 ); 
       // creation de la couleur verte
       QColor couleurV ( 0, 255, 0 ); 

      if( reception == 0)
      {
            pere->ui.pushButton_6->setPalette(couleurV);               
            reception = 1;
      }
      else
      {
            pere->ui.pushButton_6->setPalette(couleurR);  
            reception = 0;
      }

}
 else if( ev->type() == (QEvent::Type)(1004))
        pere->ui.progressBar->setValue(power);

 return 1;
}
