#include "urbi.h"
#include "captureimage.h"


ConnectAibo::ConnectAibo(QWidget *parent)
        : QWidget(parent)
{
     robotI = NULL;  
     buffer = NULL;
     bufferRGB = NULL;   

     traitementEnCours=0; 
     visible = 0;
     capture = 0;
     captureImage = 0;

     compteur=0;

     VideoFileName = "video.mpeg";
     nbimagescaptures=0;
     frames=0;
     initrecord=0;
     closerecord=0;
}

UCallbackAction ConnectAibo::command(const UMessage &msg) 
{

  if (msg.type != MESSAGE_DATA || msg.value->type != urbi::DATA_BINARY || msg.value->binary->type != urbi::BINARY_IMAGE)
       return URBI_CONTINUE;

 if(traitementEnCours == 0)
 {
      traitementEnCours=1;
	urbi::UImage& img = msg.value->binary->image;

    		    int width = img.width;
    		    int height = img.height;
                int size3d = img.width * img.height * 3;

      if (buffer == NULL && bufferRGB == NULL)
      {
    		       buffer = new unsigned char[img.width * img.height * 3 * 8]; // 24 bits
     		       bufferRGB = new unsigned char[img.width * img.height * 4 * 8];  // 32 bits
      }



                   if(img.imageFormat == IMAGE_JPEG )
                           convertJPEGtoRGB ( (const byte *) img.data, img.size, (byte *) buffer , size3d ); // on copie les donnees dans tab
                   else if ( img.imageFormat == urbi::IMAGE_YCbCr)
         		           convertYCrCbtoRGB( (const byte *) img.data, img.size, (byte*) buffer );
        	       else if ( img.imageFormat == urbi::IMAGE_RGB)
           		          memcpy(  buffer, img.data, img.size );
        	       else
            		       memcpy( buffer, img.data, img.size );
                
                // conversion de l'image RGB 24 en RGB 32
                   for(int i=0; i<=width; i++)
                    {
                     for(int j=0; j<=height; j++)
                      {
                               bufferRGB[(4*i) + (j*width*4) + 0] = buffer[(3*i) + (j*width*3) + 2];  // bleu
                               bufferRGB[(4*i) + (j*width*4) + 1] = buffer[(3*i) + (j*width*3) + 1];  //vert
                               bufferRGB[(4*i) + (j*width*4) + 2] = buffer[(3*i) + (j*width*3) + 0];  // rouge
                               bufferRGB[(4*i) + (j*width*4) + 3] = 0;        // vide
                       }
                     }    
                    
                   // copie de l'image dans QImage
                   imageQT=QImage( bufferRGB, width, height, QImage::Format_RGB32 ); // on stocke l'image

      // on recupere la largeur et la hauteur
      int widthLabel = pere->ui.label_2->width();
      int heightLabel  = pere->ui.label_2->height();

      if( heightLabel < (widthLabel * height / width) )
          imageQT2 = imageQT.scaledToHeight ( heightLabel - 2 );          // on double la taille
      else if( widthLabel < (heightLabel * width / height) )
          imageQT2 = imageQT.scaledToWidth ( widthLabel - 2 );          // on double la taille
      else 
          imageQT2 = imageQT;
  


      if(capture == 1)
      {

                 nbimagescaptures = nbimagescaptures + 1; // on augmente de 1 le compteur d'images captures
                 frames = frames + 1;

                 if(initrecord == 1) // on initialise l'enregistrement
                {
                    // on recupere le nom du fichier video
                    VideoFileName = pere->ui.lineEdit_4->text();

                    QFile fileinit;
                    fileinit.setFileName(VideoFileName);  // ouverture du fichier video.mpeg
                    fileinit.remove (); // on efface le fichier
                    fileinit.close();             

                    file.setFileName(VideoFileName);  // ouverture du fichier video.mpeg
                    file.open(QIODevice::WriteOnly | QIODevice::Append );


                    // On initialise FFMPEG
                    ////////////////////////
                    // must be called before using avcodec lib 
                    avcodec_init();
                
                    // register all the codecs (you can also register only the codec
                    // you wish to have smaller code 
                    avcodec_register_all();
                
                    // find the mpeg1 video encoder 
                    codec = avcodec_find_encoder(CODEC_ID_MPEG1VIDEO);
                
                    // AVCodec alloc context 
                    c= NULL;
                    c= avcodec_alloc_context();
                    // put sample parameters 
                    c->bit_rate = 400000;
                    // resolution must be a multiple of two 
                    c->width = img.width;
                    c->height = img.height;
                    // frames per second 
                    c->time_base= (AVRational){1,25}; // 1 seconde = 25 images
                    c->gop_size = 10; // emit one intra frame every ten frames 
                    c->max_b_frames=1;
                    c->pix_fmt = PIX_FMT_YUV420P;
                
                    // open it 
                    avcodec_open(c, codec) ;
                
                    // Allocate an AVFrame structure
                    picture = avcodec_alloc_frame();
                
                    pictureRGB24 = avcodec_alloc_frame();
              
                    // the codec gives us the frame size, in samples 
                
                    // alloc image and output buffer 
                    outbuf_size = 100000;
                    outbuf = (uint8_t* ) malloc(outbuf_size);
                
                    size = avpicture_get_size(PIX_FMT_YUV420P, c->width, c->height); // size for YUV420P 
                    picture_buf1 = (uint8_t* )malloc( sizeof(uint8_t)*size ); 
                    picture_buf2 = (uint8_t* )malloc( sizeof(uint8_t)*size ); 
                    picture_buf3 = (uint8_t* )malloc( sizeof(uint8_t)*size ); 
                
                    sizeRGB24 = avpicture_get_size(PIX_FMT_RGB24, c->width, c->height); // size for RGB24 
                    picture_bufRGB24 = (uint8_t* )malloc( sizeof(uint8_t)*sizeRGB24 ); 
                
                    picture->data[0] = picture_buf1; // correspond a Y
                    picture->data[1] = picture_buf2; // correspond a U
                    picture->data[2] = picture_buf3; // correspond a V
                    picture->linesize[0] = c->width; // largeur
                    picture->linesize[1] = c->width/2; // moitie de la largeur
                    picture->linesize[2] = c->width/2; // moitie de la largeur
                
                    pictureRGB24->data[0] = picture_bufRGB24; // correspond a l'image RGB
                    pictureRGB24->linesize[0] = c->width*3; //largeur
                } // fin du mode initialisation



                //////////////////////////////
                // enregistrement de l'image
               
           	        fflush(stdout);
                    // creation de l'image RGB24
                    avpicture_fill((AVPicture *)pictureRGB24, (uint8_t*) buffer, PIX_FMT_RGB24, width, height);
                
                    // conversion RGB24 en YUV420P
                    img_convert((AVPicture *)picture, PIX_FMT_YUV420P, (AVPicture*)pictureRGB24, PIX_FMT_RGB24, width, height);
                
             	        // encode the image 
              	        out_size = avcodec_encode_video(c, outbuf, outbuf_size, picture);
                        file.write((const char*)outbuf, out_size); // write to file
                        fflush(stdout);
                        out_size = avcodec_encode_video(c, outbuf, outbuf_size, NULL);
                        file.write((const char*)outbuf, out_size); // write to file


                //////////////////////////////
                // fin de l'enregistrement de l'image



                 if(closerecord == 1) // on termine l'enregistrement
                {
                   capture=0; // on termine le mode capture
                   closerecord=0; 


                    // get the delayed frames 
                    int z=0;
                    for(z=0; out_size; z++) 
                    {
                        fflush(stdout);
                
                        out_size = avcodec_encode_video(c, outbuf, outbuf_size, NULL);
                        file.write((const char*)outbuf, out_size); // write to file
                    }
                
                    // add sequence end code to have a real mpeg file
                    outbuf[0] = 0x00;
                    outbuf[1] = 0x00;
                    outbuf[2] = 0x01;
                    outbuf[3] = 0xb7;
                    file.write((const char*)outbuf, 4); // write to file
                    file.close();
                    free(picture_buf);
                
                    // Close the codec
                    avcodec_close(c);
                    av_free(c);
                    av_free(picture);

                }// fin du mode de fermeture du fichier

      } // fin du mode enregistrement


       e1 = new QEvent((QEvent::Type)(1001));
       QCoreApplication::postEvent(this,e1);

      traitementEnCours=0;
  }

  return URBI_CONTINUE;
  
  
}

///////////////////////////////////////////
// se declenche quand des evenements sont recus
bool ConnectAibo::event(QEvent *ev)
{

 if( ev->type() == (QEvent::Type)(1001))
 {
      pere->ui.label_2->setPixmap(QPixmap::fromImage(imageQT2));     // on affiche
      pere->ui.lcdNumber->display ( (int) nbimagescaptures );  // on donne le nb d'images capturees
 }

 return 1;
}
