#include "urbi.h"
#include "captureimage.h"

#include "urbi_page1.cpp"

ConnectAibo::ConnectAibo(QWidget *parent)
        : QWidget(parent)
{
     robotI = NULL;  
     buffer = NULL;
     bufferRGB = NULL;   

     traitementEnCours=0; 

     compteur=0;


///////////////////////////////////////////////
// Traitement de stabilisation

   cmd_nfeatures = 50;
   compteur_images = 0;
   nFrames=5; // on travaille sur 5 images

   bufferKLT1 = NULL;
   bufferKLT2 = NULL;
   bufferKLT3 = NULL;
   bufferKLT4 = NULL;
   bufferKLT5 = NULL;
   bufferKLT_temp = NULL;

   bufferKLT1_32 = NULL;
   bufferKLT2_32 = NULL;
   bufferKLT3_32 = NULL;
   bufferKLT3_32bis = NULL;

   xa1_tab = NULL;
   ya1_tab = NULL;
   xa2_tab = NULL;
   ya2_tab = NULL;
   diff_tab = NULL;
   variance_tab = NULL;

   tc = KLTCreateTrackingContext();
   KLTPrintTrackingContext(tc);
   fl = KLTCreateFeatureList(cmd_nfeatures); // permet de stocker une liste de features
   fl_temp = KLTCreateFeatureList(cmd_nfeatures); // permet de stocker une liste de features
   ft = KLTCreateFeatureTable(nFrames, cmd_nfeatures); // permet de stocker des listes de features
   tc->sequentialMode = TRUE; // les images se suivent. Ce mode fait que le traitement est plus rapide

   fh = KLTCreateFeatureHistory(nFrames); // permet de stocker des listes de features

   global_Tx = 0;
   global_Ty = 0;
   global_teta = 0;

   somme_global_Tx = 0;
   somme_global_Ty = 0;
   somme_global_teta = 0; 

// Traitement de stabilisation
///////////////////////////////////////////////


}

UCallbackAction ConnectAibo::command(const UMessage &msg) 
{

  if (msg.type != MESSAGE_DATA || msg.value->type != urbi::DATA_BINARY || msg.value->binary->type != urbi::BINARY_IMAGE)
       return URBI_CONTINUE;

 if(traitementEnCours == 0)
 {
      traitementEnCours=1;
	urbi::UImage& img = msg.value->binary->image;

    		    int width = img.width;
    		    int height = img.height;

      if (buffer == NULL && bufferRGB == NULL)
      {
    		       buffer = new unsigned char[img.width * img.height * 3 * 8]; // 24 bits
     		       bufferRGB = new unsigned char[img.width * img.height * 4 * 8];  // 32 bits
      }

                int size3b = width*height*3;
    		    if( img.imageFormat == urbi::IMAGE_JPEG )  // cas ou l'image est un jpeg
    		        convertJPEGtoRGB( (const byte *) img.data, img.size, (byte *) buffer, size3b);
                else if ( img.imageFormat == urbi::IMAGE_YCbCr)
    		        convertYCrCbtoRGB( (const unsigned char *) img.data, img.size, (unsigned char*) buffer );
                else if ( img.imageFormat == urbi::IMAGE_RGB)
    		        memcpy(  buffer, img.data, img.size );
                else
    		        memcpy( buffer, img.data, img.size );
            
                // conversion de l'image RGB 24 en RGB 32
                for(int i=0; i<=width; i++)
                {
                             for(int j=0; j<=height; j++)
                             {
                                  bufferRGB[(4*i) + (j*width*4) + 0] = buffer[(3*i) + (j*width*3) + 2];  // bleu
                                  bufferRGB[(4*i) + (j*width*4) + 1] = buffer[(3*i) + (j*width*3) + 1];  //vert
                                  bufferRGB[(4*i) + (j*width*4) + 2] = buffer[(3*i) + (j*width*3) + 0];  // rouge
                                  bufferRGB[(4*i) + (j*width*4) + 3] = 0;        // vide
                             }
                }    
                
                // copie de l'image dans QImage
                imageQT=QImage( bufferRGB, width, height, QImage::Format_RGB32 ); // on stocke l'image


      // on recupere la largeur et la hauteur
      int widthLabel = pere->ui.label_2->width();
      int heightLabel  = pere->ui.label_2->height();

      if( heightLabel < (widthLabel * height / width) )
      {
          imageQT2 = imageQT.scaledToHeight ( heightLabel - 2 );          // on double la taille
      }
      else if( widthLabel < (heightLabel * width / height) )
      {
          imageQT2 = imageQT.scaledToWidth ( widthLabel - 2 );          // on double la taille
      }
      else 
      {
          imageQT2 = imageQT;
      }






///////////////////////////////////////////////
// Traitement de stabilisation



// *****************************************************************
    // au premier demarrage, on initialise toutes les variables
    stabilisation_init(width, height);
// *****************************************************************



// *****************************************************************
   // conversion de l'image RGB 32 en niveaux de gris
   stabilisation_convertgris(width, height);
// *****************************************************************



// *****************************************************************
   // stockage des features des images
   stabilisation_stockfeatures( width, height);
// *****************************************************************



// *****************************************************************
   // on augmente le compteur tant qu'il est inferieur a 10
   if(compteur_images <= 10)
     compteur_images++;
// *****************************************************************



// *****************************************************************
     // on cherche a trouver le deplacement entre 2 images
     stabilisation_deplacement(width, height);
// *****************************************************************



// *****************************************************************
     // conversion de l'image RGB 32 en niveaux de gris avec points d'interets
     stabilisation_convertgris_with_points(width, height);
// *****************************************************************



// *****************************************************************
     // on recadre l'image pour compenser le bruit
     stabilisation_recadrage(width, height);
// *****************************************************************


// Traitement de stabilisation
/////////////////////////////////








 
       e1 = new QEvent((QEvent::Type)(1001));
       QCoreApplication::postEvent(this,e1);
      
      traitementEnCours=0;
  }


  return URBI_CONTINUE;
    
}

///////////////////////////////////////////
// se declenche quand des evenements sont recus
bool ConnectAibo::event(QEvent *ev)
{

 if( ev->type() == (QEvent::Type)(1001))
 {
      pere->ui.label_2->setPixmap(QPixmap::fromImage(imageQT2));     // on affiche l'image normale
      pere->ui.label_12->setPixmap(QPixmap::fromImage(imageQT4));     // on affiche l'image avec les points d'interet
      pere->ui.label_6->setPixmap(QPixmap::fromImage(imageQT5));     // on affiche l'image corrigee

      pere->ui.lineEdit_4->setText( QString::number(global_Tx) );
      pere->ui.lineEdit_5->setText( QString::number(global_Ty) );
      pere->ui.lineEdit_6->setText( QString::number(global_teta) );

      pere->ui.lineEdit_7->setText( QString::number(somme_global_Tx) );
      pere->ui.lineEdit_8->setText( QString::number(somme_global_Ty) );
      pere->ui.lineEdit_9->setText( QString::number(somme_global_teta) );
 }

 return 1;
}

