#include "urbi.h"
#include "captureimage.h"


ConnectAibo::ConnectAibo(QWidget *parent)
        : QWidget(parent)
{
     robotI = NULL;  
     buffer = NULL;
     bufferRGB = NULL;   

     traitementEnCours=0; 
     captureimg=0;
     captureImage=0;

     compteur=0;

     audemarrage = 0; // pour dire que ca ne s'est jamais lanc

// code opencv
     win_size = 10;
     MAX_COUNT = 500;
     points[0] = NULL; points[1] = NULL;
     status = NULL;
     count = 0;
     need_to_init = 0;
     night_mode = 1; // points d'interet sont sur fond noir
     flags = 0;
     add_remove_pt = 0;

     image = NULL;
     grey = NULL;
     prev_grey = NULL;
     pyramid = NULL;
     prev_pyramid = NULL;
// code opencv

}

UCallbackAction ConnectAibo::command(const UMessage &msg) 
{

  if (msg.type != MESSAGE_DATA || msg.value->type != urbi::DATA_BINARY || msg.value->binary->type != urbi::BINARY_IMAGE)
       return URBI_CONTINUE;

 if(traitementEnCours == 0)
 {
      traitementEnCours=1;
	urbi::UImage& img = msg.value->binary->image;

    		    int width = img.width;
    		    int height = img.height;

      if (buffer == NULL && bufferRGB == NULL)
      {
    		       buffer = new unsigned char[img.width * img.height * 3 * 8]; // 24 bits
     		       bufferRGB = new unsigned char[img.width * img.height * 4 * 8];  // 32 bits
      }


    		    if( img.imageFormat == urbi::IMAGE_JPEG )  // cas ou l'image est un jpeg
    		        imageQT.loadFromData ( (const uchar *) img.data, img.size, "JPEG" );
     		   else 
               {
                   if ( img.imageFormat == urbi::IMAGE_YCbCr)
         		           convertYCrCbtoRGB( (const unsigned char *) img.data, img.size, (unsigned char*) buffer );
      		       else if ( img.imageFormat == urbi::IMAGE_RGB)
       		          memcpy(  buffer, img.data, img.size );
       		       else
        		       memcpy( buffer, img.data, img.size );
            
                      // conversion de l'image RGB 24 en RGB 32
                       for(int i=0; i<=width; i++)
                       {
                             for(int j=0; j<=height; j++)
                             {
                                  bufferRGB[(4*i) + (j*width*4) + 0] = buffer[(3*i) + (j*width*3) + 2];  // bleu
                                  bufferRGB[(4*i) + (j*width*4) + 1] = buffer[(3*i) + (j*width*3) + 1];  //vert
                                  bufferRGB[(4*i) + (j*width*4) + 2] = buffer[(3*i) + (j*width*3) + 0];  // rouge
                                  bufferRGB[(4*i) + (j*width*4) + 3] = 0;        // vide
                             }
                       }    
                
                       // copie de l'image dans QImage
                      imageQT=QImage( bufferRGB, width, height, QImage::Format_RGB32 ); // on stocke l'image
                }

        imageQT3=imageQT; // on copie dans imageQT3, l'image capturee sans modification

// code opencv
     // on enregistre l'image sur le disque dur
     imageQT.save("image_clean.jpg","JPG",100);

      int i, k;
      const char* filename = (char*)"image_clean.jpg";


      // chargement de l'image et des buffers
      if( audemarrage == 0 )
      {
            // allocate all the buffers 
            image = cvLoadImage( filename, 1 );
            grey = cvCreateImage( cvSize(image->width,image->height), 8, 1 );
            prev_grey = cvCreateImage( cvSize(image->width,image->height), 8, 1 );
            pyramid = cvCreateImage( cvSize(image->width,image->height), 8, 1 );
            prev_pyramid = cvCreateImage( cvSize(image->width,image->height), 8, 1 );
            points[0] = (CvPoint2D32f*)cvAlloc(MAX_COUNT*sizeof(points[0][0]));
            points[1] = (CvPoint2D32f*)cvAlloc(MAX_COUNT*sizeof(points[0][0]));
            status = (char*)cvAlloc(MAX_COUNT);
            flags = 0;
            audemarrage = 1;
        }
        else
            image = cvLoadImage( filename, 1 );


        cvCvtColor( image, grey, CV_BGR2GRAY );


/////////////////////////////////
// switch the "night" mode on/off
        if( night_mode )
            cvZero( image );

/////////////////////////////////


/////////////////////////////////
// auto-initialize tracking
        if( need_to_init )
        {
            /* automatic initialization */
            IplImage* eig = cvCreateImage( cvGetSize(grey), 32, 1 );
            IplImage* temp = cvCreateImage( cvGetSize(grey), 32, 1 );
            double quality = 0.01;
            double min_distance = 10;

            count = MAX_COUNT;
            cvGoodFeaturesToTrack( grey, eig, temp, points[1], &count, quality, min_distance, 0, 3, 0, 0.04 );
            cvFindCornerSubPix( grey, points[1], count, cvSize(win_size,win_size), cvSize(-1,-1), cvTermCriteria(CV_TERMCRIT_ITER|CV_TERMCRIT_EPS,20,0.03));
            cvReleaseImage( &eig );
            cvReleaseImage( &temp );

            add_remove_pt = 0;
        }
/////////////////////////////////



        else if( count > 0 )
        {
            cvCalcOpticalFlowPyrLK( prev_grey, grey, prev_pyramid, pyramid, points[0], points[1], count, cvSize(win_size,win_size), 3, status, 0, cvTermCriteria(CV_TERMCRIT_ITER|CV_TERMCRIT_EPS,20,0.03), flags );
            flags |= CV_LKFLOW_PYR_A_READY;
            for( i = k = 0; i < count; i++ )
            {
                if( add_remove_pt )
                {
                    double dx = pt.x - points[1][i].x;
                    double dy = pt.y - points[1][i].y;

                    if( dx*dx + dy*dy <= 25 )
                    {
                        add_remove_pt = 0;
                        continue;
                    }
                }
                
                if( !status[i] )
                    continue;
                
                points[1][k++] = points[1][i];
                cvCircle( image, cvPointFrom32f(points[1][i]), 3, CV_RGB(0,255,0), -1, 8,0);
            }
            count = k;
        }


        if( add_remove_pt && count < MAX_COUNT )
        {
            points[1][count++] = cvPointTo32f(pt);
            cvFindCornerSubPix( grey, points[1] + count - 1, 1, cvSize(win_size,win_size), cvSize(-1,-1), cvTermCriteria(CV_TERMCRIT_ITER|CV_TERMCRIT_EPS,20,0.03));
            add_remove_pt = 0;
        }

        CV_SWAP( prev_grey, grey, swap_temp );
        CV_SWAP( prev_pyramid, pyramid, swap_temp );
        CV_SWAP( points[0], points[1], swap_points );
        need_to_init = 0;


    cvSaveImage("image_cv.jpg", image); // on enregistre l'image sur le disque dur

    cvReleaseImage( &image ); // on efface l'image

     // on charge le nouveau fichier image
     imageQT.load("image_cv.jpg");
// code opencv


      // on recupere la largeur et la hauteur
      int widthLabel = pere->ui.label_2->width();
      int heightLabel  = pere->ui.label_2->height();

      if( heightLabel < (widthLabel * height / width) )
      {
          imageQT2 = imageQT.scaledToHeight ( heightLabel - 2 );          // on double la taille
          imageQT4 = imageQT3.scaledToHeight ( heightLabel - 2 );          // on double la taille
      }
      else if( widthLabel < (heightLabel * width / height) )
      {
          imageQT2 = imageQT.scaledToWidth ( widthLabel - 2 );          // on double la taille
          imageQT4 = imageQT3.scaledToWidth ( widthLabel - 2 );          // on double la taille
      }
      else 
      {
          imageQT2 = imageQT;
          imageQT4 = imageQT3;
      }

  
/////////////////////////////////////////////////////////////  
// Creation de imageQT5 qui contient l'image normale + les modifications dues aux mouvements  
  
int widthimage = imageQT2.width();
int heightimage = imageQT2.height();
int a,b;
QRgb value; // va stocker la couleur du pixel
QRgb valueblack; // va stocker la couleur du pixel
valueblack = qRgb(0, 0, 0); // noir

imageQT5 = imageQT4; // imageQT5 prend la forme de l'image normale sans modification


// on parcourt toute l'image pour recuperer les valeurs
for(a = 0; a < widthimage; a++)
{
  for(b=0; b < heightimage; b++)
  {
     value = imageQT2.pixel( a , b );
     if(value != valueblack)
     {
        if(qBlue(value) > 30 || qGreen(value) > 30 || qRed(value) > 30) // pour supprimer le bruit qui est noir mais pas 100% noir
          imageQT5.setPixel(a, b, value);
     }

  }     
}

//////////////////////////////////////////////////////////////  
  
       e1 = new QEvent((QEvent::Type)(1001));
       QCoreApplication::postEvent(this,e1);
      
      if(captureimg == 1)
      {
                 QString nameFile="image_000.jpg";
                 QFile test;
                 QString number;
                 int j=1000;
                 for(int i=1 ; i<j ; i++)
                 {    
                                  number.setNum(i);
                                  if(i < 10)
                                       nameFile = "image_00" + number + ".jpg"; 
                                  else if(i < 100)
                                       nameFile = "image_0" + number + ".jpg"; 
                                  else if(i < 1000)
                                       nameFile = "image_" + number + ".jpg";                                        
                                  test.setFileName(nameFile);
                                  if( !test.exists() )
                                      break;
                 }

                 imageQT5.save(nameFile,"JPG",100);
                 captureimg=0;           
      }


      traitementEnCours=0;
  }

  return URBI_CONTINUE;
    
}



///////////////////////////////////////////
// se declenche quand des evenements sont recus
bool ConnectAibo::event(QEvent *ev)
{

 if( ev->type() == (QEvent::Type)(1001))
 {
      pere->ui.label_2->setPixmap(QPixmap::fromImage(imageQT4));     // on affiche l'image normale
      pere->ui.label_5->setPixmap(QPixmap::fromImage(imageQT2));     // on affiche les points d'interet sur un fond noir
      pere->ui.label_6->setPixmap(QPixmap::fromImage(imageQT5));     // on affiche l'image normale + les points d'interet

 }

 return 1;
}

