/*! \file mainform.hh
 *******************************************************************************

 File: mainform.hh\n
 Definition of the MainForm class.

 This file is part of 
 %URBI Lab, version __urbilabversion__\n
 (c) Jean-Christophe Baillie, 2004-2005.

 Permission to use, copy, modify, and redistribute this software for
 non-commercial use is hereby granted.

 This software is provided "as is" without warranty of any kind,
 either expressed or implied, including but not limited to the
 implied warranties of fitness for a particular purpose.

 For more information, comments, bug reports: http://urbi.sourceforge.net

 **************************************************************************** */

#ifndef MAINFORM_H
#define MAINFORM_H

#include <qvariant.h>
#include <QMainWindow>
//Added by qt3to4:
#include <QHBoxLayout>


class QHBoxLayout;
class QAction;
class QMenu;
class QSplitter;
class QWidget;
class QSettings;

class UMsgWindow;
class UDebugWindow;
class UHostNameForm;
class URobotMng;

extern const QString   APP_KEY;  ///< application key
extern       QSettings settings; ///< contains the user settings

//! Mainform class is the main window of URBI Lab
/*! Mainform is the main window of URBI Lab from which you have access
    to all the functions of URBI Lab.

    There's menu, toolbar, message and debug clients organized in separate
    tab panels.

*/
class MainForm : public QMainWindow {

  Q_OBJECT

 public:
  
  MainForm( QWidget* parent = 0);   
                                           
  ~MainForm();

  
  UMsgWindow*    msgWindow;   ///< window containing tabs for all message connections
  UDebugWindow*  debugWindow; ///< window containing tabs for all debug connections
  
  QSplitter*     mainFormSplitter;  ///< verticalwise space manager

  /* menus */

  QMenu*    fileMenu;          ///< file submenu
  QMenu*    viewMenu;          ///< view submenu 
  QMenu*    helpMenu;          ///< help submenu


  /* menu/toolbar actions */
  QAction*       fileNewMsgAction;
  QAction*       fileNewDebugAction;
  QAction*       fileOpenAction;
  QAction*       fileSaveAction;
  QAction*       fileSaveAsAction;
  QAction*       fileCloseMsgAction;
  QAction*       fileCloseDebugAction;
  QAction*       fileExitAction;
 

  QAction*       helpAboutAction;
  QAction*       viewToggleDebugVisibilityAction;

  UHostNameForm* hostNameForm;  ///< dialog box that ask for robot name
  
  URobotMng*     robotMng;      ///< robot manager
  
 public slots:
  
  virtual void fileNewMsg                ();
  virtual void fileNewDebug              ();
  virtual void fileCloseMsg              ();
  virtual void fileCloseDebug            ();
  virtual void fileExit                  ();
  
  virtual void viewToggleDebugVisibility ();
  
  virtual void helpAbout                 ();

 protected:
  
  QHBoxLayout* MainFormLayout;   ///< main window layout manager

 
 private:

  void loadSettings (); 
  void saveSettings (); 

  /* user settings */
  int  initX;        ///< initial x-coordinate of the main window
  int  initY;        ///< initial y-coordinate of the main window
  int  initHeight;   ///< initial height of the main window
  int  initWidth;    ///< initial width of the main window

};

#endif // #ifndef MAINFORM_H
