/*! file uconn.cpp
*******************************************************************************

File: uconn.cppn
Implementation of the UConn class.

This file is part of 
%URBI Lab, version __urbilabversion__n
(c) Jean-Christophe Baillie, 2004-2005.

Permission to use, copy, modify, and redistribute this software for
non-commercial use is hereby granted.

This software is provided "as is" without warranty of any kind,
either expressed or implied, including but not limited to the
implied warranties of fitness for a particular purpose.

For more information, comments, bug reports: http://urbi.sourceforge.net

**************************************************************************** */

#include <string.h>

#include <qapplication.h>
#include <qevent.h>

#include <uclient.h> //liburbi
#include "ucustomevent.hh"
#include "uconn.hh"
#include "umsgtab.hh"


   //! UConn constructor.
   /*!
     param client is the UClient object
     param tab is the UMsgTab object that uses this connection
     param type is type of the connection
     param id is id of the connection
   */
   UConn::UConn( UClient* client, 
		 UMsgTab* tab, 
		 UConnType type, 
		 char* id ) 
     : client(client), 
       id(id), 
       tab(tab), 
       type(type),  
       graphedVariablesCnt(0) { 
     };



//! UConn destructor.
UConn::~UConn() {
  qDebug("UConn::~UConn: deleting msg client...");
  delete client;
}



//! Sets the connection id.
/*! Callback function called by UClient callback system
  when message with connection id arrives.

  param msg is the message with connection id
*/
UCallbackAction 
UConn::setConnId( const UMessage& msg) {
  qDebug("UConn::setConnId: setting up client connection id (%s)...", msg.stringValue);

  id = new char[10];                  // id conn length
  strcpy( id,  msg.stringValue + 4 ); // we dont take first 4 char (the line message contains "ID: ")

  return URBI_REMOVE;  // once we receive the connection id we unregister the function
}



//! Forward the UMessage message to the UMsgTab object.
/*! Callback function called by UClient callback system
  when any message arrives.

  Forwarding has to be done by posting event,
  because this gets executed in a non-GUI thread and function
  receiveMsg() has to be executed in the GUI thread.

  param msg is the message to be received
*/
UCallbackAction 
UConn::receive( const UMessage& msg ) {

  // constructs event object and makes a copy of the message
  UCustomEvent* e = new UCustomEvent( (QEvent::Type) UCustomEvent::EVENT_RECEIVE_MSG );
  e->msg = new UMessage( msg );  //deep copy

  switch(type) {
  case MsgConn:
    qApp->postEvent( tab, e );
    break;

    // this should never be used, messages containing graphing (double, sound, video) 
    // information are sent directly to UGraph object
  case GraphConn:
  case AVConn:
  default:
    Q_ASSERT( true );
    break;
  }

  return URBI_CONTINUE;
}


