/*! \file uconn.hh
 *******************************************************************************

 File: uconn.hh\n
 Definition of the UConn class.

 This file is part of 
 %URBI Lab, version __urbilabversion__\n
 (c) Jean-Christophe Baillie, 2004-2005.

 Permission to use, copy, modify, and redistribute this software for
 non-commercial use is hereby granted.

 This software is provided "as is" without warranty of any kind,
 either expressed or implied, including but not limited to the
 implied warranties of fitness for a particular purpose.

 For more information, comments, bug reports: http://urbi.sourceforge.net

 **************************************************************************** */
#ifndef UCONN_H
#define UCONN_H


class UClient; //liburbi

class UMsgTab;

//! UConn class encapsules UClient connection and other corresponding parametres. 
/*! It contains UClient object, connection id, type of connection.
    In case of type is MsgConn it holds a pointer to a UMsgTab object
    that uses this connection.

    All variables are public for simplified access.
*/
class UConn { 

 public:
  
  enum UConnType {
    MsgConn=0,
    GraphConn,
    AVConn
  };  ///< type of connection

  UConn( UClient* client, 
	 UMsgTab* tab, 
	 UConnType type = MsgConn, 
	 char* id = 0 ); 
  //: client(_client), id(_id), tab(_tab), type(_type),  graphedVariablesCnt(0) { /*empty*/ };
  ~UConn();

  UCallbackAction setConnId( const UMessage& msg ); 
  UCallbackAction receive  ( const UMessage& msg ); 


  UClient*  client;    ///< the connection client
  char*     id;        ///< id of the connection
  UMsgTab*  tab;       ///< UMsgTab object which uses this connection
  UConnType type;      ///< type of the connection
  int       graphedVariablesCnt; ///< (only for GraphConn) number of simultaneously graphed variables by local UClient,
                                 ///< serves as limitation
  
};


#endif //#ifndef UCONN_H
