/*
 *   ucustomevent.hh 
 *
 *   Implementation of UCustomEvent
 *   desc: implements custom event class that allows to send user defined events
 *         used for messaged posting, video rendering, graphing, etc. 
 */

/*! \file  ucustomevent.hh
 *******************************************************************************

 File:  ucustomevent.hh\n
 Definition of the UCustomEvent class.

 This file is part of 
 %URBI Lab, version __urbilabversion__\n
 (c) Jean-Christophe Baillie, 2004-2005.

 Permission to use, copy, modify, and redistribute this software for
 non-commercial use is hereby granted.

 This software is provided "as is" without warranty of any kind,
 either expressed or implied, including but not limited to the
 implied warranties of fitness for a particular purpose.

 For more information, comments, bug reports: http://urbi.sourceforge.net

 **************************************************************************** */
#ifndef UCUSTOMEVENT_H
#define UCUSTOMEVENT_H

#include <qevent.h>
//Added by qt3to4:


#include <uclient.h> //liburbi

//! UCustomEvent class implements a custom event
/*! It's used for passing UMessage messages from non-GUI to GUI threads.


*/
class UCustomEvent : public QEvent {
 
 public:

  enum UCustomEventType {
    EVENT_RECEIVE_MSG = QEvent::User, // constant range dedicated for user event types begins at QEvent::User, so we use it
    EVENT_PLOT_DOUBLE,
    EVENT_PLOT_IMAGE,
    EVENT_PLOT_SOUND,
  }; ///< custom event type
  
  UMessage *msg; ///< event can contain a message


  UCustomEvent( QEvent::Type type ) 
    :QEvent(  type ) { msg = 0;};

  //UCustomEvent( QEvent::Type type, void *data ) 
  //:QCustomEvent( type, data ) { msg = 0;};

  ~UCustomEvent(){ if(msg) delete msg; }; // deletes msg and the buffers allocated within the msg object as well


};

#endif // #ifndef UMSGTAB_H
