/*! file udebugtab.cpp
*******************************************************************************

File: udebugtab.cppn
Implementation of the UDebugTab class.

This file is part of 
%URBI Lab, version __urbilabversion__n
(c) Jean-Christophe Baillie, 2004-2005.

Permission to use, copy, modify, and redistribute this software for
non-commercial use is hereby granted.

This software is provided "as is" without warranty of any kind,
either expressed or implied, including but not limited to the
implied warranties of fitness for a particular purpose.

For more information, comments, bug reports: http://urbi.sourceforge.net

**************************************************************************** */

#include <qlayout.h>
#include <qtextedit.h>
#include <qstring.h>
#include <qtabwidget.h>
   //Added by qt3to4:
#include <QVBoxLayout>

#include "udebugtab.hh"

   //! UDebugTab constructor.
   /*! Sets up the textfield.

   param parent is this widget's parent in Qt hierarchy
   param name within Qt
   */
   UDebugTab::UDebugTab( QWidget* parent)
     : QWidget( parent ) {

       debugTabLayout = new QVBoxLayout( this);

       debugTextEdit = new QTextEdit( this);
       //debugTextEdit->setTextFormat( Qt::PlainText );
       debugTextEdit->setFontFamily( "Courier" ); //TODO: let the user choose
       debugTextEdit->setFontPointSize( 10 ); //TODO: let the user choose 
       debugTextEdit->setReadOnly( TRUE );

       debugTabLayout->addWidget( debugTextEdit );
     }


//! UDebugTab destructor.
/*! Destroys the tab and makes the respective URobot object close
  the client used by this tab.

*/
UDebugTab::~UDebugTab() {
  qDebug("UDebugTab::~UDebugTab: destroying tab...");
  robot->closeDebugClient();
}


//! Sets pointer to the associated URobot object.
/*!
  param robot is a pointer to URobot object
*/
void
UDebugTab:: setRobot( URobot* robot ) { 
  this->robot = robot; 
};


//! Appends the debug message to the text field.
/*!
  param msg is the debug message to display
*/
void 
UDebugTab::append( QString& msg ) { 
  debugTextEdit->append( msg ); 
}


void 
UDebugTab::clientConnected( ) {
  QString tmp = "Connected to ";
  tmp += robot->getHostName();
  append( tmp );
}

//! Tells the user that an error occured when connecting to the robot.
void 
UDebugTab::clientError( URobot::DebugClientErrType type ) {
  //(URobot::DebugClientErrType) type;
  // TODO:popup dialog for reconnecting

  //((QTabWidget*)parentWidget())->removePage( this );
  //delete this;
}

//! Tells the user that the connection has been closed by the other side.
void 
UDebugTab::clientClosed( ) {
  // TODO:popup dialog for reconnecting

  //((QTabWidget*)parentWidget())->removePage( this );
  //delete this;
}
