
#include <QGridLayout>
#include <QTimer>
#include <QToolBar>
#include <uclient.h> //liburbi

#include <qwt_scale_div.h>

#include "udoublegraphwindow.hh"
#include "ugraph.hh"
#include "ucustomevent.hh"
#include "urobot.hh"
 
UDoubleGraphWindow::UDoubleGraphWindow( QWidget* parent)
  : UGraphWindow( parent ) {

    limit = 30;
    stoped = false;
   
    QToolBar *tool = new QToolBar(this);
    QWidget * oWidget = new QWidget(tool);
    QHBoxLayout * oLayout = new QHBoxLayout(oWidget);
    
    
    oWidget->setLayout(oLayout);
    stopButton = new QPushButton( "&Stop");
    oLayout->addWidget(stopButton);
    clearButton = new QPushButton("&Clear");
    oLayout->addWidget(clearButton);
    limitBox = new QSpinBox();
    oLayout->addWidget(limitBox);
    limitBox->setPrefix("Limit to  ");
    limitBox->setSuffix("  seconds");
    limitBox->setMaximum(3600000);
    limitBox->setMinimum(0);
    limitBox->setValue(limit);
    tool->addWidget(oWidget);
    plot = new QwtPlot();
    setCentralWidget(plot);
    addToolBar(tool);
    //label->setPixmap( (const QPixmap) image );

    resize( QSize(640, 480).expandedTo(minimumSizeHint()) );
    // clearWState( WState_Polished );
    connect( stopButton, SIGNAL(clicked()), this, SLOT(stopStart()));
    connect( clearButton, SIGNAL(clicked()), this, SLOT(clear()));
    connect( limitBox, SIGNAL(valueChanged(int)), this, SLOT(limitChanged(int)));
  }

void UDoubleGraphWindow::initialize(URobot * robot, QString & variable, const QString & uniquetag) {
  this->robot = robot;
  this->variable = variable;
  tag = uniquetag;
  setWindowTitle( "Graph for "+variable+" on "+robot->getHostName() ); 
  plot->setTitle(variable+" on "+robot->getHostName());
  //setCentralWidget(plot);
  curve = new QwtPlotCurve("Graph");
  curve->attach(plot);
  //plot->setAxisScale(QwtPlot::xBottom, 0, 1000);
  //plot->setAxisScale(QwtPlot::yLeft, 0, 1000);
  plot->replot();

  plot->canvas()->setPaintAttribute(QwtPlotCanvas::PaintPacked, true);
}


UDoubleGraphWindow::~UDoubleGraphWindow() {
 delete plot;   
 delete curve;                                      
}

bool 
UDoubleGraphWindow::updateGraph( const UMessage& msg ) {
  while ((!x.empty()) && x.front() < msg.timestamp - limit*1000) {
    x.pop_front();
    y.pop_front();
  }
  x.push_back(msg.timestamp);
  y.push_back(msg.doubleValue);
  curve->setData( x, y);
  plot->replot();
  return true;
}

/*
 *  Sets the strings of the subwidgets using the current
 *  language.
 */
void UDoubleGraphWindow::languageChange()
{
}


void UDoubleGraphWindow::limitChanged(int nLimit) {
  limit = nLimit;
}

void UDoubleGraphWindow::clear() {
  x.clear();
  y.clear();
  //not a slot, doesnt work QTimer::singleShot(100, plot, SLOT( repaint()));
}

void UDoubleGraphWindow::stopStart() {                   
  stoped = !stoped;
  QString cmd = stoped?"freeze ":"unfreeze ";
  cmd.append(tag+",");
  robot->getGraphClient()->send(cmd.toAscii().constData());
  stopButton->setText(stoped?"&Start":"&Stop");
  if (stoped) {
    //switch to fixed scale mode
    const QwtScaleDiv * scale = plot->axisScaleDiv(QwtPlot::xBottom);
    qDebug("axis  scale 1 %f %f", scale->lBound(), scale->hBound());
    double l = scale->lBound();
    double r = scale->hBound();
    plot->setAxisScale(QwtPlot::xBottom, l, r);

    scale = plot->axisScaleDiv(QwtPlot::yLeft);
    l = scale->lBound();
    r = scale->hBound();
    qDebug("axis  scale 2 %lf %lf", scale->lBound(), scale->hBound());
    plot->setAxisScale(QwtPlot::yLeft, l, r);
    plot->replot();
#ifdef WIN32
#warning "disabling ScrollZoomer om win32 platform" 
    zoomer = new QwtPlotZoomer(plot->canvas());
#else
    zoomer = new ScrollZoomer(plot->canvas());
#endif

    zoomer->setRubberBandPen(QPen(Qt::black, 0, Qt::DotLine));
    zoomer->setTrackerPen(QPen(Qt::black));

  }
  else {
    //resume autoscaling
    plot->setAxisAutoScale(QwtPlot::xBottom);
    plot->setAxisAutoScale(QwtPlot::yLeft);
    delete zoomer; 
  }
}

