/*
 *   ugraph.hh 
 *
 *   Implementation of UGraph
 *   desc: class that provides graphing and stores all current graphing processes
 *         by graphing we mean presenting to the user double or video or sound variables
 */

#ifndef UGRAPH_H
#define UGRAPH_H

#include <qobject.h>
#include <QHash>

//Added by qt3to4:
#include <QEvent>

#include <uclient.h> //liburbi

class QEvent;

class UGraphWindow;
class URobot;

class UGraph : public QObject {

  Q_OBJECT

 public:
  UGraph();
  ~UGraph();

  bool event ( QEvent* e );

  void registerGraphProcess( URobot* robot, QString& uniqueTag, QString& var, 
			     UMessageType msgType, UBinaryMessageType binType );
  
  UCallbackAction plotEventDouble( const UMessage& msg );
  UCallbackAction plotEventImage( const UMessage& msg );
  UCallbackAction plotEventSound( const UMessage& msg );
  
  void plotDouble( UMessage& msg );
  void plotImage( UMessage& msg );
  void plotSound( UMessage& msg );

  void stopGraph( QString uniqueTag ); 
  void stopGraph( UGraphWindow* window );
 
 private:
  // for internal use, wraps graph procces description data
  class UGraphProcess{
  public:
    UGraphProcess( URobot* r, QString& v, UMessageType mt, UBinaryMessageType bt, UGraphWindow* w = NULL)
      : robot(r), var(v), msgType(mt), binType(bt), window(w) { };
    URobot* robot;
    QString var;
    UMessageType msgType;
    UBinaryMessageType binType;
    UGraphWindow* window;
  };

  QHash<QString, UGraphProcess*> graphProcessList;
};

#endif // #ifndef UGRAPH_H
