/*
 *   ugraphwindows.cpp 
 *
 *   Implementation of UGraph
 *   desc: class that implements window for presenting the values (double, image, sound)
 *         
 */
#include <qwidget.h>
#include <qevent.h>
#include <qlabel.h>
#include <qpixmap.h>
#include <qlayout.h>
//Added by qt3to4:
#include <QGridLayout>

#include <uclient.h> //liburbi

#include "ugraphwindow.hh"
#include "ugraph.hh"
#include "ucustomevent.hh"
#include "urobot.hh"

UGraphWindow::UGraphWindow( QWidget* parent )
  : QMainWindow( parent), buffer32(0) {


  setCentralWidget( new QWidget( this) );
  layout =  new QGridLayout( centralWidget()); 
 
  label = new QLabel( centralWidget());
  image = new QPixmap();
  //label->setPixmap( (const QPixmap) image );
  
 
  languageChange();
  resize( QSize(200, 200).expandedTo(minimumSizeHint()) );
  // clearWState( WState_Polished );
}

void UGraphWindow::initialize(URobot * robot, QString & var, const QString & uniquetag) {
  layout->addWidget( label, 1, 1 );
  spacer1 = new QSpacerItem( 0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum );
  layout->addItem( spacer1, 1, 0 );
  spacer2 = new QSpacerItem( 0, 0, QSizePolicy::Minimum, QSizePolicy::Expanding );
  layout->addItem( spacer2, 0, 1 );
  spacer3 = new QSpacerItem( 0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum );
  layout->addItem( spacer3, 1, 2 );
  spacer4 = new QSpacerItem( 0, 0, QSizePolicy::Minimum, QSizePolicy::Expanding );
  layout->addItem( spacer4, 2, 1 );
  setWindowTitle("graph "+var+" for "+robot->getHostName());

}


UGraphWindow::~UGraphWindow() {
  
}

bool 
UGraphWindow::event( QEvent* e ) {

  if(e->type() == QEvent::Hide) {
    graph.stopGraph( this );
    }
  // handling is passed along, deletion of QEvent object inside
  return QMainWindow::event( e );
}

bool 
UGraphWindow::updateGraph( const UMessage& msg ) {
  uchar* buffer24 = new uchar[msg.image.width * msg.image.height*3];
  int bsz = msg.image.width * msg.image.height*3;
  if( msg.image.imageFormat == IMAGE_JPEG )
    convertJPEGtoRGB( (const byte*)msg.image.data, msg.image.size, (byte*)buffer24, bsz);
  else
    convertYCrCbtoRGB( (const byte*)msg.image.data, msg.image.size, (byte*)buffer24 );
  
  // converts 24-bit BGR to 32-bit RGB
  if (!buffer32)
    buffer32 = new uchar[bsz + bsz / 3]();
  
  for( int i = 0; i < bsz / 3; i++){
    buffer32[4 * i]     = buffer24[3 * i + 2]; // swap R and B
    buffer32[4 * i + 1] = buffer24[3 * i + 1];
    buffer32[4 * i + 2] = buffer24[3 * i + 0]; // swap R and B
    buffer32[4 * i + 3] = 0xff;                // pad by 0s
  }
  delete [] buffer24;
  // constructs image and fills with data 
  QImage image( buffer32, msg.image.width, msg.image.height, QImage::Format_RGB32);
 
  //   else if( msg.image.imageFormat == IMAGE_YCbCr) { // YCbCr             
//     uchar* buffer24 = new uchar[msg.image.size]();
//     if( !convertYCrCbtoRGB( (const byte*)msg.image.data, msg.image.size, (byte*)buffer24 ) ) {
//       qWarning("UGraphWindow::updateGraph: error converting image from YCrCb to RGB...");
//       return false;
//     }
	
//     // convert 24-bit BGR to 32-bit RGB
//     uchar* buffer32 = new uchar[msg.image.size + msg.image.size / 3](); // buffer for 32-bit RGB
//     for( int i = 0; i < msg.image.size / 3; i++){
//       buffer32[4 * i] = buffer24[3 * i + 2];     // swap R and B
//       buffer32[4 * i + 1] = buffer24[3 * i + 1];
//       buffer32[4 * i + 2] = buffer24[3 * i + 0]; // swap R and B
//       buffer32[4 * i + 3] = 0xff;                // pad by 0s
//     }
//     delete image;
//     image = new QImage( buffer32, msg.image.width, msg.image.height, 32, 0, 0, QImage::IgnoreEndian);
  	
//     delete [] buffer24;
	
//   }

  label->setPixmap( QPixmap::fromImage(image) );

  return true;
}

/*
 *  Sets the strings of the subwidgets using the current
 *  language.
 */
void UGraphWindow::languageChange()
{
   
}
