/*! \file  umsgoutpane.hh
 *******************************************************************************

 File: umsgoutpane.hh\n
 Definition of the UMsgOutPane class.

 This file is part of 
 %URBI Lab, version __urbilabversion__\n
 (c) Jean-Christophe Baillie, 2004-2005.

 Permission to use, copy, modify, and redistribute this software for
 non-commercial use is hereby granted.

 This software is provided "as is" without warranty of any kind,
 either expressed or implied, including but not limited to the
 implied warranties of fitness for a particular purpose.

 For more information, comments, bug reports: http://urbi.sourceforge.net

 **************************************************************************** */

#ifndef UMSGOUTPANE_H
#define UMSGOUTPANE_H


#include <QWidget>
class QTextEdit;

class QString;

class UCmdLine;
class UMsgTab;
class UGraph;

extern UGraph graph; //TODO:remove

//! UMsgOutPane class contains command line and text field. 
/*! It consists of a command line, where messages composed
    and a text field, where send messages are echoed.

    It provides the forwarding of a command from the command line to the robot.

*/
class UMsgOutPane : public QWidget {
  
  Q_OBJECT
  
 public:

  UMsgOutPane( QWidget* parent = NULL);
  ~UMsgOutPane();
  
  void setMsgTab( UMsgTab* tab );
  UCmdLine * getCmdLine() {return cmdLine;}
 public slots:

  void send( const QString& );
  void send(); 

 private:

  void loadSettings();
  void saveSettings();

  UCmdLine*  cmdLine;        ///< command line
  QTextEdit* msgOutTextEdit; ///< text field for echoing commands
  UMsgTab*   msgTab;         ///< message tab containing this pane
  int startIndex;            ///< number of lines loaded from history

};


#endif // #ifndef UMSGOUTPANE_H
