/*! file umsgwindow.cpp
*******************************************************************************

File: umsgwindow.cppn
Implementation of the UMsgWindow class.

This file is part of 
%URBI Lab, version __urbilabversion__n
(c) Jean-Christophe Baillie, 2004-2005.

Permission to use, copy, modify, and redistribute this software for
non-commercial use is hereby granted.

This software is provided "as is" without warranty of any kind,
either expressed or implied, including but not limited to the
implied warranties of fitness for a particular purpose.

For more information, comments, bug reports: http://urbi.sourceforge.net

**************************************************************************** */

#include <qstring.h>
#include <qwidget.h>

#include "umsgwindow.hh"
#include "umsgtab.hh"
#include "urobot.hh"

   //! UDebugWindow constructor.
   /*! 
     Construct the widget that will hold msg tabs and hides it.

     param parent is this windows's parent in Qt hierarchy
     param name within Qt
   */ 
   UMsgWindow::UMsgWindow( QWidget* parent)
     : QTabWidget( parent) {

       // there are no msg tabs yet, so we hide the window
       if( count() == 0 )
	 hide();
     }


//! UMsgWindow destructor.
UMsgWindow::~UMsgWindow() {
  // no need to delete anything, Qt does it for us
}


//! Adds a new tab to the window.
/*! 
  param robot is the URobot object that the tab will be using 
  when communicating with the robot 

*/
void
UMsgWindow::addNewTab( URobot* robot ) {

  // we get the robot name
  const char *hostName = robot->getHostName( );

  // we store robots in a list and unique suffices for the tabs names
  if( msgTabIds.find( hostName ) == msgTabIds.end( ) ) // there's no tab existing for the robot yet
    msgTabIds.insert( hostName, 0 );                   // so we put it in the list of tabs for the robot


  //label of the tab consist of the robot name and order number of the connection to the robot 
  QString tabName( hostName );
  tabName += ":" + QString::number( ++msgTabIds[hostName] ); 

  // creates new tab
  UMsgTab* tab = new UMsgTab( this);

  // interconnects robot and tab
  tab->setRobot( robot );
  robot->setupMsgClient( tab );
 
  // adds the tab to the window
  addTab( tab, tabName );

  // brings the tab to the front
  setCurrentIndex( indexOf( tab ));

}


//! Removes current tab from the window and destroys it.
void
UMsgWindow::closeCurrentTab( ) {
  if( count() == 0 )
    return;

  QWidget* tab = currentWidget();
  int idx = currentIndex();
  qDebug( "UMsgWindow::closeCurrentTab: removing and deleting current tab..." );

  removeTab( idx );
  delete tab;


}
