/*! file urobotmng.cpp
*******************************************************************************

File: urobotmng.cppn
Implementation of the URobotMng class.

This file is part of 
%URBI Lab, version __urbilabversion__n
(c) Jean-Christophe Baillie, 2004-2005.

Permission to use, copy, modify, and redistribute this software for
non-commercial use is hereby granted.

This software is provided "as is" without warranty of any kind,
either expressed or implied, including but not limited to the
implied warranties of fitness for a particular purpose.

For more information, comments, bug reports: http://urbi.sourceforge.net

**************************************************************************** */

#include <string.h>

#include "urobotmng.hh"
#include "urobot.hh"
   //Added by qt3to4:
#include <QLinkedList>

   //! URobotMng constructor.
   URobotMng::URobotMng( ) {
     qDebug( "URobotMng::URobotMng: robot mng created..." );

   }

//! URobotMng destructor.
URobotMng::~URobotMng( ) {
  qDebug( "URobotMng::URobotMng: robot mng destroyed..." );
}


//! Returns URobot object for given hostname.
/*!
  If robot with the hostname isn't in the list yet, it creates it and puts
  it on the list.
*/
URobot*
URobotMng::getRobot( QString hostName ) {
   for ( QLinkedList<URobot *>::iterator it = robotList.begin();
       it != robotList.end(); it++) {
    if( (*it)->getHostName()== hostName )  {
      qDebug( "URobotMng::robot: robot %s found...returning...", hostName.toAscii().constData() );
      return (*it);
    }
  }

  // we havent found a robot of supplied hostName, so we create it and put it on the list
  qDebug( "URobotMng::robot: robot %s NOT found...creating robot...", hostName.toAscii().constData() );
  URobot * robot = new URobot( hostName.toAscii() );
  robotList.append( robot );

  return robot;
}


// //! Returns true if URobot object with hostname exists.
// bool
// URobotMng::robotExists( const char *hostName ) const {
//   bool found = false;
//   URobot* robot;

//   QPtrListIterator<URobot> it( *robotList ); 
//   while( ( robot =  it.current() ) != 0 ) {
//     ++it;
//     if( strcmp( robot->getHostName(), hostName ) == 0 )
//       found = true;
//   }

//   return found;
// }

// //! Deletes URobot object with given hostname.
// void
// URobotMng::deleteRobot( const char *hostName ) {
//   URobot* robot;

//   QPtrListIterator<URobot> it( *robotList ); 
//   while( ( robot =  it.current() ) != 0 ) {
//     ++it;
//     if( strcmp( robot->getHostName(), hostName ) == 0 )
//       delete robot;
//   }
// }

