#include "data_plot.h"

//////////////////////////////////////////////////
// On initialise la fenetre principale
DataPlot::DataPlot(QWidget *parent): QwtPlot(parent)
{
    PLOT_SIZE = 201;   

    // Disable polygon clipping
    QwtPainter::setDeviceClipping(false);

    // We don't need the cache here
    canvas()->setPaintAttribute(QwtPlotCanvas::PaintCached, false);
    canvas()->setPaintAttribute(QwtPlotCanvas::PaintPacked, false);

#ifdef Q_WS_X11
    canvas()->setAttribute(Qt::WA_PaintOnScreen, true);
#endif

    // Assign a title
    setTitle("URBI Graph Variable");
    insertLegend(new QwtLegend(), QwtPlot::BottomLegend);

    alignScales();

    d_x = NULL;
    d_z = NULL;


#if 0
    //  Insert zero line at y = 0
    QwtPlotMarker *mY = new QwtPlotMarker();
    mY->setLabelAlignment(Qt::AlignRight|Qt::AlignTop);
    mY->setLineStyle(QwtPlotMarker::HLine);
    mY->setYValue(0.0);
    mY->attach(this);
#endif

    // Axis 
    setAxisTitle(QwtPlot::xBottom, "Time/seconds");
    setAxisScale(QwtPlot::xBottom, -2, 2);

    setAxisTitle(QwtPlot::yLeft, "Values");
    setAxisScale(QwtPlot::yLeft, -2, 2);
    


}



//////////////////////
//  Mise en forme de la fenetre avec bon repositionnement des elements
void DataPlot::alignScales()
{
    // The code below shows how to align the scales to
    // the canvas frame, but is also a good example demonstrating
    // why the spreaded API needs polishing.

    canvas()->setFrameStyle(QFrame::Box | QFrame::Plain );
    canvas()->setLineWidth(1);

    for ( int i = 0; i < QwtPlot::axisCnt; i++ )
    {
        QwtScaleWidget *scaleWidget = (QwtScaleWidget *)axisWidget(i);
        if ( scaleWidget )
            scaleWidget->setMargin(0);

        QwtScaleDraw *scaleDraw = (QwtScaleDraw *)axisScaleDraw(i);
        if ( scaleDraw )
            scaleDraw->enableComponent(QwtAbstractScaleDraw::Backbone, false);
    }
}


//////////////////////////////////////////////////
// On ajoute des valeurs a la liste
void DataPlot::addValuesVariable(double value, double heure)
{

    if(d_x == NULL)
    {
        d_x = new double[PLOT_SIZE];  
        d_z = new double[PLOT_SIZE];  

        //  Initialize data
        for (int i = 0; i< PLOT_SIZE; i++)
        {
           d_x[i] = heure;     // time axis
           d_z[i] = 0;
        }

        // Insert new curves
        cLeft = new QwtPlotCurve("Data Global Variable");
        cLeft->attach(this);

        // Set curve styles
        cLeft->setPen(QPen(Qt::blue));

        // Attach (don't copy) data. Both curves use the same x array.
        cLeft->setRawData(d_x, d_z, PLOT_SIZE);

        valmin=value;
        valmax=value;
    }

       // on fait un dcalage a gauche des donnees
      int nbOctets = (PLOT_SIZE-1)*sizeof(double);
      memmove ( d_z, d_z+1, nbOctets ); 
      memmove ( d_x, d_x+1, nbOctets ); 

      d_z[PLOT_SIZE - 1] = value;
      d_x[PLOT_SIZE - 1] = heure;

    setAxisScale  (QwtPlot::xBottom, (int)( d_x[0] - 1) , (int)( d_x[PLOT_SIZE - 1] + 1) ); // on indique le min et le max de l'axe des temps

    if(valmax < value)
    {
              valmax = value;
              setAxisScale(QwtPlot::yLeft, (int)valmin-1, (int)valmax+1);
    }
    else if(valmin > value)
    {
              valmin = value;
              setAxisScale(QwtPlot::yLeft, (int)valmin-1, (int)valmax+1);
    }


    // update the display
    replot();
}



void DataPlot::videPlotVariable()
{
    // On libere la memoire des tableaux
    if (d_x != NULL)
    {
            delete cLeft;
            delete d_x; 
            delete d_z;
            d_x = NULL;
            d_z = NULL;
            cLeft = NULL;
      }
}

void DataPlot::resize()
{
    // On cherche les min et max
    if (d_z != NULL)
    {
            double min = d_z[0];
            double max = d_z[0];
            for (int i = 0; i< PLOT_SIZE; i++)
            {
               if(max < d_z[i])
                 max = d_z[i];
               else if(min > d_z[i])
                 min = d_z[i];
            }

              valmin = min;
              valmax = max;
              setAxisScale(QwtPlot::yLeft, (int)valmin-1, (int)valmax+1);

              // update the display
              replot();

      }
}
