function urbiMessage = urbiFullParse(string, timeStamp, con)
% Perform the full parsing of a message to be passed to a callback function
%  This function returns a struct containing all the informations
%  contained in the message :
%
%   urbiMessage.timeStamp  : the time stamp of the message
%   urbiMessage.tag        : the tag of the message
%   urbiMessage.type       : the type of the message: 'numeric', 'string',
%   'system', 'image', 'sound', or 'otherBIN')
%
%   if type == numeric, then urbiMessage.value (the numeric value of the message)
%   if type == string,  then urbiMessage.value (the string value of the message)
%   if type == system,  then urbiMessage.value (the string value of the system message)
%   if type == image,   then urbiMessage.image is the same as the result of urbiGetImage(con)
%     (with the fields matrix, width, height, timeStamp)
%   if type == sound,   then urbiMessage.sound is the same as the result of urbiGetSound(con, length)
%     (with the fields samples, Fs, nbits, length)
%   if type == otherBIN then urbiMessage.bin (the vector of bytes of the BIN info)


io_buffer = 'imageB.jpg';
 
f = sscanf(string{2}, '%f') ;
if ~isempty(f)
    type =  'numeric';
    value = f;
    image = [];
    sound = [];
    bin   = [];
else
    s = string{2};
    if strcmp(s, 'BIN')
        if strcmp(string{4}, 'jpeg')
            type = 'image';
            value = [];
            pnet(con, 'readtofile', io_buffer, sscanf(string{3}, '%u')) ;
            data = imread(io_buffer) ;
            image = struct('matrix',data,'width',sscanf(string{5}, '%u'),'height', sscanf(string{6}, '%u'), 'timeStamp', timeStamp);
            sound = [];
            bin   = [];
        elseif strcmp(string{4}, 'raw')
            type = 'sound';
            value = [];
            image = [];
            % Parse the sound and store the structure
            Fs      = str2num ( string{6} );
            length   = str2num ( string{3} );
            nbits    = str2num ( string{7} );
            channel = str2num ( string{5} );
            duration = length/(Fs*channel*(nbits/8))*1000;
            samples = pnet(con, 'read', length, 'noblock');
            sound = struct('samples',samples,'Fs',Fs,'nbits', nbits, 'length', length, 'channel', channel, 'duration',duration);
            bin   = [];
        else
            type = 'otherBIN';
            value = [];
            image = [];
            sound = [];
            bin   = pnet(con, 'read', str2num ( string{3} ), 'noblock');
        end;
   else
       if strcmp(s, '***')
           type = 'system';
           value = sprintf('%s ',string{3:end});
       elseif strcmp(s, '!!!')
           type = 'error';
           value = sprintf('%s ',string{3:end});         
       else
           type = 'string';
           value = sprintf('%s ',string{2:end});
       end;
       image = [];
       sound = [];
       bin   = [];
    end;
end;

urbiMessage = struct('timeStamp', timeStamp, 'type', type, 'value', value, 'image', image, 'sound', sound, 'bin', bin);
