function [data,timeStamp, tag, type] = urbiGet(con)
% Read the next message comming from the server and outputs the data
% This function reads any message coming from the server, parses it, and
% returns a structure 'data' containing the data. The data are the same as
% in a urbiMessage, but in a different order to suit matlab conventions
% about missing output arguments
%
%   timeStamp  : the time stamp of the message
%   tag        : the tag of the message
%   type       : the type of the message: 'numeric', 'string',
%   'system', 'error', 'image', 'sound', or 'otherBIN')
%
%   if type == numeric, then data is the numeric value of the message
%   if type == string,  then data is the string value of the message
%   if type == system,  then data is the string value of the system message
%   if type == image,   then data is the same as the result of urbiGetImage(con)
%     (with the fields matrix, width, height, timeStamp)
%   if type == sound,   then data is the same as the result of urbiGetSound(con, length)
%     (with the fields samples, Fs, nbits, length)
%   if type == otherBIN then data  is the vector of bytes of the BIN info



string = pnet(con, 'readline', 'view', 'noblock');
disp(string);


data = [];
type = 'nothing';
timeStamp = -1;
tag = '';

if (~isempty(string))
    ans_string = strread(string, '%s');
    [timeStamp, tag] = strread(ans_string{1}, '[%u:%s]', 'whitespace', ']');
            %display(ans_string) ; 
            
umessage = urbiFullParse(ans_string, timeStamp, con);

type = umessage.type;

if (strcmp(type,'numeric') || strcmp(type,'string') || strcmp(type,'system') || strcmp(type,'error'))
    data = umessage.value;
elseif(strcmp(type,'image'))
    data = umessage.image;
elseif(strcmp(type,'sound'))
    data = umessage.sound;
elseif(strcmp(type,'otherBIN'))
    data = umessage.bin;
end;

end ;
