function [V, time] = urbiGetDevicesFields(con, listDevicesFields)
% Read multiple device fields or  variables value at the same time
% The result of this function is a vector V with the values in the same
% order as in the argument list 
% e.g. V = getDevicesFields(aibo, {'headPan.val','myVariable.val'});

urbiClearConnection(con) ;
tag_base = 'dgfstag';
tag_base_size = size(tag_base, 2);

n = size(listDevicesFields, 2);
V = zeros(1, n);
cmd = [];
% Compose the command string
for i=1:n
    tag{i} = [tag_base num2str(i)];
    cmd = [cmd ' & ' tag{i} ':' listDevicesFields{i} ];
end;
cmd = cmd(4:size(cmd, 2));
% Send the command
urbiSend(con, [cmd ';']);
% Retrieve the results
for i=1:n
    o_tag = '';
    while ~strncmp (o_tag, tag_base, tag_base_size)
        r = pnet(con, 'readline', 'noblock');
        while isempty(r)
            r = pnet(con, 'readline', 'noblock');
        end;
        ans_string = strread (r, '%s');
        [timeStamp, o_tag] = strread(ans_string{1}, '[%u:%s]', 'whitespace', ']');
        o_tag = char(o_tag);
    end;
    index = str2num(o_tag(tag_base_size + 1:size(o_tag, 2)));
    V(index) = str2num(ans_string{2});
end;
time = timeStamp;
