function result = urbiGetNImages(con, n,deviceName)
% Gets n successive images from the urbi server  (default device : camera)
% This function gets n images from the specified device, or from the
% device "camera" if it is not specified.
% The parameters for the camera have to set by the user with urbiSend
%
% The result is an array of structure with images ranked in their time order
% The access to each image is made with
% images(1).matrix, images(1).width, images(1).height, images(1).time
% images(2).matrix, images(2).width, images(2).height, images(2).time ...

if (nargin == 2)
  deviceName='camera';
end

io_buffer = 'imageB.jpg';
temp = [];
T = [];
result = [];
for index=1:n
    urbiSend(con, [ deviceName ';']);
    type = 'nothing' ;
    while ~strcmp(type, 'BIN')
        r = pnet(con, 'readline');
        if (~isempty(r))
            [timeStamp, tag, typeTmp, rnum, rstring] = urbiParse(r) ;
            type = typeTmp ;
        end;
    end;
    [timeStamp, tag, type, rnum, rstring] = urbiParse(r) ;
    width = sscanf(rstring{5}, '%u');
    height = sscanf(rstring{6}, '%u');
    nbytes = sscanf(rstring{3}, '%u') ;
    pnet(con, 'readtofile', io_buffer, nbytes) ;
    matrix = imread(io_buffer) ;
    time = timeStamp;
    temp = [temp ; struct('matrix',matrix,'width',width,'height',height, 'time', time)];
    T = [T ; [index, time]];
end;
% Sort the images
sorted_index = sortrows(T, 2);
for i=1:n
    result = [ result ; temp( sorted_index(i, 1) ) ];
end;
