function sound = urbiGetSound(con,duration, deviceName)
% Gets 'duration' ms of sound recorded by the server (micro.val by default),
%
% The result is a Sound struct:
%   sound.samples   : vector of samples
%   sound.Fs        : sampling frequency used by the server
%   sound.channel   : number of channels used by the server
%   sound.nbits     : number of bits used by the server
%   sound.length    : number of samples in the sound
%   sound.duration  : duration in ms

if (nargin==2)
    deviceName='micro.val';
end

%urbiClearConnection(con,1);

% Get the sound informations
soundUnit = urbiGetSoundUnit(con,deviceName);
nb_to_get = floor(duration / (soundUnit.duration));


% command to get the right number of sound samples
ref_tag = 'gsnd_tag';
urbiClearConnection(con,0);
urbiSend(con, [ 'for(i=0;i<' num2str(nb_to_get) ';i++) gsnd_tag:' deviceName ';' ]);

% Reception of the samples
length = 0;
samples = [];

for i=1:(nb_to_get)
% getting header
o_tag = '';
while ~strcmp (o_tag, ref_tag)
  r = pnet(con, 'readline', 'noblock');
  while isempty(r)
    r = pnet(con, 'readline', 'noblock');
  end;
  ans_string = strread (r, '%s');
  [timeStamp, o_tag] = strread(ans_string{1}, '[%u:%s]', 'whitespace', ']');
  o_tag = char(o_tag);
end;

% Data stream length is in bytes
length =  length + str2num ( ans_string{3} );

% getting samples
samples = [samples pnet(con, 'read', soundUnit.length) ];

end;

% duration
duration = length/(soundUnit.Fs*soundUnit.channel*(soundUnit.nbits/8))*1000;


% constructing output
sound = struct('samples',samples,'Fs',soundUnit.Fs,'nbits', soundUnit.nbits, 'length', length, 'channel', soundUnit.channel, 'duration', duration);


