function sound = urbiGetSoundUnit(con,deviceName)
% Gets one unit of sound recorded by the server (micro.val by default),
% 
% This function gets a sound from the URBI server. The length of the sound
% is the duration of a urbi server cycle (e.g. 32 ms for aibo)
%
% The result is a Sound struct:
%   sound.samples   : vector of samples
%   sound.Fs        : sampling frequency used by the server
%   sound.channel   : number of channels used by the server
%   sound.nbits     : number of bits used by the server
%   sound.length    : number of samples in the sound
%   sound.duration  : duration in ms

if (nargin == 1)
    deviceName = 'micro.val';
end


ref_tag = 'gsnd_tag';

urbiClearConnection(con,0);
urbiSend(con, [ 'gsnd_tag:' deviceName ';' ]);

% getting header
o_tag = '';
while ~strcmp (o_tag, ref_tag)
  r = pnet(con, 'readline', 'noblock');
  while isempty(r)
    r = pnet(con, 'readline', 'noblock');
  end;
  ans_string = strread (r, '%s');
  [timeStamp, o_tag] = strread(ans_string{1}, '[%u:%s]', 'whitespace', ']');
  o_tag = char(o_tag);
end;

% BIN type : ans_string{2} == 'BIN'

% Data stream length is in bytes
length = str2num ( ans_string{3} );

% raw format : ans_string{4} == 'raw'

% number of channels :
channel = str2num ( ans_string{5} );

% Sample Rate in HZ
Fs = str2num ( ans_string{6} );

% bits per sample
nbits = str2num ( ans_string{7} );

% getting samples
samples = pnet(con, 'read', length);

% duration
duration = length/(Fs*channel*(nbits/8))*1000;

% constructing output
sound = struct('samples',samples,'Fs',Fs,'nbits', nbits, 'length', length, 'channel', channel, 'duration', duration);
