function callBackId = urbiSetCallback(con, f, mytag)
% Associate a callback to "mytag" on the connection "con"
% This function will associate a function f to a tag on a given connection.
% If the function urbiProcessEvent reads a message with this tag on this
% connection, the function f will be called with a urbiMessage as argument:
%
%   urbiCallBackAction = f(urbiMessage)
%
% with :
%   urbiMessage.timeStamp  : the time stamp of the message
%   urbiMessage.tag        : the tag of the message
%   urbiMessage.type       : the type of the message: 'numeric', 'string',
%   'system', 'image', 'sound', or 'otherBIN')
%
%   if type == numeric, then urbiMessage.value (the numeric value of the message)
%   if type == string,  then urbiMessage.value (the string value of the message)
%   if type == system,  then urbiMessage.value (the string value of the system message)
%   if type == image,   then urbiMessage.image is the same as the result of urbiGetImage(con)
%     (with the fields matrix, width, height, timeStamp)
%   if type == sound,   then urbiMessage.sound is the same as the result of urbiGetSound(con, length)
%     (with the fields samples, Fs, nbits, length)
%   if type == otherBIN then urbiMessage.bin (the vector of bytes of the BIN info)
%
% The callback function f should return either 0, and in this case the callback is deleted,
% or 1, and in this case the callback continues,
%
% urbiSetCallBack returns an id for the callback so that later on it can be
% deleted manually using urbiDeleteCallBack(callBackId);

global Internal_Call_Back_Array Internal_Call_Back_connections
persistent Internal_Call_Back_identificator 

if (size(Internal_Call_Back_identificator, 1) == 0)
    % First call return id zero
    Internal_Call_Back_identificator = 0;
    Internal_Call_Back_connections = con;
    Internal_Call_Back_Array{con + 1} = [];
else
    % Other calls increase the next id
    Internal_Call_Back_identificator = Internal_Call_Back_identificator + 1;
    if (~ismember(con, Internal_Call_Back_connections))
        Internal_Call_Back_connections = [con Internal_Call_Back_connections];
        Internal_Call_Back_Array{con +  1} = [];        
    end;
end;

Internal_Call_Back_Array{con + 1} = [Internal_Call_Back_Array{con + 1} ; 
struct('id', Internal_Call_Back_identificator, 'function', f, 'tag', mytag) ];
callBackId = Internal_Call_Back_identificator;
 
