#include "vidcap.h"

UStart(vidcap);

vidcap::vidcap(std::string s) : UObject(s)
{
	UBindFunction (vidcap, init);
	UBindFunction (vidcap, start);
	UBindFunction (vidcap, startdelay);
	UBindFunction (vidcap, stop);
	UBindFunction (vidcap, initrecord);
	UBindFunction (vidcap, record);
	UBindFunction (vidcap, endrecord);

	UBindVar(vidcap,image);
	UBindVar(vidcap,initimage);
	UBindVar(vidcap,frequency);
	UBindVar(vidcap,deviceCapture);
	UBindVar(vidcap,delay);

	UNotifyChange(image,&vidcap::valueChanged);
	UNotifyChange(initimage,&vidcap::initvalueChanged);

	frequency = 40; // 1 capture image toutes les 40 ms, soit 25 images par seconde
	deviceCapture="camera.val"; // on recupere les images sur camera.val
	tab = NULL; // stockage de l'image
	state = 0; // 0=>pas d'enregistrement et 1=>enregistrement
	frames = 0; // permet de suivre l'evolution des images
	delay=10;
        URBI(( +connection("all"): echo("send \"vidcap.start();\" to start record");));
}


vidcap::~vidcap()
{
	delete tab;
}


int vidcap::init(void)
{
  return 1; // Singleton
}

void vidcap::start(void)
{
  printf("function start\n");

  frames=0;
  state=1;
  URBI(( vidcap.initimage=camera.val, ));
  URBI(( vidcap.initrecord(), ));
  URBI(( tagvidcap:every(vidcap.frequency){vidcap.image=camera.val}, ));

  URBI(( +connection("all"): echo("send \"vidcap.stop();\" to stop record");));
  printf("end function start\n");
    return ;
}

void vidcap::startdelay(int delay_s)
{
  delay=delay_s;
  printf("function start with delay\n");
  frames=0;
  state=1;
  URBI(( vidcap.initimage=camera.val, ));
  URBI(( vidcap.initrecord(), ));
  URBI(( tagvidcap:every(vidcap.frequency){vidcap.image=camera.val}, ));

  URBI(( timedelay = 0; timedelay= 1 time:vidcap.delay*1000, ));
  URBI(( at(timedelay == 1) vidcap.stop(); ));

  printf("end function start\n");
    return ;
}

void vidcap::stop(void)
{
    printf("function stop\n");

    frames=0;
    state=0;
    URBI(( +connection("all"):stop tagvidcap; ));
    URBI(( vidcap.endrecord(), ));
    printf("end function stop\n");
    URBI(( +connection("all"): echo("End of record");));
    return ;
}


void vidcap::initrecord(void)
{
    printf("function initrecord\n");

    while( tab == NULL)
    {
	printf("en attente du malloc de tab\n");
    }

    const char *filename = "video.mpg";

    // On initialise FFMPEG
    ////////////////////////
    // must be called before using avcodec lib 
    avcodec_init();

    // register all the codecs (you can also register only the codec
    // you wish to have smaller code 
    avcodec_register_all();

    // find the mpeg1 video encoder 
    codec = avcodec_find_encoder(CODEC_ID_MPEG1VIDEO);
    if (!codec) {
        fprintf(stderr, "codec not found\n");
        exit(1);
    }

    // AVCodec alloc context 
    c= NULL;
    c= avcodec_alloc_context();
    // put sample parameters 
    c->bit_rate = 400000;
    // resolution must be a multiple of two 
    c->width = widthimage;
    c->height = heigthimage;
    // frames per second 
    int nbimagesseconde = (int) (1000 / (ufloat)frequency);
//    c->time_base= (AVRational){1,nbimagesseconde}; // 1 seconde = x images
    c->time_base= (AVRational){1,25}; // 1 seconde = 25 images
    c->gop_size = 10; // emit one intra frame every ten frames 
    c->max_b_frames=1;
    c->pix_fmt = PIX_FMT_YUV420P;

    // open it 
    if (avcodec_open(c, codec) < 0) {
        fprintf(stderr, "could not open codec\n");
        exit(1);
    }

    // Allocate an AVFrame structure
    picture = avcodec_alloc_frame();
    if(picture==NULL){
        fprintf(stderr, "could not alloc frame\n");
        exit(1);
    }

    pictureRGB24 = avcodec_alloc_frame();
    if(pictureRGB24==NULL){
        fprintf(stderr, "could not alloc frame\n");
        exit(1);
    }

    // the codec gives us the frame size, in samples 
    f = fopen(filename, "wb");
    if (!f) {
        fprintf(stderr, "could not open %s\n", filename);
        exit(1);
    }

    // alloc image and output buffer 
    outbuf_size = 100000;
    outbuf = (uint8_t* )malloc(outbuf_size);

    size = avpicture_get_size(PIX_FMT_YUV420P, c->width, c->height); // size for YUV420P 
    picture_buf1 = (uint8_t* )malloc( sizeof(uint8_t)*size ); 
    picture_buf2 = (uint8_t* )malloc( sizeof(uint8_t)*size ); 
    picture_buf3 = (uint8_t* )malloc( sizeof(uint8_t)*size ); 

    sizeRGB24 = avpicture_get_size(PIX_FMT_RGB24, c->width, c->height); // size for RGB24 
    picture_bufRGB24 = (uint8_t* )malloc( sizeof(uint8_t)*sizeRGB24 ); 

    picture->data[0] = picture_buf1; // correspond a Y
    picture->data[1] = picture_buf2; // correspond a U
    picture->data[2] = picture_buf3; // correspond a V
    picture->linesize[0] = c->width; // largeur
    picture->linesize[1] = c->width/2; // moitie de la largeur
    picture->linesize[2] = c->width/2; // moitie de la largeur

    pictureRGB24->data[0] = picture_bufRGB24; // correspond a l'image RGB
    pictureRGB24->linesize[0] = c->width*3; //largeur

    printf("end function initrecord\n");

    return;
}



void vidcap::endrecord(void)
{
    printf("function endrecord\n");

    // get the delayed frames 
    for(; out_size; i++) 
    {
        fflush(stdout);

        out_size = avcodec_encode_video(c, outbuf, outbuf_size, NULL);
        printf("write frame %3d (size=%5d)\n", i, out_size);
        fwrite(outbuf, 1, out_size, f);
    }

    // add sequence end code to have a real mpeg file
    outbuf[0] = 0x00;
    outbuf[1] = 0x00;
    outbuf[2] = 0x01;
    outbuf[3] = 0xb7;
    fwrite(outbuf, 1, 4, f);
    fclose(f);
    free(picture_buf);

    // Close the codec
    avcodec_close(c);
    av_free(c);
    av_free(picture);

    printf("end function endrecord\n");
    return;
}





void vidcap::record(void)
{
    int frames_cap=frames;

    return ;
}



int vidcap::initvalueChanged(UVar &camera)
{    
    printf("function initvaluechanged\n");

    UImage imgsrc = (UImage) camera;
    int width=imgsrc.width;
    int height=imgsrc.height;

    if( tab == NULL)
    {
      tab = new char[width*height*3];
      widthimage = width;
      heigthimage = height;
    }

    printf("end function initvaluechanged\n");
    return 0;
}



int vidcap::valueChanged(UVar &camera)
{    

    printf("function valuechanged\n");

   if( state == 0) // pas enregistrement
	return 0;

   else if( state == 1) // enregistrement
   {

	frames++;

	   UValue variablecamera = camera.val();
	   if( variablecamera.type != DATA_BINARY )
	   {
	       return 0;
	   }

	    UImage imgsrc = (UImage) camera;

	    int width=imgsrc.width;
	    int height=imgsrc.height;


	   if(imgsrc.imageFormat == IMAGE_JPEG )
	   {
	       int size=width*height*3;
	       convertJPEGtoRGB ( (const byte *) imgsrc.data, imgsrc.size, (byte *) tab , size ); // on copie les donnees dans tab
	   }

	   else if(imgsrc.imageFormat == IMAGE_YCbCr )
	   {
	       convertYCrCbtoRGB( (const byte *) imgsrc.data, imgsrc.size, (byte *) tab );
	   }

	   else // cas RGB24
	   {
		    int i,j;
		    for(i=0; i<width; i++)
		    {
		             for(j=0; j<height; j++)
		             {
		                tab[(i+j*width)*3+0] = imgsrc.data[(i+j*width)*3+0]; // R
		                tab[(i+j*width)*3+1] = imgsrc.data[(i+j*width)*3+2]; // G
		                tab[(i+j*width)*3+2] = imgsrc.data[(i+j*width)*3+1]; // B     
		             }
		    }
	   }


//////////////////////////////////////////////

	        fflush(stdout);
    // creation de l'image RGB24
    avpicture_fill((AVPicture *)pictureRGB24, (uint8_t*) tab, PIX_FMT_RGB24, width, height);
//     SaveFrame(pictureRGB24, width, height, 0); // permet de sauver l'image
	// a ce niveau la, l'image RGB est correctement stokee dans pictureRGB24

    // conversion RGB24 en YUV420P
    img_convert((AVPicture *)picture, PIX_FMT_YUV420P, (AVPicture*)pictureRGB24, PIX_FMT_RGB24, width, height);
// c'est au niveau de cette conversion que tout deconne


	        // encode the image 
	        out_size = avcodec_encode_video(c, outbuf, outbuf_size, picture);
	        printf("encoding frame %3d (size=%5d)\n", i, out_size);
	        fwrite(outbuf, 1, out_size, f);


/////////////////////////////////////////////


   }

 	return 0;
}


void vidcap::SaveFrame(AVFrame *pFrame, int width, int height, int iFrame)
{
    FILE *pFile;
    char szFilename[32];
    int  y;

    // Open file
    sprintf(szFilename, "frame%d.ppm", iFrame);
    pFile=fopen(szFilename, "wb");
    if(pFile==NULL)
        return;

    // Write header
    fprintf(pFile, "P6\n%d %d\n255\n", width, height);

    // Write pixel data
    for(y=0; y<height; y++)
        fwrite(pFrame->data[0]+y*pFrame->linesize[0], 1, width*3, pFile);

    // Close file
    fclose(pFile);
}


#if 0
permet de mettre une grande quantite de texte en commentaire
#endif
