/*
 * CFtpServer :: a FTP Server Class Library
 * Copyright (C) 2005, Poumailloux Julien aka theBrowser
 *
 * Mail :: thebrowser@gmail.com

Copyright (c) 2005 POUMAILLOUX Julien

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated
documentation files (the "Software"), to deal in the
Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall
be included in all copies or substantial portions of the
Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH
THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

To succesfully link program which uses CFtpServer, you need to use following command:
g++ -o ftp main.o CFtpServer.o -lnsl -lpthread
( Add '-lsocket' too under SunOS and maybe some other OS. )

*/

#include "CFtpServer.h"

/****************************************
 * CONSTRUCTOR && DESTRUCTOR
 ***************************************/

CFtpServer::CFtpServer(void)
{
	int i=0;
	this->szMonth[i++] = "Jan";	this->szMonth[i++] = "Feb";
	this->szMonth[i++] = "Mar";	this->szMonth[i++] = "Apr";
	this->szMonth[i++] = "May";	this->szMonth[i++] = "Jun";
	this->szMonth[i++] = "Jul";	this->szMonth[i++] = "Aug";
	this->szMonth[i++] = "Sep";	this->szMonth[i++] = "Oct";
	this->szMonth[i++] = "Nov";	this->szMonth[i++] = "Dec";

	this->usPort=21;
	this->iNbUser = 0;
	this->iNbClient = 0;
	this->iReplyMaxLen = 4096;

	this->DataPortRange.usStart = 100;
	this->DataPortRange.iNumber = 900;

	this->iCheckPassDelay = 0; // <- Anti BruteForcing Protection
	this->bIsAccepting = false;
	this->bIsListening = false;
	this->AnonymousUser = NULL;
	this->bAllowAnonymous = false;
	this->UserListHead=UserListLast = NULL;
	this->ClientListHead = this->ClientListLast=NULL;
    srand((unsigned) time(NULL));
}

CFtpServer::~CFtpServer(void)
{
	if( this->IsListening() )
		this->StopListening();
	this->DelAllClient();
	this->DelAllUser();
}

/****************************************
 * CONFIG
 ***************************************/

bool CFtpServer::AllowAnonymous( bool bDoAllow, char *szStartPath )
{
	if( bDoAllow ) {
		if( this->IsAnonymousAllowed() == false ) {
			if( (this->AnonymousUser = this->AddUser( "anonymous", NULL, szStartPath )) == NULL )
				return false;
			this->SetUserPriv( this->AnonymousUser, CFtpServer::READFILE | CFtpServer::LIST );
		}
		this->bAllowAnonymous = true;
		return true;
	} else {
		if( this->IsAnonymousAllowed() == true ) {
			if( this->DelUser( this->AnonymousUser ) == false )
				return false;
			this->AnonymousUser = NULL;
			this->bAllowAnonymous = false;
		}
		return true;
	}
	return false;
}

bool CFtpServer::SetPort( unsigned short int usPort )
{
	if( !this->IsListening() ) {
		if( usPort != 0 ) {
			this->usPort = usPort;
			return true;
		}
	}
	return false;
}

bool CFtpServer::SetDataPortRange( unsigned short int usDataPortStart, int iNumber )
{
	if( iNumber  >= 1 && usDataPortStart > 0
		&& (usDataPortStart + iNumber) <= 65535 ) {
		this->DataPortRange.usStart = usDataPortStart;
		this->DataPortRange.iNumber = iNumber;
		return true;
	}
	return false;
}

/****************************************
 * NETWORK
 ***************************************/

bool CFtpServer::StartListening( void )
{	if( this->bIsListening == false ) {
		int on = 1;
		struct sockaddr_in sin;
		sin.sin_family = AF_INET;
		sin.sin_addr.s_addr = INADDR_ANY;
		sin.sin_port = htons( this->usPort );
		this->ListeningSock = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
		if( this->ListeningSock == INVALID_SOCKET
			#ifndef _WIN32 // On win32, SO_REUSEADDR allow 2 sockets to listen on the tcp-port.
				|| setsockopt( this->ListeningSock, SOL_SOCKET, SO_REUSEADDR,(char *) &on, sizeof( on ) ) == SOCKET_ERROR
			#endif
			|| bind( this->ListeningSock, (struct sockaddr *) &sin, sizeof( struct sockaddr_in ) ) == SOCKET_ERROR
			|| listen( this->ListeningSock, 0) == SOCKET_ERROR ) {
			this->CloseSocket( this->ListeningSock );
			this->bIsListening = false;
		} else this->bIsListening = true;
	}
	return this->bIsListening;
}

bool CFtpServer::StopListening( void )
{
	if( this->IsListening() ) {
		this->CloseSocket( this->ListeningSock );
		#ifdef WIN32
			if( this->hAcceptingThread ) CloseHandle( this->hAcceptingThread );
		#else
			pthread_detach( this->pAcceptingThreadID );
		#endif
	}
	this->bIsListening = false;
	return true;
}

bool CFtpServer::StartAccepting( void )
{
	if( this->bIsAccepting == false && this->bIsListening == true ) {
		#ifdef WIN32
			this->hAcceptingThread = (HANDLE) _beginthreadex( NULL, 0,
				this->StartAcceptingEx, this, 0, &this->uAcceptingThreadID );
			if( this->hAcceptingThread != 0 )
				this->bIsAccepting = true;
		#else
			if( pthread_create( &this->pAcceptingThreadID, NULL,
				this->StartAcceptingEx, this ) == 0 )
				this->bIsAccepting = true;

		#endif
	}
	return this->bIsAccepting;
}

#ifdef WIN32
	unsigned CFtpServer::StartAcceptingEx( void *pvParam )
#else
	void* CFtpServer::StartAcceptingEx( void *pvParam )
#endif
{
	class CFtpServer *CFtpServerEx = (CFtpServer*) pvParam;

	SOCKET TempSock; struct sockaddr_in Sin;
	socklen_t Sin_len = sizeof(struct sockaddr_in);
	
	while (1)
	{
		TempSock = accept( CFtpServerEx->ListeningSock, (struct sockaddr *) &Sin, &Sin_len);
        if( TempSock != INVALID_SOCKET ) {
			struct CFtpServer::ClientNode *NewClient = 
				CFtpServerEx->AddClient( CFtpServerEx, TempSock, &Sin );
			if( NewClient != NULL ) {
				#ifdef WIN32
					NewClient->hClientThread = (HANDLE) _beginthreadex( NULL, 0,
						CFtpServerEx->ClientShell, NewClient, 0, &NewClient->dClientThreadId );
				#else
					pthread_create( &NewClient->pClientThreadID, NULL,
						CFtpServerEx->ClientShell, NewClient );
				#endif
			}
        } else
			break;
	}
	#ifdef WIN32
		_endthreadex( 0 );
		return 0;
	#else
		pthread_exit( 0 );
	#endif
}

/****************************************
 * CLIENT SHELL
 ***************************************/
#ifdef WIN32
	unsigned CFtpServer::ClientShell( void *pvParam )
#else
	void* CFtpServer::ClientShell( void *pvParam )
#endif
{
	struct CFtpServer::ClientNode *Client=(struct CFtpServer::ClientNode*)pvParam;
	class CFtpServer *CFtpServerEx=Client->CFtpServerEx;

	CFtpServerEx->SendReply( Client, "220 Browser Ftp Server.\r\n"); // Send "Welcome"

	Client->eStatus = Status_Waiting;

	int Err=0;
	char szCmd[320];
	char *psNextCmd = NULL;;
	memset( &szCmd, 0x0, 320 );
	int iCmdMaxLen = MAX_PATH + 16, iCmdLen=0, iByteRecv, iNextiCmdLen;
	
	struct stat64 st;
	char *pszPath = NULL;
	char *pszCmdArg = NULL;

	// Main Loop
	while ( 1 ) {

		iByteRecv = 0;
		pszCmdArg = NULL;
		memset( &st, 0x0, sizeof( st ) );
		memset( &szCmd, 0x0, iCmdLen ); iCmdLen=0;
		if( pszPath ) {
			delete [] pszPath;
			pszPath = NULL;
		}

		// Copy the "old-next line" from the buffer to the Command Buffer
		if( psNextCmd && iNextiCmdLen > 0 ) {
			memcpy( szCmd, psNextCmd, iNextiCmdLen );
			iByteRecv = iNextiCmdLen;
			iNextiCmdLen = 0;
			psNextCmd = NULL;
		}

		// Receive a line and in case we received more, we put the rest in a buffer
		while( !memchr( szCmd, '\n', iByteRecv ) ) {
			if( (iCmdMaxLen-iByteRecv) <= 0 ) {
				Err = -1; break;
			}
			Err=recv( Client->CtrlSock, szCmd + iByteRecv, iCmdMaxLen - iByteRecv, 0 );
			if( Err > 0 ) {
				iByteRecv += Err;
			} else break;
		}
		if( Err <= 0 ) {
			break; // SOCKET_ERROR
		} else {
			psNextCmd = strstr( szCmd, "\r\n" );
			if( psNextCmd ) {
				psNextCmd[0] = '\0';
				psNextCmd += 2;
				iCmdLen = strlen( szCmd );
				iNextiCmdLen = iByteRecv - iCmdLen - 2;
			} else if( psNextCmd = strchr( szCmd, '\n' ) ) {
				psNextCmd[0] = '\0';
				psNextCmd += 1;
				iCmdLen = strlen( szCmd );
				iNextiCmdLen = iByteRecv - iCmdLen - 1;
			}
			if( iNextiCmdLen == 0 )
				psNextCmd = NULL;
		}

		// Separate the Cmd and the Arguments
		char *pszSpace = strchr( szCmd, ' ' );
		if( pszSpace ) { 
			pszSpace[0] = '\0';
			pszCmdArg = pszSpace + 1;
		}

		// Convert the Cmd to uppercase
		#ifdef WIN32
			_strupr( szCmd );
		#else
			char *psToUpr = szCmd;
			while(	psToUpr && psToUpr[0] ) {
				*psToUpr = toupper( *psToUpr );
				psToUpr++;
			}
		#endif

		// Enumerate the Commands
		if( !strcmp( szCmd, "QUIT" ) ) {
			CFtpServerEx->SendReply( Client, "221 Goodbye.\r\n");
			break;

		} else if( !strcmp( szCmd, "USER" ) ) {

			if( !pszCmdArg ) {
				CFtpServerEx->SendReply( Client, "501 Invalid number of arguments.\r\n");
			} else {
				if( CFtpServerEx->bAllowAnonymous && !strcasecmp( pszCmdArg, "anonymous" ) 
					&& CFtpServerEx->AnonymousUser ) {
					Client->bIsLogged = true;
					Client->bIsAnonymous = true;
					CFtpServerEx->SendReply( Client, "230 User Logged In.\r\n");
					Client->User = CFtpServerEx->AnonymousUser;
				} else {
					Client->User = CFtpServerEx->SearchUserFromName(pszCmdArg);
					if( Client->User && Client->User->bIsEnabled == false )
						Client->User = NULL;
					CFtpServerEx->SendReply( Client, "331 Password required for this user.\r\n");
				}
			}
			continue;

		} else if( !strcmp( szCmd, "PASS" ) ) {

			if( Client->bIsAnonymous ) {
				Client->bIsLogged = true;
				CFtpServerEx->SendReply( Client, "230 User Logged In.\r\n");
			} else {
				#ifdef CFTPSERVER_ANTIBRUTEFORCING
					#ifdef WIN32
						Sleep( CFtpServerEx->iCheckPassDelay );
					#else
						usleep( CFtpServerEx->iCheckPassDelay );
					#endif
				#endif
				if( pszCmdArg ) {
					if( Client->User && Client->User->szPasswd && !strcmp(pszCmdArg, Client->User->szPasswd ) ) {
						if( Client->User->iMaxClient == CFtpServer::Unlimited
							||  Client->User->iNbClient < Client->User->iMaxClient )
						{
							Client->bIsLogged=true;
							Client->User->iNbClient += 1;
							CFtpServerEx->SendReply( Client, "230 User Logged In.\r\n");
						} else CFtpServerEx->SendReply( Client, "421 Too many users logged in for this account.\r\n");
					} else {
						CFtpServerEx->SendReply( Client, "530 Please login with valid USER and PASS.\r\n");
						#ifdef CFTPSERVER_ANTIBRUTEFORCING
							break;
						#endif
					}
				} else CFtpServerEx->SendReply( Client, "501 Invalid number of arguments.\r\n");
			}
			continue;

		} else if( !Client->bIsLogged ) {

			CFtpServerEx->SendReply( Client, "530 Please login with USER and PASS.\r\n");
			continue;

		} else if( !strcmp( szCmd, "NOOP" ) || !strcmp( szCmd, "ALLO" ) ) {

			CFtpServerEx->SendReply( Client, "200 NOOP Command Successful.\r\n");
			continue;

		#ifdef CFTPSERVER_USE_EXTRACMD
		} else if( !strcmp( szCmd, "SITE" ) ) {
			char *pszSiteArg = pszCmdArg;
			if( pszCmdArg ) {
				unsigned char ucExtraCmd = CFtpServerEx->GetUserExtraCmd( Client->User );
				if( !strncasecmp( pszCmdArg, "EXEC ", 5 ) ) {
					if( (ucExtraCmd & CFtpServer::ExtraCmd_EXEC) == CFtpServer::ExtraCmd_EXEC ) {
						if( strlen( pszSiteArg ) >= 5 )
							system( pszSiteArg + 5 );
							CFtpServerEx->SendReply( Client, "200 SITE EXEC Successful.\r\n" );
						continue;
					} else CFtpServerEx->SendReply( Client, "550 Permission denied.\r\n");
				}
			}
			CFtpServerEx->SendReply( Client, "501 Syntax error in arguments.\r\n");
			continue;
		#endif

		} else if( !strcmp( szCmd, "HELP" ) ) {

			CFtpServerEx->SendReply( Client, "500 No Help Available.\r\n");
			continue;

		} else if( !strcmp( szCmd, "REIN" ) ) {

			Client->User = NULL;
			Client->bIsLogged = false;
			Client->bBinaryMode = true;
			Client->bIsAnonymous = false;
			strcpy( Client->szCurrentDir, "/" );
			Client->eDataMode = CFtpServerEx->Mode_None;
			if( Client->pszRenameFromPath ) free( Client->pszRenameFromPath );
			// /!\ the current transfert must not be stopped
			CFtpServerEx->SendReply( Client, "200 Reinitialize Successful.\r\n");
			continue;

		} else if( !strcmp( szCmd, "SYST" ) ) {

			CFtpServerEx->SendReply( Client,"215 UNIX Type: L8\r\n");
			continue;

		} else if( !strcmp( szCmd, "STRU" ) ) {

			if( pszCmdArg ) {
				if( !strcmp( pszCmdArg, "F" ) ) {
					CFtpServerEx->SendReply( Client, "200 STRU Command Successful.\r\n" );
				} else
					CFtpServerEx->SendReply( Client, "504 STRU failled. Parametre not implemented\r\n" );
			} else CFtpServerEx->SendReply( Client, "501 Invalid number of arguments.\r\n" );
			continue;

		} else if( !strcmp( szCmd, "MODE" ) ) {

			if( pszCmdArg ) {
				if( !strcmp( pszCmdArg, "S" ) ) {
					CFtpServerEx->SendReply( Client, "200 Mode set to S.\r\n");
				} else if( !strcmp( pszCmdArg, "C" ) ) { // here zlib compression
					CFtpServerEx->SendReply( Client, "502 MODE non-implemented.\r\n" );
				} else
					CFtpServerEx->SendReply2( Client, "504 \"%s\": Unsupported transfer MODE.\r\n", pszCmdArg );
			} else CFtpServerEx->SendReply( Client, "501 Invalid number of arguments.\r\n" );
			continue;

		} else if( !strcmp( szCmd, "TYPE" ) ) {

			if( pszCmdArg ) {
				if( pszCmdArg[0] == 'A' ) {
					Client->bBinaryMode=false;
					CFtpServerEx->SendReply( Client, "200 ASCII transfer mode active.\r\n");
					//CFtpServerEx->SendMessage( Client, "502 Unimplemented TYPE.\r\n");
				} else if( pszCmdArg[0] == 'I' ) { // ASCII isn't supported infact..
					Client->bBinaryMode=true;
					CFtpServerEx->SendReply( Client, "200 Binary transfer mode active.\r\n");
				} else 
					CFtpServerEx->SendReply( Client, "550 Error - unknown binary mode.\r\n");
			} else CFtpServerEx->SendReply( Client, "501 Invalid number of arguments.\r\n");
			continue;

		} else if( !strcmp( szCmd, "PORT" ) ) {

			unsigned int iIp1 = 0, iIp2 = 0, iIp3 = 0, iIp4 = 0, iPort1 = 0, iPort2 = 0;
			if( Client->eStatus == Status_Waiting && pszCmdArg
				&& sscanf(pszCmdArg, "%u,%u,%u,%u,%u,%u", &iIp1, &iIp2, &iIp3, &iIp4, &iPort1, &iPort2) == 6
				&& iIp1 <= 255 && iIp2 <= 255 && iIp3 <= 255 && iIp4 <= 255
				&& iPort1 <= 255 && iPort2 <= 255 && (iIp1|iIp2|iIp3|iIp4) != 0
				&& (iPort1|iPort2) != 0)
			{

				if( Client->eDataMode != Mode_None ) {
					CFtpServerEx->CloseSocket( Client->DataSock );
					Client->DataSock = INVALID_SOCKET;
					Client->eDataMode = Mode_None;
				}

				char szClientIP[32];
				sprintf( szClientIP, "%u.%u.%u.%u", iIp1, iIp2, iIp3, iIp4 );
				if( (inet_addr( szClientIP ) == Client->ulClientIP)
					|| Client->ulClientIP == inet_addr( "127.0.0.1" )
				) {
					Client->eDataMode = Mode_PORT;
					Client->usDataPort = ( iPort1 * 256 ) + iPort2;
					CFtpServerEx->SendReply( Client, "200 PORT command successful.\r\n");
				} else
					CFtpServerEx->SendReply( Client, "501 PORT address does not match originator.\r\n");
			} else
				CFtpServerEx->SendReply( Client, "501 Syntax error in arguments.\r\n");
			continue;

		} else if( !strcmp( szCmd, "PASV" ) ) {

			if( Client->eStatus == Status_Waiting ) {
				
				if( Client->eDataMode != Mode_None ) {
					CFtpServerEx->CloseSocket( Client->DataSock );
					Client->DataSock = INVALID_SOCKET;
					Client->eDataMode = Mode_None;
				}
				Client->DataSock = socket( AF_INET, SOCK_STREAM, IPPROTO_TCP );
				int on = 1;
				struct sockaddr_in sin;
				sin.sin_family = AF_INET;
				sin.sin_addr.s_addr = Client->ulServerIP;

				if( Client->DataSock != INVALID_SOCKET
				#ifndef WIN32
					&& setsockopt( Client->DataSock, SOL_SOCKET, SO_REUSEADDR,(char *) &on, sizeof( on ) ) != SOCKET_ERROR
				#endif
				) {
					int checked_port_count = 0;
					do {
						if( checked_port_count >= CFtpServerEx->DataPortRange.iNumber ) {
							checked_port_count = -1;
							CFtpServerEx->CloseSocket( Client->DataSock );
							Client->DataSock = INVALID_SOCKET;
							break;
						}
						checked_port_count ++;
						Client->usDataPort = (int)
							((float) rand() / RAND_MAX * (CFtpServerEx->DataPortRange.iNumber-1) + CFtpServerEx->DataPortRange.usStart );
						sin.sin_port = htons( Client->usDataPort );
					} while( bind( Client->DataSock, (struct sockaddr *) &sin, sizeof(struct sockaddr_in)) == SOCKET_ERROR
						|| listen( Client->DataSock, 1) == SOCKET_ERROR );

					if( checked_port_count == -1 ) {
						CFtpServerEx->SendReply( Client, "451 Internal error - No more data port available.\r\n" );
						break;
					}

					unsigned int uDataPort2 = Client->usDataPort % 256;					
					unsigned int uDataPort1 = ( Client->usDataPort - uDataPort2 ) / 256;
					unsigned long ulIp = ntohl( Client->ulServerIP ); // Get the local IP

					CFtpServerEx->SendReply2( Client, "227 Entering Passive Mode (%lu,%lu,%lu,%lu,%u,%u).\r\n",
						(ulIp >> 24) & 255, (ulIp >> 16) & 255, (ulIp >> 8) & 255, ulIp & 255,
						uDataPort1, uDataPort2 );

					Client->eDataMode = Mode_PASV;
				}
			} else CFtpServerEx->SendReply( Client, "425 You're already connected.\r\n");
			continue;


		} else if( !strcmp( szCmd, "LIST" ) || !strcmp( szCmd, "NLST" ) ) {

			CFtpServerEx->LIST_Command( Client, pszCmdArg, szCmd );
			continue;

		} else if( !strcmp( szCmd, "CWD" ) || !strcmp( szCmd, "XCWD" ) ) {

			if( pszCmdArg ) {
				pszPath = CFtpServerEx->BuildPath( Client->User->szStartDir, Client->szCurrentDir, pszCmdArg );
				if( pszPath && stat64( pszPath, &st ) == 0 ) {
					strcpy( Client->szCurrentDir, pszPath + strlen( Client->User->szStartDir ) );
					if( Client->szCurrentDir[0] == '\0' )
						strcpy( Client->szCurrentDir, "/" );
					CFtpServerEx->SendReply( Client, "250 CWD command successful.\r\n");
				} else CFtpServerEx->SendReply( Client, "550 No such file or directory.\r\n");
			} else CFtpServerEx->SendReply( Client, "501 Invalid number of arguments.\r\n");
			continue;

		} else if( !strcmp( szCmd, "FEAT" ) ) {

			CFtpServerEx->SendReply( Client, "211-Features:\r\n"
				" MDTM\r\n"
				" SIZE\r\n"
				"211 End\r\n" );

		} else if( !strcmp( szCmd, "MDTM" ) ) {

			if( pszCmdArg ) {
				pszPath = CFtpServerEx->BuildPath( Client->User->szStartDir, Client->szCurrentDir, pszCmdArg );
				struct tm *t;
				if( pszPath && !stat64( pszPath, &st )
					&& ( S_ISREG( st.st_mode ) )
					&& (t = gmtime((time_t *) &(st.st_mtime))) )
				{
					CFtpServerEx->SendReply2( Client, "213 %04d%02d%02d%02d%02d%02d\r\n",
						t->tm_year + 1900, t->tm_mon + 1, t->tm_mday,
						t->tm_hour, t->tm_min, t->tm_sec );
				} else
					CFtpServerEx->SendReply( Client, "550 No such file or directory.\r\n");
			} else
				CFtpServerEx->SendReply( Client, "501 Invalid number of arguments.\r\n");
			continue;

		} else if( !strcmp( szCmd, "PWD" ) || !strcmp( szCmd, "XPWD" )) {

			CFtpServerEx->SendReply2( Client, "257 \"%s\" is current directory.\r\n",
				Client->szCurrentDir);
			continue;

		} else if( !strcmp( szCmd, "CDUP" ) || !strcmp( szCmd, "XCUP" ) ) {

			char *szBuf = strchr( Client->szCurrentDir, '/' );
			if( szBuf == Client->szCurrentDir ) {
					szBuf[ 1 ] = '\0';
			} else szBuf[ 0 ] = '\0';
			CFtpServerEx->SendReply( Client, "250 CDUP command successful.\r\n");
			continue;

		} else if( !strcmp( szCmd, "STAT" ) ) {

			if( pszCmdArg ) {
				CFtpServerEx->LIST_Command( Client, pszCmdArg, szCmd );
			} else
				CFtpServerEx->SendReply( Client, "211 :: CFtpServer / Browser FTP Server:: thebrowser@gmail.com\r\n");

			continue;

		} else if( !strcmp( szCmd, "ABOR" ) ) {

			if( Client->eStatus != Status_Waiting ) {
				CFtpServerEx->CloseSocket( Client->DataSock );
				Client->DataSock = INVALID_SOCKET;
				CFtpServerEx->SendReply( Client, "426 Previous command has been finished abnormally.\r\n" );
				Client->eDataMode = Mode_None;
				Client->eStatus = Status_Waiting;
			}
			CFtpServerEx->SendReply( Client, "226 ABOR command successful.\r\n");
			continue;

		} else if( !strcmp( szCmd, "REST" ) ) {

			if( pszCmdArg && Client->eStatus == CFtpServer::Status_Waiting ) {
				#ifdef WIN32
					Client->CurrentTransfer.i64RestartAt = _atoi64( pszCmdArg );
				#else
					Client->CurrentTransfer.i64RestartAt = atoll( pszCmdArg );
				#endif
				CFtpServerEx->SendReply( Client, "350 REST command successful.\r\n");
			} else CFtpServerEx->SendReply( Client, "501 Syntax error in arguments.\r\n");
			continue;

		} else if( !strcmp( szCmd, "RETR" ) ) {

			if( !(Client->User->ucPriv & CFtpServer::READFILE) ) {
				CFtpServerEx->SendReply( Client, "550 Permission denied.\r\n");
				continue;
			}
			if( pszCmdArg ) {

				if( Client->eStatus != Status_Waiting )	{
					CFtpServerEx->SendReply( Client, "425 You're already connected.\r\n");
					continue;
				}
				if( Client->eDataMode == Mode_None  || !CFtpServerEx->OpenDataConnection(Client) ) {
					CFtpServerEx->SendReply( Client, "425 Can't open data connection.\r\n");
					continue;
				}

				pszPath = CFtpServerEx->BuildPath( Client->User->szStartDir, Client->szCurrentDir, pszCmdArg );
				if( pszPath && stat64( pszPath, &st ) == 0 && ! S_ISDIR( st.st_mode ) ) {
					Client->eStatus = Status_Downloading;
					strcpy( Client->CurrentTransfer.szPath, pszPath );
					Client->CurrentTransfer.Client = Client;
					CFtpServerEx->SendReply( Client, "150 Opening data connection.\r\n");
					#ifdef WIN32
						Client->CurrentTransfer.hTransferThread = (HANDLE) _beginthreadex( NULL, 0,
							CFtpServerEx->RetrieveThread, Client,
							0, &Client->CurrentTransfer.uTransferThreadId );
					#else
						pthread_create( &Client->CurrentTransfer.pTransferTreadID, NULL,
							CFtpServerEx->RetrieveThread, Client );
					#endif

				} else CFtpServerEx->SendReply( Client, "550 File not found.\r\n");

			} else CFtpServerEx->SendReply( Client, "501 Syntax error in arguments.\r\n");
			continue;

		} else if( !strcmp( szCmd, "STOR" ) || !strcmp( szCmd, "APPE" ) || !strcmp( szCmd, "STOU" ) ) {

			CFtpServerEx->STOR_Command( Client, pszCmdArg, szCmd );
			continue;

		} else if( !strcmp( szCmd, "SIZE" ) ) {

			if( pszCmdArg ) {
				pszPath = CFtpServerEx->BuildPath( Client->User->szStartDir, Client->szCurrentDir, pszCmdArg );
				if( pszPath && stat64( pszPath, &st ) == 0 ) {
					if( S_ISREG( st.st_mode ) && !S_ISDIR( st.st_mode ) ) {
						#ifdef WIN32
							CFtpServerEx->SendReply2( Client, "213 %I64i\r\n", st.st_size );
						#else
							CFtpServerEx->SendReply2( Client, "213 %lli\r\n", st.st_size );
						#endif
					} else {
						CFtpServerEx->SendReply2( Client, "550 %s: not a regular file.\r\n", pszCmdArg);
					}
				} else
					CFtpServerEx->SendReply( Client, "550 No such file.\r\n");
			} else
				CFtpServerEx->SendReply( Client, "501 Syntax error in arguments.\r\n");
			continue;

		} else if( !strcmp( szCmd, "DELE" ) ) {

			if( !(Client->User->ucPriv & CFtpServer::DELETEFILE) ) {
				CFtpServerEx->SendReply( Client, "550 Permission denied.\r\n");
				continue;
			}
			if( pszCmdArg ) {
				pszPath = CFtpServerEx->BuildPath( Client->User->szStartDir, Client->szCurrentDir, pszCmdArg );
				if( pszPath && stat64( pszPath, &st ) == 0 ) {
					if( remove( pszPath ) != -1 ) {
						CFtpServerEx->SendReply( Client, "250 DELE command successful.\r\n");
					} else
						CFtpServerEx->SendReply( Client, "550 Can' t Remove or Access Error.\r\n");
				} else
					CFtpServerEx->SendReply( Client, "550 No such file.\r\n");
			} else
				CFtpServerEx->SendReply( Client, "501 Syntax error in arguments.\r\n");
			continue;

		} else if( !strcmp( szCmd, "RNFR" ) ) {

			if( !(Client->User->ucPriv & CFtpServer::DELETEFILE) ) {
				CFtpServerEx->SendReply( Client, "550 Permission denied.\r\n");
				continue;
			}
			if( pszCmdArg ) {
				pszPath = CFtpServerEx->BuildPath( Client->User->szStartDir, Client->szCurrentDir, pszCmdArg );
				if( pszPath && stat64( pszPath, &st ) == 0 ) {
					Client->pszRenameFromPath = strdup( pszPath );
					CFtpServerEx->SendReply( Client, "350 File or directory exists, ready for destination name.\r\n");
				} else
					CFtpServerEx->SendReply( Client, "550 No such file or directory.\r\n");
			} else
				CFtpServerEx->SendReply( Client, "501 Syntax error in arguments.\r\n");
			continue;

		} else if( !strcmp( szCmd, "RNTO" ) ) {

			if( pszCmdArg ) {
				if( Client->pszRenameFromPath ) {
					pszPath = CFtpServerEx->BuildPath( Client->User->szStartDir, Client->szCurrentDir, pszCmdArg );
					if( pszPath && rename( Client->pszRenameFromPath, pszPath ) == 0 ) {
						CFtpServerEx->SendReply( Client, "250 Rename successful.\r\n");
					} else
						CFtpServerEx->SendReply( Client, "550 Invalid argument.\r\n");
					free( Client->pszRenameFromPath );
				} else
					CFtpServerEx->SendReply( Client, "503 Bad sequence of commands\r\n");
			} else
				CFtpServerEx->SendReply( Client, "501 Syntax error in arguments.\r\n");
			continue;

		} else if( !strcmp( szCmd, "MKD" ) || !strcmp( szCmd, "XMKD" ) ) {

			if( !(Client->User->ucPriv & CFtpServer::CREATEDIR) ) {
				CFtpServerEx->SendReply( Client, "550 Permission denied.\r\n");
				continue;
			}
			if( pszCmdArg ) {
				pszPath = CFtpServerEx->BuildPath( Client->User->szStartDir, Client->szCurrentDir, pszCmdArg );
				if( pszPath && stat64( pszPath, &st ) != 0 ) {
					#ifdef WIN32
						if( _mkdir( pszPath ) == -1 ) {
					#else
						if( mkdir( pszPath, 0777 ) ) {
					#endif
						CFtpServerEx->SendReply( Client, "550 MKD Error Creating DIR.\r\n");
					} else
						CFtpServerEx->SendReply( Client, "250 MKD command successful.\r\n");
				} else
					CFtpServerEx->SendReply( Client, "550 File Already Exists.\r\n");
			} else
				CFtpServerEx->SendReply( Client, "501 Syntax error in arguments.\r\n");
			continue;

		} else if( !strcmp( szCmd, "RMD" ) || !strcmp( szCmd, "XRMD" ) ) {

			if( !(Client->User->ucPriv & CFtpServer::DELETEDIR) ) {
				CFtpServerEx->SendReply( Client, "550 Permission denied.\r\n");
				continue;
			}
			if( pszCmdArg ) {
				pszPath = CFtpServerEx->BuildPath( Client->User->szStartDir, Client->szCurrentDir, pszCmdArg );
				if( pszPath && stat64( pszPath, &st ) == 0 ) {
					#ifdef WIN32
						if ( _rmdir( pszPath ) == -1 ) {
					#else
						if ( rmdir( pszPath ) == -1 ) {
					#endif
						CFtpServerEx->SendReply( Client, "450 Internal error deleting the directory.\r\n");
					} else
						CFtpServerEx->SendReply( Client, "250 Directory deleted successfully.\r\n");
				} else
					CFtpServerEx->SendReply( Client, "550 Directory not found.\r\n");
			} else
				CFtpServerEx->SendReply( Client, "501 Syntax error in arguments.\r\n");
			continue;

		} else {
			CFtpServerEx->SendReply( Client, "500 Command not understood.\r\n" );
			continue;
		}

	}

	CFtpServerEx->DelClient( CFtpServerEx, Client );

	#ifdef WIN32
		_endthreadex( 0 );
		return 0;
	#else
		pthread_exit( 0 );
		return NULL;
	#endif
}

/****************************************
 * NETWORK
 ***************************************/

bool CFtpServer::SendReply( struct CFtpServer::ClientNode *Client, const char *pszReply )
{
	if( pszReply && send(Client->CtrlSock, pszReply, strlen( pszReply ), 0) != SOCKET_ERROR )
		return true;
	return false;
}

bool CFtpServer::SendReply2( struct CFtpServer::ClientNode *Client, const char *pszList, ... )
{
	if( pszList && Client && Client->CtrlSock != INVALID_SOCKET ) {
        char *pszBuffer = new char[ Client->CFtpServerEx->iReplyMaxLen ];
        va_list args;
		va_start( args, pszList );
        _vsnprintf( pszBuffer, Client->CFtpServerEx->iReplyMaxLen, pszList , args);
		bool b = this->SendReply( Client, pszBuffer );
		delete [] pszBuffer;
		return b;
    }
    return false;
}

/****************************************
 * TRANSFER
 ***************************************/

struct CFtpServer::LIST_FindFileInfo *CFtpServer::LIST_FindFirstFile( char *szPath )
{
	if( szPath ) {

		struct LIST_FindFileInfo *fi = new struct LIST_FindFileInfo;
		memset( fi, 0x0, sizeof (struct LIST_FindFileInfo));
		strcpy( fi->szDirPath, szPath );

		#ifdef WIN32
			char* szPathTmp = new char[ strlen( szPath ) + 3 ];
			sprintf( szPathTmp, "%s/*", szPath );
			fi->hFindFile = FindFirstFile( szPathTmp, &fi->FileData );
			delete [] szPathTmp;
			if( fi->hFindFile != INVALID_HANDLE_VALUE ) {
				sprintf( fi->szFullPath, "%s/%s", fi->szDirPath, fi->FileData.cFileName );
				fi->pszName = fi->szFullPath + strlen( fi->szDirPath ) + 1;
				return fi;
			}
		#else
			if( (fi->dp = opendir( szPath )) != NULL )
				if( (fi->dir_entry = readdir64( fi->dp )) != NULL ) {
					sprintf( fi->szFullPath, "%s%s%s", fi->szDirPath, (!strcmp("/",fi->szDirPath)) ? "": "/",
						fi->dir_entry->d_name );
					fi->pszName = fi->szFullPath + strlen( fi->szDirPath );
					if( strcmp("/",fi->szDirPath) != 0 ) fi->pszName++;
					return fi;
				}
		#endif
		delete fi;
	}
	return NULL;
}

int CFtpServer::LIST_FindNextFile( struct CFtpServer::LIST_FindFileInfo *fi )
{
	if( fi ) {
		#ifdef WIN32
			int i = FindNextFile( fi->hFindFile, &fi->FileData );
			if( i ) {
				sprintf( fi->szFullPath, "%s/%s", fi->szDirPath, fi->FileData.cFileName );
				fi->pszName = fi->szFullPath + strlen( fi->szDirPath ) + 1;
				return i;
			}
		#else
			if( (fi->dir_entry = readdir64( fi->dp )) != NULL ) {
				sprintf( fi->szFullPath, "%s%s%s", fi->szDirPath, (!strcmp("/",fi->szDirPath)) ? "": "/",
					fi->dir_entry->d_name );
				fi->pszName = fi->szFullPath + strlen( fi->szDirPath );
				if( strcmp("/",fi->szDirPath) != 0 ) fi->pszName++;
				return true;
			}
		#endif
	}
	return false;
}

void CFtpServer::LIST_FindClose( struct CFtpServer::LIST_FindFileInfo *fi )
{
	if( fi ) {
		#ifdef WIN32
			FindClose( fi->hFindFile );
		#else
			closedir( fi->dp );
		#endif
		delete fi;
	}
	return;
}

bool CFtpServer::LIST_Command( struct CFtpServer::ClientNode *Client, char *pszCmdArg, const char *pszCmd )
{
	if( !Client ) return false;
	class CFtpServer *CFtpServerEx = Client->CFtpServerEx;

	if( !(Client->User->ucPriv & CFtpServer::LIST) ) {
		CFtpServerEx->SendReply( Client, "550 Permission denied.\r\n");
		return false;
	}
	if( Client->eStatus != Status_Waiting )	{
		CFtpServerEx->SendReply( Client, "425 You're already connected.\r\n");
		return false;
	}
	SOCKET *pSockListDir;

	if( strcmp( pszCmd, "STAT" ) != 0  ) {
		if( Client->eDataMode == Mode_None  || !CFtpServerEx->OpenDataConnection(Client) ) {
			CFtpServerEx->SendReply( Client, "425 Can't open data connection.\r\n");
			return false;
		} else
			CFtpServerEx->SendReply( Client, "150 Opening data connection for directory list.\r\n");
		pSockListDir = &Client->DataSock;
	} else {
		CFtpServerEx->SendReply( Client, "213-Status follows:\r\n");
		pSockListDir = &Client->CtrlSock;
	}

	Client->eStatus = Status_Listing;

	bool opt_a=false, opt_d=false, opt_F=false, opt_l=false;

	// Extract parametres
	char *pszArg = pszCmdArg;
	if( pszCmdArg ) {
		while( *pszArg == '-' ) {
			while ( pszArg++ && isalnum((unsigned char) *pszArg) ) {
				switch( *pszArg ) {
					case 'a':
						opt_a = true;
						break;
					case 'd':
						opt_d = true;
						break;
					case 'F':
						opt_F = true;
						break;
					case 'l':
						opt_l = true;
						break;
				}
			}
			while( isspace( (unsigned char) *pszArg ) )
				pszArg++;
		}
	}

	char *szPath = CFtpServerEx->BuildPath( Client->User->szStartDir, Client->szCurrentDir, pszArg );

	struct stat64 stDir;
	if( szPath && stat64( szPath, &stDir ) == 0 && S_ISDIR( stDir.st_mode ) ) { // file exist and is a Diretory

		struct tm *t;
		char szYearOrHour[25]="";
		char szBuf[ MAX_PATH+64+100+100 ]= "";

		if( opt_d ) {
			t = gmtime((time_t *) &stDir.st_mtime); // UTC Time
			if (time(NULL) - stDir.st_mtime > 180 * 24 * 60 * 60) {
				sprintf( szYearOrHour, "%5d", t->tm_year + 1900 );
			} else
				sprintf( szYearOrHour, "%02d:%02d", t->tm_hour, t->tm_min );
			#ifdef WIN32
				sprintf( szBuf, "%s 1 user group %14I64i %s %2d %s %s%s\r\n",
			#else
				sprintf( szBuf, "%s 1 user group %14lli %s %2d %s %s%s\r\n",
			#endif
				( S_ISDIR( stDir.st_mode ) ) ? "drwx------" : "-rwx------", stDir.st_size,
				CFtpServerEx->szMonth[ t->tm_mon ], t->tm_mday,	szYearOrHour,
				(!strcmp( pszCmd, "STAT" )) ? szPath + strlen( Client->User->szStartDir ) : ".",
				opt_F ? "/" : "" );
			send( *pSockListDir, szBuf, strlen( szBuf ), 0);
		} else {

			struct LIST_FindFileInfo *fi = LIST_FindFirstFile( szPath );

			if( fi != NULL ) {
				struct stat64 st;
				do
				{
					if( !strcmp( pszCmd, "NLST" ) ) {
						sprintf( szBuf, "%s\r\n", fi->szFullPath + strlen( Client->User->szStartDir ) + 1 );
						send( *pSockListDir, szBuf, strlen( szBuf ), 0);
					} else
					if( fi->pszName[0] != '.' || opt_a ) {

						stat64( fi->szFullPath, &st );

						t = gmtime((time_t *) &st.st_mtime); // UTC Time
						//t = localtime((time_t *) &st.st_mtime); // Local Time
						if (time(NULL) - st.st_mtime > 180 * 24 * 60 * 60) {
							sprintf( szYearOrHour, "%5d", t->tm_year + 1900 );
						} else
							sprintf( szYearOrHour, "%02d:%02d", t->tm_hour, t->tm_min );

						#ifdef WIN32
							sprintf( szBuf, "%s 1 user group %14I64i %s %2d %s %s%s\r\n",
						#else
							sprintf( szBuf, "%s 1 user group %14lld %s %2d %s %s%s\r\n",
						#endif
							( S_ISDIR( st.st_mode ) ? "drwx------" : "-rwx------" ),
							( S_ISDIR( st.st_mode ) ? (__int64)0 : st.st_size ),
							CFtpServerEx->szMonth[ t->tm_mon ], t->tm_mday, szYearOrHour,
							fi->pszName, (opt_F && S_ISDIR( st.st_mode )) ? "/" : "" );

						send( *pSockListDir, szBuf, strlen( szBuf ), 0);
					}

				} while( LIST_FindNextFile( fi ) );

				LIST_FindClose( fi );
			}
		}

	}

	if( strcmp( pszCmd, "STAT" ) ) {
		CFtpServerEx->SendReply( Client, "226 Transfer complete.\r\n");
		CFtpServerEx->CloseSocket( Client->DataSock );
		Client->DataSock = INVALID_SOCKET;
		Client->eDataMode = Mode_None;
	} else
		CFtpServerEx->SendReply( Client, "213 End of status\r\n");
	Client->eStatus = Status_Waiting;
	if( szPath != NULL ) delete [] szPath;
	return false;
}

bool CFtpServer::STOR_Command( struct CFtpServer::ClientNode *Client, char *pszCmdArg, const char *pszCmd )
{

	if( !Client ) return false;
	class CFtpServer *CFtpServerEx = Client->CFtpServerEx;

	if( !(Client->User->ucPriv & CFtpServer::WRITEFILE) ) {
		CFtpServerEx->SendReply( Client, "550 Permission denied.\r\n");
		return false;
	}

	if( pszCmdArg || (!strcmp( pszCmd, "STOU" )) ) {

		if( Client->eStatus != Status_Waiting )	{
			CFtpServerEx->SendReply( Client, "425 You're already connected.\r\n");
			return false;
		}
		if( Client->eDataMode == Mode_None  || !CFtpServerEx->OpenDataConnection(Client) ) {
			CFtpServerEx->SendReply( Client, "425 Can't open data connection.\r\n");
			return false;
		}

		Client->eStatus = Status_Uploading;
		struct stat64 st;

		char *szTempPath = NULL;
		if( !strcmp( pszCmd, "STOU" ) ) {
			char szFileUniqueName[20]="";
			do
			{
				if( szTempPath ) delete [] szTempPath;
				sprintf( szFileUniqueName, "file.%i", (rand()*99999,RAND_MAX) );
				szTempPath = CFtpServerEx->BuildPath( Client->User->szStartDir, Client->szCurrentDir, szFileUniqueName );
			} while( stat64( szTempPath, &st ) == 0 );
		}

		char *szPath;
		if( szTempPath ) {
			szPath = szTempPath;
		} else 
			szPath = CFtpServerEx->BuildPath( Client->User->szStartDir, Client->szCurrentDir, pszCmdArg );

		if( szPath ) {
			//if( eCmd == CFtpServer::Cmd_STOR )
			//if( eCmd == CFtpServer::Cmd_STOU )
			if( !strcmp( pszCmd, "APPE" ) ) {
				if( stat64( szPath, &st ) == 0 ) {
					Client->CurrentTransfer.i64RestartAt = st.st_size;
				} else
					Client->CurrentTransfer.i64RestartAt = (__int64)0;
			}
			strcpy( Client->CurrentTransfer.szPath, szPath );
			Client->CurrentTransfer.Client = Client;
			if( !strcmp( pszCmd, "STOU" ) ) {
				CFtpServerEx->SendReply2( Client, "150 FILE: %s\r\n", szPath + strlen( Client->User->szStartDir ) );
			} else
				CFtpServerEx->SendReply( Client, "150 Opening data connection.\r\n");
			delete [] szPath;
			
			#ifdef WIN32
				Client->CurrentTransfer.hTransferThread = (HANDLE) _beginthreadex( NULL, 0,
					CFtpServerEx->StoreThread, Client,
					0, &Client->CurrentTransfer.uTransferThreadId );
			#else
				pthread_create( &Client->CurrentTransfer.pTransferTreadID, NULL,
					CFtpServerEx->StoreThread, Client );
			#endif
			return true;
		}

	} else CFtpServerEx->SendReply( Client, "501 Syntax error in arguments.\r\n");

	return false;
}

bool CFtpServer::OpenDataConnection( struct CFtpServer::ClientNode *Client )
{
	if( Client->eDataMode == Mode_PASV ) {

		SOCKET sTmpWs;
		struct sockaddr_in sin;
		socklen_t sin_len = sizeof( struct sockaddr_in );

		sTmpWs = accept( Client->DataSock, (struct sockaddr *) &sin, &sin_len);
		this->CloseSocket( Client->DataSock );

		Client->DataSock = sTmpWs;
		if( sTmpWs == INVALID_SOCKET )
				return false;
		return true;

	} else if( Client->eDataMode == Mode_PORT ) {

		Client->DataSock = socket( AF_INET, SOCK_STREAM, IPPROTO_TCP );
		if( Client->DataSock == INVALID_SOCKET )
			return false;

		struct sockaddr_in BindSin;
		BindSin.sin_family = AF_INET;
		BindSin.sin_port = htons( (unsigned short) ((float) rand() / RAND_MAX * (this->DataPortRange.iNumber-1) + this->DataPortRange.usStart ) );
		BindSin.sin_addr.s_addr = Client->ulServerIP;

		int on = 1; // Here the strange behaviour of SO_REUSEPORT under win32 is welcome.
		#ifdef SO_REUSEPORT
			(void) setsockopt( Client->DataSock, SOL_SOCKET, SO_REUSEPORT, (char *) &on, sizeof on);
		#else
			(void) setsockopt( Client->DataSock, SOL_SOCKET, SO_REUSEADDR, (char *) &on, sizeof on);
		#endif

		if( bind( Client->DataSock, (struct sockaddr *) &BindSin, sizeof( struct sockaddr_in ) ) == SOCKET_ERROR )
			return false;

		struct sockaddr_in ConnectSin;
		ConnectSin.sin_family = AF_INET;
		ConnectSin.sin_port = htons( Client->usDataPort );
		ConnectSin.sin_addr.s_addr = Client->ulClientIP;

		if( connect( Client->DataSock, (struct sockaddr *) &ConnectSin, sizeof(struct sockaddr_in) ) != SOCKET_ERROR )
			return true;
	}
	return false;
}

#ifdef WIN32
	unsigned CFtpServer::StoreThread( void *pvParam )
#else
	void* CFtpServer::StoreThread( void *pvParam )
#endif
{
	struct CFtpServer::ClientNode *Client = (struct CFtpServer::ClientNode*) pvParam;
	if( Client ) {

		FILE *File;
		long DataRecv=0;
		class CFtpServer *CFtpServerEx = Client->CFtpServerEx;

		char szOpenMode[3]; int len=0;
		szOpenMode[1]='b'; szOpenMode[2]='\0';
		if( Client->CurrentTransfer.i64RestartAt > 0 ) {
			szOpenMode[0]='a';
		} else szOpenMode[0]='w';

		if ( (File=fopen64( Client->CurrentTransfer.szPath , szOpenMode) ) != NULL ) {
			if( (Client->CurrentTransfer.i64RestartAt > 0
				 && fseeko64( File, Client->CurrentTransfer.i64RestartAt, 0) == 0 )
				|| (Client->CurrentTransfer.i64RestartAt == 0) )
			{
				char Buffer[4*1024]="";
				while( Client->DataSock != INVALID_SOCKET && len>=0 )
				{
					len = recv( Client->DataSock, Buffer, sizeof(Buffer), 0 );
					if( len > 0 ) {
						fwrite( Buffer , sizeof(char) , len , File);
						DataRecv += len;
					} else
						break;
				}
			}
		}
		if( File ) fclose( File );
		CFtpServerEx->CloseSocket( Client->DataSock );
		Client->DataSock = INVALID_SOCKET;
		if( Client->CtrlSock != INVALID_SOCKET ) {
			if( len == 0 ) {
				CFtpServerEx->SendReply( Client, "226 Transfer complete.\r\n");
			} else {
				CFtpServerEx->SendReply( Client, "550 Can 't receive file.\r\n");
				remove( Client->CurrentTransfer.szPath );
			}
		}
		memset( &Client->CurrentTransfer, 0x0, sizeof( Client->CurrentTransfer ) );
		Client->eStatus = Status_Waiting;
	}
	#ifdef WIN32
		_endthreadex( 0 );
		return 0;
	#else
		pthread_exit( 0 );
	#endif
}

#ifdef WIN32
	unsigned CFtpServer::RetrieveThread( void *pvParam)
#else
	void* CFtpServer::RetrieveThread( void *pvParam)
#endif
{
	struct CFtpServer::ClientNode *Client = (struct CFtpServer::ClientNode*) pvParam;
	if( Client ) {

        FILE *File; int BlockSize; int len=0;
		class CFtpServer *CFtpServerEx = Client->CFtpServerEx;

        if ( (File=fopen64( Client->CurrentTransfer.szPath , "rb") ) != NULL ) {
			if( (Client->CurrentTransfer.i64RestartAt > 0
				 && fseeko64( File, Client->CurrentTransfer.i64RestartAt, 0) == 0)
				|| (Client->CurrentTransfer.i64RestartAt == 0) )
			{
				char Buffer[4*1024];
				while( Client->DataSock != INVALID_SOCKET &&
					(BlockSize = fread( Buffer , sizeof(char) , sizeof(Buffer) , File)) != 0 ) {
					len = send( Client->DataSock, Buffer, BlockSize, 0);
					if( len <= 0 ) break;
				}
			}
		}
		if( File ) fclose( File );
		CFtpServerEx->CloseSocket( Client->DataSock );
		Client->DataSock = INVALID_SOCKET;
		if( Client->CtrlSock != INVALID_SOCKET ) {
			if( len >= 0 ) {
				CFtpServerEx->SendReply( Client, "226 Transfer complete.\r\n");
			} else
				CFtpServerEx->SendReply( Client, "550 Can 't Send File.\r\n");
		}
		memset( &Client->CurrentTransfer, 0x0, sizeof( Client->CurrentTransfer ) );
		Client->eStatus = Status_Waiting;
	}
	#ifdef WIN32
		_endthreadex( 0 );
		return 0;
	#else
		pthread_exit( 0 );
	#endif
}

/****************************************
 * FILE
 ***************************************/

char* CFtpServer::BuildPath( const char *szStartPath, const char *szCurrentPath, char *szAskedPath )
{
	if( szStartPath && szCurrentPath ) {
		int iPathMaxLen; char* szPath = NULL;
		if( !szAskedPath || szAskedPath[0] == '\0'  ) {
			iPathMaxLen = strlen( szStartPath ) + strlen( szCurrentPath ) + strlen("//*.*") + 1;
			szPath = new char[ iPathMaxLen ];
			sprintf(szPath, "%s%s", szStartPath, szCurrentPath );
		} else {
			CFtpServer::SimplifyPath( szAskedPath );
			if( szAskedPath[0]=='/' ) {
				iPathMaxLen = strlen( szStartPath ) + strlen( szAskedPath ) + strlen("//*.*") + 1;
				szPath=new char[iPathMaxLen];
				sprintf(szPath, "%s%s", (!strcmp( "/", szStartPath ) ) ? "" : szStartPath, szAskedPath );
			} else {
				iPathMaxLen = strlen( szStartPath ) + strlen( szCurrentPath ) + strlen( szAskedPath ) + strlen("///*.*") + 1;
				szPath=new char[iPathMaxLen];
				sprintf(szPath, "%s%s/%s", szStartPath,
					(strlen(szCurrentPath)>1)?szCurrentPath:"",
					szAskedPath );
			}
		}
		int szPathLen = strlen( szPath );
		if( szPathLen > 1 && szPath[ szPathLen -1 ] == '/' ) { 
			szPath[ strlen( szPath ) -1 ] = '\0';
			szPathLen += -1;
		}
		if( szPathLen <= MAX_PATH ) {
			return szPath;
		} else {
			delete [] szPath;
		}
	}
	return NULL;
}

bool CFtpServer::SimplifyPath( char *pszPath )
{
	if( !pszPath || *pszPath == 0 )
		return false;

	char *a;

	for( a = pszPath; *a != '\0'; ++a )
		if( *a == '\\' ) *a = '/';
	while( ( a = strstr( pszPath, "//") ) != NULL )
		memcpy( a, a + 1 , strlen( a ) );
	while( ( a = strstr( pszPath, "/./" ) ) != NULL )
		memcpy( a, a + 2, strlen( a + 1 ) );
	if( !strncmp( pszPath, "./", 2 ) )
		memcpy( pszPath, pszPath + 1, strlen( pszPath ) );
	if( !strncmp( pszPath, "../", 3) )
		memcpy( pszPath, pszPath + 2, strlen( pszPath + 1 ) );

	a = pszPath;
	while( (a = strstr( a, "/..")) ) {
		char *b = a;
		if( a[3] == '/' ) {
			if( a == pszPath ) {
				memcpy( pszPath, pszPath + 3, strlen( pszPath + 3 ) );
			} else {
				while( b != pszPath ) {
					b--;
					if( *b == '/' ) {
						memcpy( b, a + 3, strlen( a + 2 ) );
						break;
					}
				}
				if( b == pszPath )
					memcpy( pszPath, a + 3, strlen( a + 2 ) );
			}
		} else if( a[3] == '\0' ) {
			a[0] = '\0';
		} else
			a++;
	}
	return true;
}

/****************************************
 * USER
 ***************************************/

struct CFtpServer::UserNode *CFtpServer::AddUser(char *szName,char *szPasswd, char *szStartDir)
{
	if( szStartDir && szName && ( szPasswd || !strcasecmp( "anonymous", szName ) ) ) {

		if( this->SearchUserFromName( szName ) != NULL )
			return NULL; // User's name already exists.

		char *szSDEx = strdup( szStartDir );

		this->SimplifyPath( szSDEx );

		// Verify that the StartPath exist
		struct stat64 st;
		if( stat64( szSDEx, &st ) != 0 ) {
			if( szSDEx[ strlen( szSDEx )-1 ] == '/' ) {
				szSDEx[ strlen( szSDEx )-1 ] = '\0';
				if( stat64( szSDEx, &st ) != 0 ) {
					free( szSDEx );
					return NULL;
				}
			} else {
				free( szSDEx );
				return NULL;
			}
		}
		struct CFtpServer::UserNode *NewUser = new struct CFtpServer::UserNode;
		memset( NewUser, 0x0, sizeof( struct CFtpServer::UserNode ) );

		NewUser->szName = new char[ strlen( szName )+1 ];
		strcpy( NewUser->szName, szName );

		if( szPasswd ) {
			NewUser->szPasswd = new char[ strlen( szPasswd )+1 ];
			strcpy( NewUser->szPasswd, szPasswd );
		}

		NewUser->szStartDir = new char[ strlen(szStartDir)+1 ];
		strcpy( NewUser->szStartDir, szSDEx );

		if( this->UserListLast == NULL ) {
			this->UserListHead = this->UserListLast = NewUser;
			NewUser->PrevUser = NULL;
			NewUser->NextUser = NULL;
		} else {
			NewUser->PrevUser = this->UserListLast;
			this->UserListLast->NextUser = NewUser;
			this->UserListLast = NewUser;
			NewUser->NextUser = NULL;
		}

		NewUser->bIsEnabled = true;
		this->iNbUser += 1;

		return NewUser;
	}
	return NULL;
}

#ifdef CFTPSERVER_USE_EXTRACMD
bool CFtpServer::SetUserExtraCmd( struct CFtpServer::UserNode *User, unsigned char ucExtraCmd )
{
	if( User && (ucExtraCmd & ~(ExtraCmd_EXEC)) == 0) {
		User->ucExtraCmd = ucExtraCmd;
		return true;
	}
	return false;
}
#endif

bool CFtpServer::SetUserMaxClient( struct CFtpServer::UserNode *User, int iMaxClient )
{
	if( User && iMaxClient >= -1 ) {
		User->iMaxClient = iMaxClient;
		return true;
	}
	return false;

}

bool CFtpServer::SetUserPriv( struct CFtpServer::UserNode *User, unsigned char ucPriv )
{
	if( User && (ucPriv & ~(READFILE|WRITEFILE|DELETEFILE|LIST|CREATEDIR|DELETEDIR)) == 0) {
		User->ucPriv = ucPriv;
		return true;
	}
	return false;
}

bool CFtpServer::DelUser(struct CFtpServer::UserNode *User)
{
	if( User != NULL ) {

		User->bIsEnabled = false;
		if( User->iNbClient > 0 ) {
			struct CFtpServer::ClientNode *Client = this->ClientListHead;
			while ( Client && User->iNbClient > 0 ) {
				if( Client->bIsLogged && Client->User == User )
					DelClient( this, Client );
				Client = Client->NextClient;
			}
		}

		if( this->UserListHead == User ) {
			if( this->UserListHead = User->NextUser )
				this->UserListHead->PrevUser=NULL;
		}
		if( this->UserListLast == User ) {
			if( this->UserListLast = User->PrevUser )
				this->UserListLast->NextUser=NULL;
		}
		if( User->szStartDir != NULL ) delete [] User->szStartDir;
		if( User->szName != NULL ) delete [] User->szName;
		if( User->szPasswd != NULL ) delete [] User->szPasswd;
		delete User;
		this->iNbUser+=-1;
		return true;
	}
	return false;
}

struct CFtpServer::UserNode * CFtpServer::SearchUserFromName( char *szName )
{
	if( szName ) {
		struct CFtpServer::UserNode *SearchUser=UserListHead;
		while( SearchUser ) {
			if( SearchUser->szName && !strcmp(szName, SearchUser->szName) )
				return SearchUser;
			SearchUser=SearchUser->NextUser;
		}
	}
	return NULL;
}

void CFtpServer::DelAllUser( void )
{
	struct CFtpServer::UserNode *User = this->UserListHead;
	struct CFtpServer::UserNode *UserNext = NULL;
	while( User ) {
		UserNext = User->NextUser;
		this->DelUser( User );
		User = UserNext;
	}
	return;
}


/****************************************
 * CLIENT
 ***************************************/

struct CFtpServer::ClientNode *CFtpServer::AddClient( CFtpServer *CFtpServerEx, SOCKET Sock, struct sockaddr_in *Sin )
{
	if( Sock != INVALID_SOCKET && &Sin && CFtpServerEx ) {

		struct CFtpServer::ClientNode *NewClient = new struct CFtpServer::ClientNode;
		memset( NewClient, 0x0, sizeof(struct CFtpServer::ClientNode) );
		
		NewClient->CtrlSock = Sock;
		NewClient->DataSock = INVALID_SOCKET;

		struct sockaddr ServerSin;
		socklen_t Server_Sin_Len = sizeof( ServerSin );
		getsockname( Sock, &ServerSin, &Server_Sin_Len );
 		NewClient->ulServerIP = (((struct sockaddr_in *) &(ServerSin))->sin_addr).s_addr;
		NewClient->ulClientIP = (((struct sockaddr_in *) (Sin))->sin_addr).s_addr;

		NewClient->CFtpServerEx = CFtpServerEx;

		strcpy(NewClient->szCurrentDir,"/");

		if( CFtpServerEx->ClientListLast == NULL ) {
			CFtpServerEx->ClientListHead=CFtpServerEx->ClientListLast=NewClient;
			NewClient->PrevClient=NULL;
			NewClient->NextClient=NULL;
		} else {
			CFtpServerEx->ClientListLast->NextClient = NewClient;
			NewClient->PrevClient = CFtpServerEx->ClientListLast;
			NewClient->NextClient = NULL;
			CFtpServerEx->ClientListLast = NewClient;
		}

		CFtpServerEx->iNbClient += 1;

		return NewClient;
	}
	return NULL;
}

bool CFtpServer::DelClient(class CFtpServer *CFtpServerEx, struct CFtpServer::ClientNode * Client)
{
	if( Client != NULL ) {
		if( this->ClientListHead==Client ) {
			if( this->ClientListHead=Client->NextClient )
				this->ClientListHead->PrevClient=NULL;
		}
		if( this->ClientListLast==Client ) {
			if( this->ClientListLast=Client->PrevClient )
				this->ClientListLast->NextClient=NULL;
		}

		CFtpServerEx->CloseSocket( Client->CtrlSock );
		CFtpServerEx->CloseSocket( Client->DataSock );

		Client->CtrlSock = INVALID_SOCKET;
		Client->DataSock = INVALID_SOCKET;
		// the threads should now end up

		if( Client->eStatus == CFtpServer::Status_Uploading
			|| Client->eStatus == CFtpServer::Status_Downloading ) {
			#ifdef WIN32
				WaitForSingleObject( Client->CurrentTransfer.hTransferThread, INFINITE );
				CloseHandle( Client->CurrentTransfer.hTransferThread );
			#else
				pthread_join( Client->CurrentTransfer.pTransferTreadID, NULL );
			#endif
		}

		#ifdef WIN32
			CloseHandle( Client->hClientThread );
		#else
			pthread_detach( Client->pClientThreadID );
		#endif

		if( Client->User )
			Client->User->iNbClient += -1;
		if( Client->pszRenameFromPath ) free( Client->pszRenameFromPath );
		delete Client;
		if( CFtpServerEx ) CFtpServerEx->iNbClient+=-1;
		return true;
	}
	return false;
}

void CFtpServer::DelAllClient( void )
{
	struct CFtpServer::ClientNode *Client = this->ClientListHead;
	struct CFtpServer::ClientNode *ClientNext = NULL;
	while( Client ) {
		ClientNext = Client->NextClient;
		this->DelClient( this, Client );
		Client = ClientNext;
	}
	return;
}
