
#include "ftpserver.h"


int thread(void *ptclass)
{

 // on recupere l'adresse de notre classe
 ftpserver *pt = static_cast<ftpserver *>(ptclass);

    // on verifie que des gens ont le droit de se connecter au serveur FTP
	if( pt->FtpUser ) 
    {

		printf("- FtpUser successfuly created ! :)\r\n");

		pt->FtpServer.SetUserMaxClient( pt->FtpUser, CFtpServer::Unlimited );

		pt->FtpServer.SetUserPriv( pt->FtpUser, CFtpServer::READFILE | CFtpServer::WRITEFILE | CFtpServer::LIST | CFtpServer::DELETEFILE | CFtpServer::CREATEDIR | CFtpServer::DELETEDIR );
		
		#ifdef CFTPSERVER_USE_EXTRACMD // See CFtpServer.h, not defined by default
			pt->FtpServer.SetUserExtraCmd( FtpUser, CFtpServer::ExtraCmd_EXEC );
			// Security Warning ! Only here for example.
			// the last command allow the user to call the 'system()' C function!
		#endif
	} 
    else 
    {
		printf("- Unable to create FtpUser.\r\n");
        std::string message;
        message = "+connection(\"all\"): \"-Unable to create FtpUser\";";
        urbi::send(message.c_str());
        return 1;
    }



    // on verifie que le serveur est capable d'ecouter
    if( pt->FtpServer.StartListening() ) 
    {
        printf("- Server is listening ! :)\r\n");
    } 
    else
    {
        printf("- Unable to listen.\r\n");
        std::string message;
        message = "+connection(\"all\"): \"-Unable to listen.\";";
        urbi::send(message.c_str());
        return 1;
    } 



    // on verifie que le serveur peut accepter des connexions
    if( pt->FtpServer.StartAccepting() ) 
    {
		printf("- Server successfuly started ! :)\r\n");

		while( pt->valwhile )
		{
					#ifdef WIN32
						Sleep( 1000 );
					#else
						usleep( 1000 );
					#endif
		}

		pt->FtpServer.StopListening();
    } 
    else
    {
		printf("- Unable to accept incoming connections.\r\n");
        std::string message;
        message = "+connection(\"all\"): \"-Unable to accept incoming connections.\";";
        urbi::send(message.c_str());
        return 1;
    } 



     return 1;
}


UStart(ftpserver);

// Creation de l'objet ftpserver 
ftpserver::ftpserver(std::string s) : UObject(s) 
{
	UBindFunction(ftpserver,init);
	UBindFunction(ftpserver, change_port);
	UBindFunction(ftpserver, allow_anonymous);
	UBindFunction(ftpserver, add_user);
	UBindFunction(ftpserver,start);
	UBindFunction(ftpserver,stop);


	#ifdef WIN32
		if( WSAStartup( MAKEWORD(2, 0), &WSAData) != 0 ) 
        {
			printf("- WSAStartup failure: WSAGetLastError=%d\r\n", WSAGetLastError() );
        	std::string message;
            message = "+connection(\"all\"): \"-WSAStartup failure: WSAGetLastError\";";
            urbi::send(message.c_str());
			return;
		}
	#endif

	FtpServer.SetPort(21); // default FTP port is 21
	FtpServer.SetDataPortRange( 100,100 ); // data TCP-Port range = [100-199]

	#ifdef CFTPSERVER_ANTIBRUTEFORCING // See CFtpServer.h, not defined by default
		FtpServer.SetCheckPassDelay( 1500 ); // kind of bruteforcing protection
	#endif

    t1 = NULL;
    valwhile=1;

}


// Destruction de l'objet ftpserver
ftpserver::~ftpserver(void)
{

}

// empecher que l'on puisse faire un new ftpserver()
int ftpserver::init(void)
{
  return 1; // Singleton
}

// changer le port de connexion au serveur FTP
void ftpserver::change_port(int nb_port)
{
	FtpServer.SetPort(nb_port); 
}

// autoriser ou non les connexions anonymes
void ftpserver::allow_anonymous(int allow)
{
    if(allow == 0)
    {
    	FtpServer.AllowAnonymous( false, "." ); 
		printf("- anonymous users are not allow ! \r\n");
    }
    else if(allow == 1)
    {
    	if( FtpServer.AllowAnonymous( true, "." ) == false ) 
        { // allow anonymous user
    		printf("- Unable to create anonymous user; Make sure its StartPath exists!\r\n");
        	std::string message;
            message = "+connection(\"all\"): \"-WSAStartup failure: WSAGetLastError\";";
            urbi::send(message.c_str());
    	}
        else
        	printf("- anonymous users are allow ! \r\n");
    }

}

// ajouter des utilisateurs pouvant se connecter au serveur
void ftpserver::add_user(std::string name, std::string pass)
{
	#ifdef WIN32
		FtpUser = FtpServer.AddUser( (char*)name.c_str(), (char*)pass.c_str(), ".");
	#else
		FtpUser = FtpServer.AddUser( (char*)name.c_str(), (char*)pass.c_str(), ".");
	#endif
    printf("- new user is allow ! \r\n");
}

// on demarre le serveur FTP
void ftpserver::start(void)
{
    // creation du thread qui met a jour les valeurs

    if(t1 == NULL)
    {
       valwhile = 1;
       t1 = SDL_CreateThread(thread, (void*) this);
    }
}

// on arrete le serveur FTP
void ftpserver::stop(void)
{
    // arret du thread qui met a jour les valeurs
    if(t1 != NULL)
    {
        valwhile = 0;
        SDL_KillThread (t1);
        t1 = NULL;
  		FtpServer.StopListening();
        printf("- FTP Server stop ! \r\n");
    }


}



