#include "joystick-win.h"

UStart(joystick);

// Creation de l'objet joystick
joystick::joystick(std::string s) : UObject(s) 
{
	UBindFunction(joystick,init);
	UBindFunction(joystick,info);
	UBindFunction(joystick,updatejoy);
	UBindFunction(joystick,getbutton);
	UBindFunction(joystick,getaxis);
	UBindFunction(joystick,gethat);
	UBindFunction(joystick,getballx);
	UBindFunction(joystick,getbally);

	// initialisation de toutes les variables
	joystick_ = NULL;
	nb_buttons = 0;
	nb_axis = 0;
	nb_hats = 0;
	nb_balls = 0;
}


// Destruction de l'objet joystick 
joystick::~joystick(void)
{
    // arret de la boucle loop qui lance updatejoy()
	std::string message = "stop tagjoystick" + _name + ";";
	urbi::send(message.c_str());

	SDL_JoystickClose(joystick_);
	SDL_QuitSubSystem(SDL_INIT_VIDEO|SDL_INIT_JOYSTICK);
    atexit(SDL_Quit); // fin de SDL
}

// creation de new joystick()
int joystick::init(int choix)
{
  int result=0; // permet de faire des tests pour voir si tout est ok

	// Initialisation de SDL 
	result = SDL_Init(SDL_INIT_VIDEO|SDL_INIT_JOYSTICK);
    if ( result < 0 ) // mauvaise initialisation, donc on quitte
    {
		std::string message = "+connection(\"all\"):\"Error. Couldn't initialize SDL\";";
		urbi::send(message.c_str());
		fprintf(stderr, "Couldn't initialize SDL: %s\n",SDL_GetError());
		exit(1);
	}

    // on recupere le nombre de joysticks
	int nb_joy = SDL_NumJoysticks();
 	if(choix <0 || choix >= nb_joy || nb_joy <= 0)
	{
		std::string message = "+connection(\"all\"):\"Error. Joystick doesn't found\";";
		urbi::send(message.c_str());
		SDL_QuitSubSystem(SDL_INIT_JOYSTICK);
		atexit(SDL_Quit); // fin de SDL
	    return 1; // error
	}

    // ouverture du joystick choisi
	joystick_ = SDL_JoystickOpen(choix);

    // si joystick_ est null, alors il y a une erreur et on quitte
	if ( joystick_ == NULL ) 
    {
			std::string message = "+connection(\"all\"):\"Error. Couldn't open joystick\";";
			urbi::send(message.c_str());
			SDL_QuitSubSystem(SDL_INIT_JOYSTICK);
			atexit(SDL_Quit); // fin de SDL
			return 1; // error
	} 


    // on verifie que le joystick a bien ete ouvert
   if(SDL_JoystickOpened (choix) == 0)
   {
			std::string message = "+connection(\"all\"):\"Error. Couldn't open joystick\";";
			urbi::send(message.c_str());
			SDL_QuitSubSystem(SDL_INIT_JOYSTICK);
			atexit(SDL_Quit); // fin de SDL
			return 1; // error
   }


   // on recupere le nom du joystick
	const char *name;
	name = SDL_JoystickName(SDL_JoystickIndex(joystick_));

    // on affiche des infos au sujet du joystick
	printf("Watching joystick %d: (%s)\n", SDL_JoystickIndex(joystick_), name ? name : "Unknown Joystick");
	nb_buttons = SDL_JoystickNumButtons(joystick_); 
	nb_axis = SDL_JoystickNumAxes(joystick_);
	nb_hats = SDL_JoystickNumHats(joystick_);
	nb_balls = SDL_JoystickNumBalls(joystick_);
	printf("Joystick has %d axes, %d hats, %d balls, and %d buttons\n", nb_axis , nb_hats , nb_balls , nb_buttons);


    // on demande qu'a chaque changement de la valeur du joystick, une mise a jour soit effectuee.
    SDL_JoystickEventState(SDL_ENABLE); 
	SDL_Event event; // pour gerer les evenements

	// on recupere le nom du UObject et on met a jour les valeurs du joystick
    _name = __name; // _name prend le nom du UObject
    std::string message = "tagjoystick" + _name + " : " + _name + ".updatejoy(\"" + _name + "\") ,";
	urbi::send(message.c_str());

    return 0; // plusieurs creations possibles
}


// donne la valeur d'un bouton
int joystick::getbutton(int choix)
{
    if( choix >= SDL_JoystickNumButtons(joystick_) || choix < 0 )
    {
//	std::string message = "+connection(\"all\"):\"Error. No button\";";
//	urbi::send(message.c_str());
    return -1;
    }

    // mise a jour de toutes les valeurs du joystick ouvert
//    SDL_JoystickUpdate (); 

    int etat = SDL_JoystickGetButton(joystick_,choix); 
    return etat;   
}

// donne la valeur d'un axe
int joystick::getaxis(int choix)
{

    if( choix >= SDL_JoystickNumAxes(joystick_) || choix < 0 )
    {
//	std::string message = "+connection(\"all\"):\"Error. No axis\";";
//	urbi::send(message.c_str());
    return -1;
    }

    // mise a jour de toutes les valeurs du joystick ouvert
//    SDL_JoystickUpdate (); 

    int etat = SDL_JoystickGetAxis(joystick_,choix); 
    return etat;   
}

// donne la valeur d'un chapeau
int joystick::gethat(int choix)
{

    if( choix >= SDL_JoystickNumHats(joystick_) || choix < 0 )
    {
//	std::string message = "+connection(\"all\"):\"Error. No axis\";";
//	urbi::send(message.c_str());
    return -1;
    }

    // mise a jour de toutes les valeurs du joystick ouvert
//    SDL_JoystickUpdate (); 

    int etat = SDL_JoystickGetHat(joystick_,choix); 
    return etat;   
}

// donne la valeur x d'une balle
int joystick::getballx(int choix)
{

    if( choix >= SDL_JoystickNumBalls(joystick_) || choix < 0 )
    {
//	std::string message = "+connection(\"all\"):\"Error. No ball\";";
//	urbi::send(message.c_str());
    return -1;
    }

    // mise a jour de toutes les valeurs du joystick ouvert
//    SDL_JoystickUpdate (); 

    int dx; int dy;
    int etat = SDL_JoystickGetBall(joystick_,choix,&dx,&dy); 
    return dx;   
}

// donne la valeur y d'une balle
int joystick::getbally(int choix)
{

    if( choix >= SDL_JoystickNumBalls(joystick_) || choix < 0 )
    {
//	std::string message = "+connection(\"all\"):\"Error. No ball\";";
//	urbi::send(message.c_str());
    return -1;
    }

    // mise a jour de toutes les valeurs du joystick ouvert
//    SDL_JoystickUpdate (); 

    int dx; int dy;
    int etat = SDL_JoystickGetBall(joystick_,choix,&dx,&dy); 
    return dy;   
}


// actualisation des valeurs du joystick
void joystick::updatejoy(std::string _name)
{

// SDL_WaitEvent(&event); ==> fige le programme jusqu'a ce qu'un evenement se produise

    // mise a jour de toutes les valeurs du joystick ouvert
    SDL_JoystickUpdate (); 

    int i=0;
    int value =0 ;
	std::string message = "";

    for(i=0; i<nb_buttons;i++)
    {
       std::ostringstream ostr; 
       value = SDL_JoystickGetButton(joystick_,i);
       ostr << value; 
       std::ostringstream ostr2; 
       ostr2 << i; 
       message = message + _name + ".button" + ostr2.str() + " = " + ostr.str() + " & ";
    }

    for(i=0; i<nb_axis;i++)
    {
       std::ostringstream ostr; 
       value = SDL_JoystickGetAxis(joystick_,i);
       ostr << value; 
       std::ostringstream ostr2; 
       ostr2 << i; 
       message = message + _name + ".axis" + ostr2.str() + " = " + ostr.str() + " & ";
    }

    for(i=0; i<nb_hats;i++)
    {
       std::ostringstream ostr; 
       value = SDL_JoystickGetHat(joystick_,i);
       ostr << value; 
       std::ostringstream ostr2; 
       ostr2 << i; 
       message = message + _name + ".hat" + ostr2.str() + " = " + ostr.str() + " & ";
    }

    for(i=0; i<nb_balls;i++)
    {
       int dx, dy;
       value = SDL_JoystickGetBall(joystick_,i,&dx,&dy); 
       std::ostringstream ostr; 
       value = SDL_JoystickGetHat(joystick_,i);
       ostr << dx; 
       std::ostringstream ostr3; 
       value = SDL_JoystickGetHat(joystick_,i);
       ostr3 << dy; 
       std::ostringstream ostr2; 
       ostr2 << i; 
       message = message + _name + ".hatx" + ostr2.str() + " = " + ostr.str() + " & " + _name + ".haty" + ostr2.str() + " = " + ostr3.str() + " & ";
    }

    message = message + "tagjoystick" + _name + " : " + _name + ".updatejoy(\"" + _name + "\")"  + ",";
    urbi::send(message.c_str());             
}


// donne des infos sur les joysticks presents sur l'ordi
void joystick::info(void)
{
	const char *name;
	int i;
	SDL_Joystick *joystick;
	// Initialize SDL 
	if ( SDL_Init(SDL_INIT_JOYSTICK) < 0 ) 
    {
		std::string message = "+connection(\"all\"):\"Error. Couldn't initialize SDL\";";
		urbi::send(message.c_str());
		fprintf(stderr, "Couldn't initialize SDL: %s\n",SDL_GetError());
		exit(1);
	}
	// Print information about the joysticks 
	int nb_joy = SDL_NumJoysticks();
	std::string message = "+connection(\"all\"):\"There are ";
	std::string messagetmp;
    std::ostringstream ostr; 
    ostr << nb_joy; 
    messagetmp = ostr.str(); 
    message = message + messagetmp;
	message = message + " joysticks attached\";";
	urbi::send(message.c_str());
	for ( i=0; i<SDL_NumJoysticks(); ++i ) 
    {
		name = SDL_JoystickName(i);
	
		message = "+connection(\"all\"):\"Joystick ";
		std::ostringstream ostr2; 
		ostr2 << i; 
		messagetmp = ostr2.str(); 
		message = message + messagetmp;
		message = message + " : ";
		message = message + name;
		message = message + "\";";
		urbi::send(message.c_str());
		joystick = SDL_JoystickOpen(i);
		if (joystick == NULL) 
        {
			fprintf(stderr, "SDL_JoystickOpen(%d) failed: %s\n", i, SDL_GetError());
			message = "+connection(\"all\"):\"Error. SDL_JoystickOpen() failed\";";
			urbi::send(message.c_str());
		} 
        else 
        {
			message = "+connection(\"all\"):\"* axes : ";
			std::ostringstream ostr3; 
			ostr3 << SDL_JoystickNumAxes(joystick); 
			messagetmp = ostr3.str(); 
			message = message + messagetmp;
			message = message + "\";";
			urbi::send(message.c_str());

			message = "+connection(\"all\"):\"* balls : ";
			std::ostringstream ostr4; 
			ostr4 << SDL_JoystickNumBalls(joystick); 
			messagetmp = ostr4.str(); 
			message = message + messagetmp;
			message = message + "\";";
			urbi::send(message.c_str());

			message = "+connection(\"all\"):\"* hats : ";
			std::ostringstream ostr5; 
			ostr5 << SDL_JoystickNumHats(joystick); 
			messagetmp = ostr5.str(); 
			message = message + messagetmp;
			message = message + "\";";
			urbi::send(message.c_str());

			message = "+connection(\"all\"):\"* buttons : ";
			std::ostringstream ostr6; 
			ostr6 << SDL_JoystickNumButtons(joystick); 
			messagetmp = ostr6.str(); 
			message = message + messagetmp;
			message = message + "\";";
			urbi::send(message.c_str());
			SDL_JoystickClose(joystick);
		}
	}

	SDL_QuitSubSystem(SDL_INIT_JOYSTICK);
    atexit(SDL_Quit); // fin de SDL
}
