#include "pcmicro.h"

UStart(pcmicro);

pcmicro::pcmicro(std::string s) : UObject(s)
{
	UBindVar(pcmicro,val);
	UBindFunction(pcmicro,init);
	UBindFunction(pcmicro,record);
	URBI((+connection("all"):echo("To record the PC Micro during 5 seconds, makes pcmicro.record(5);");));
}

pcmicro::~pcmicro()
{
}

int pcmicro::init(void)
{
  return 1; // Singleton
}


/////////////////////////////////////////////
// ERRCHECK
/////////////////////////////////////////////
int pcmicro::ERRCHECK(FMOD_RESULT result)
{
    if (result != FMOD_OK)
    {
        printf("FMOD error! (%d) %s\n", result, FMOD_ErrorString(result));
        return -1;
    }

    return 0;
}
/////////////////////////////////////////////
// ERRCHECK
/////////////////////////////////////////////


/////////////////////////////////////////////
// record
/////////////////////////////////////////////
USound pcmicro::record(UValue duree)
{    

     // create a USound
    USound tmpSnd;
    tmpSnd.soundFormat = SOUND_RAW;
    tmpSnd.rate = 16000;
    tmpSnd.sampleSize = 16;
    tmpSnd.sampleFormat = SAMPLE_SIGNED;
    tmpSnd.channels = 1;
    tmpSnd.data = 0;
    tmpSnd.size = 0;


   if(duree.type == DATA_DOUBLE)
    {
        if ( InitSoundSystem() == -1 )
                   return tmpSnd;

        int duree_secondes = duree;
        int lengthrecord = InitRecord(duree_secondes); 
        if ( lengthrecord == -1 )
                   return tmpSnd;

        if ( StartRecord(lengthrecord) == -1 ) 
                   return tmpSnd;

        SaveToWav(sound);

      	quit();


      	FILE *f;
      	f = fopen("record.wav","r");
      	if (!f)
           return tmpSnd;

      	USound s;

      	struct stat st;
      	stat("record.wav",&st);

      	s.data = (char * )malloc(st.st_size);
      	s.soundFormat = SOUND_WAV;
      	s.size = st.st_size;
      	fread(s.data, 1,st.st_size, f);
      	convert(s, tmpSnd);

        val = tmpSnd;

    }

    else
    {
        	URBI((+connection("all"):echo("Incorrect argument. To record the PC Micro during 5 seconds, makes pcmicro.record(5);");));
        return tmpSnd;
    }

    return tmpSnd;

}
/////////////////////////////////////////////
// record
/////////////////////////////////////////////


/////////////////////////////////////////////
// saveToWav
/////////////////////////////////////////////
#if defined(WIN32) || defined(__WATCOMC__) || defined(_WIN32) || defined(__WIN32__)
    #define __PACKED                         /* dummy */
#else
//    #define __PACKED __attribute__((packed)) /* gcc packed */
    #define __PACKED  /* gcc packed */
#endif

void pcmicro::SaveToWav(FMOD_SOUND *sound)
{
    FILE *fp;
    int             channels, bits;
    float           rate;
    void           *ptr1, *ptr2;
    unsigned int    lenbytes, len1, len2;

    if (!sound)
    {
        return;
    }

    FMOD_Sound_GetFormat  (sound, 0, 0, &channels, &bits);
    FMOD_Sound_GetDefaults(sound, &rate, 0, 0, 0);
    FMOD_Sound_GetLength  (sound, &lenbytes, FMOD_TIMEUNIT_PCMBYTES);

    {
        #if defined(WIN32) || defined(_WIN64) || defined(__WATCOMC__) || defined(_WIN32) || defined(__WIN32__)
        #pragma pack(1)
        #endif
        
        /*
            WAV Structures
        */
        typedef struct
        {
	        signed char id[4];
	        int 		size;
        } RiffChunk;
    
        struct
        {
            RiffChunk       chunk           __PACKED;
            unsigned short	wFormatTag      __PACKED;    /* format type  */
            unsigned short	nChannels       __PACKED;    /* number of channels (i.e. mono, stereo...)  */
            unsigned int	nSamplesPerSec  __PACKED;    /* sample rate  */
            unsigned int	nAvgBytesPerSec __PACKED;    /* for buffer estimation  */
            unsigned short	nBlockAlign     __PACKED;    /* block size of data  */
            unsigned short	wBitsPerSample  __PACKED;    /* number of bits per sample of mono data */
        } FmtChunk  = { {{'f','m','t',' '}, sizeof(FmtChunk) - sizeof(RiffChunk) }, 1, channels, (int)rate, (int)rate *channels * bits / 16, 1 * channels * bits / 16, bits } __PACKED;

        struct
        {
            RiffChunk   chunk;
        } DataChunk = { {{'d','a','t','a'}, lenbytes } };

        struct
        {
            RiffChunk   chunk;
	        signed char rifftype[4];
        } WavHeader = { {{'R','I','F','F'}, sizeof(FmtChunk) + sizeof(RiffChunk) + lenbytes }, {'W','A','V','E'} };

        #if defined(WIN32) || defined(_WIN64) || defined(__WATCOMC__) || defined(_WIN32) || defined(__WIN32__)
        #pragma pack()
        #endif

        fp = fopen("record.wav", "wb");
       
        /*
            Write out the WAV header.
        */
        fwrite(&WavHeader, sizeof(WavHeader), 1, fp);
        fwrite(&FmtChunk, sizeof(FmtChunk), 1, fp);
        fwrite(&DataChunk, sizeof(DataChunk), 1, fp);

        /*
            Lock the sound to get access to the raw data.
        */
        FMOD_Sound_Lock(sound, 0, lenbytes, &ptr1, &ptr2, &len1, &len2);

        /*
            Write it to disk.
        */
        fwrite(ptr1, len1, 1, fp);

        /*
            Unlock the sound to allow FMOD to use it again.
        */
        FMOD_Sound_Unlock(sound, ptr1, ptr2, len1, len2);

        fclose(fp);
    }
}
/////////////////////////////////////////////
// saveToWav
/////////////////////////////////////////////




/////////////////////////////////////////////
// Start  InitSoundSystem
/////////////////////////////////////////////
int pcmicro::InitSoundSystem(void)
{
    system  = NULL;
    sound   = NULL;
    channel = NULL;
    datalength = 0;

    result = FMOD_System_Create(&system);
    if ( ERRCHECK(result) == -1 )
       return -1;

    result = FMOD_System_GetVersion(system, &version);
    if ( ERRCHECK(result) == -1 )
       return -1;

    if (version < FMOD_VERSION)
    {
        URBI(( echo("Error!  You are using an old version of FMOD"); ));
        return -1;
    }



//        System initialization
    result = FMOD_System_SetOutput(system, FMOD_OUTPUTTYPE_DSOUND);
    if ( ERRCHECK(result) == -1 )
       return -1;


    
 //       Enumerate record devices
    result = FMOD_System_GetRecordNumDrivers(system, &numdrivers);
    if ( ERRCHECK(result) == -1 )
       return -1;

}
/////////////////////////////////////////////
// Start  InitSoundSystem
/////////////////////////////////////////////


/////////////////////////////////////////////
// Start  InitRecord
/////////////////////////////////////////////
int pcmicro::InitRecord(int duree_secondes)
{
    if(duree_secondes > 30)
        duree_secondes=30;
    else if(duree_secondes < 1)
        duree_secondes=1;

    driver = 0;
    result = FMOD_System_SetRecordDriver(system, driver);
    if ( ERRCHECK(result) == -1 )
       return -1;
  
    result = FMOD_System_Init(system, 32, FMOD_INIT_NORMAL, NULL);
    if ( ERRCHECK(result) == -1 )
       return -1;

    memset(&exinfo, 0, sizeof(FMOD_CREATESOUNDEXINFO));

    exinfo.cbsize           = sizeof(FMOD_CREATESOUNDEXINFO);
    exinfo.numchannels      = 1; // 2 = stereo et 1 = mono
    exinfo.format           = FMOD_SOUND_FORMAT_PCM16;
    exinfo.defaultfrequency = 16000; // meilleur = 44100
    exinfo.length           = exinfo.defaultfrequency * sizeof(short) * exinfo.numchannels * duree_secondes;
    
    result = FMOD_System_CreateSound(system, 0, FMOD_2D | FMOD_SOFTWARE | FMOD_OPENUSER, &exinfo, &sound);
    if ( ERRCHECK(result) == -1 )
       return -1;

    return exinfo.length;
}
/////////////////////////////////////////////
// Start  InitRecord
/////////////////////////////////////////////


/////////////////////////////////////////////
// Start  Record
/////////////////////////////////////////////
int pcmicro::StartRecord(int lengthrecord)
{
    static FMOD_CHANNEL *channel = 0;
    static int   looping   = 0;
    int          recording = 0;
    int          playing   = 0;
    unsigned int recordpos = 0;
    unsigned int playpos   = 0;
    unsigned int length;

    int FALSE = 0;
    result = FMOD_System_RecordStart(system, sound, FALSE);
    if ( ERRCHECK(result) == -1 )
        return -1;

    while( recordpos < lengthrecord/2)
    {
        printf("Record pos = %6d / %i \r", recordpos,lengthrecord/2);

        FMOD_Sound_GetLength(sound, &length, FMOD_TIMEUNIT_PCM);
        if ( ERRCHECK(result) == -1 )
           return -1 ;

        FMOD_System_IsRecording(system, &recording);
        if ( ERRCHECK(result) == -1 )
           return -1;

        FMOD_System_GetRecordPosition(system, &recordpos);
        if ( ERRCHECK(result) == -1 )
           return  -1;

        FMOD_System_Update(system);            

        if( recordpos == lengthrecord/2)
            break;
    }

    result = FMOD_System_RecordStop(system);
    if ( ERRCHECK(result) == -1 )
        return  -1;

    printf("Record pos = %6d / %i \n", recordpos,lengthrecord/2);

    return 0;

}
/////////////////////////////////////////////
// Start  Record
/////////////////////////////////////////////


/////////////////////////////////////////////
// Quit
/////////////////////////////////////////////
void pcmicro::quit(void)
{
    //    Shut down
    result = FMOD_Sound_Release(sound);
    if ( ERRCHECK(result) == -1 )
        return ;

    result = FMOD_System_Release(system);
    if ( ERRCHECK(result) == -1 )
        return ;
}
/////////////////////////////////////////////
// Quit
/////////////////////////////////////////////
