/*
 * Copyright (C) Gostai S.A.S., 2007.  All rights reserved.
 *
 * This software is provided "as is" without warranty of any kind,
 * either expressed or implied, including but not limited to the
 * implied warranties of fitness for a particular purpose.
 *
 * See the LICENSE file for more information.
 * For comments, bug reports and feedback: http://www.urbiforge.com
 */
#include "houghcircles.hh"

HoughCircles::HoughCircles() {
  storage = cvCreateMemStorage(0);
  result = 0;
}

HoughCircles::~HoughCircles() {
  cvReleaseMemStorage(&storage);
}

CvSeq&
HoughCircles::getResult() {
  return *result;
}

void
HoughCircles::emptyResult() {
  cvClearMemStorage(storage);
  result = 0;
}

void
HoughCircles::compute(const IplImage& img) {
  IplImage*	tmp = 0;

  emptyResult();
  tmp = cvCreateImage(cvGetSize(&img), 8, 1);
  cvCvtColor(&img, tmp, CV_BGR2GRAY);
  cvSmooth(tmp, tmp, CV_GAUSSIAN, 9, 9);
  result = cvHoughCircles(tmp, storage, CV_HOUGH_GRADIENT,
			2, tmp->height/4, 200, 100);
  cvReleaseImage(&tmp);
}

void
HoughCircles::writeOn(IplImage& img) const {
  if (!result)
    std::cerr << "HoughCircles::writeOn(img) called before HoughCircles::compute(img)" << std::endl;
 for (int i = 0; i < result->total; ++i) {
    float* p = (float*)cvGetSeqElem(result, i);
    cvCircle(&img, cvPoint(cvRound(p[0]),cvRound(p[1])),
	     3, CV_RGB(0,255,0), -1, 8, 0);
    cvCircle(&img, cvPoint(cvRound(p[0]),cvRound(p[1])),
	     cvRound(p[2]), CV_RGB(255,0,0), 3, 8, 0);
  }
}
