/*
 * Copyright (C) Gostai S.A.S., 2007.  All rights reserved.
 *
 * This software is provided "as is" without warranty of any kind,
 * either expressed or implied, including but not limited to the
 * implied warranties of fitness for a particular purpose.
 *
 * See the LICENSE file for more information.
 * For comments, bug reports and feedback: http://www.urbiforge.com
 */
#ifndef HOUGHLINES_HH_
#define HOUGHLINES_HH_

#include <iostream>
#include <cv.h>
#include <cxcore.h>
#include <highgui.h>
#include <math.h>
#include "visionbehavior.hpp"

#define DST		150
#define SQUARE_DST	DST * DST
#define HACK_HEIGHT	80

/** \brief VisionBehavior 's implementation of
 * the Hough's transform to get lines
 * in an image.
 */
class		HoughLines : public VisionBehavior<CvSeq> {
private:
  CvSeq*	result;
  CvMemStorage*	storage;
  bool		hack;
public:
			HoughLines(bool hack = false);
  virtual		~HoughLines();
  virtual void		writeOn(IplImage& img) const;
  virtual CvSeq&	getResult();
  void			setHack(bool);
private:
  virtual void		compute(const IplImage& img);
  virtual void		emptyResult();
  void			workOnLines();
  void			freeItems();
};

#endif /* !HOUGHLINES_HH_ */
