/*
 * Copyright (C) Gostai S.A.S., 2007.  All rights reserved.
 *
 * This software is provided "as is" without warranty of any kind,
 * either expressed or implied, including but not limited to the
 * implied warranties of fitness for a particular purpose.
 *
 * See the LICENSE file for more information.
 * For comments, bug reports and feedback: http://www.urbiforge.com
 */
#include "lktracking.hh"

LkTracking::LkTracking() {
  image = 0;
  grey = 0;
  prev_grey = 0;
  pyramid = 0;
  prev_pyramid = 0;
  win_size = 10;
  count = 0;
  need_to_init = 0;
  night_mode = 0;
  flags = 0;
  add_remove_pt = 0;
  points[0] = (CvPoint2D32f*)cvAlloc(MAX_COUNT*sizeof(points[0][0]));
  points[1] = (CvPoint2D32f*)cvAlloc(MAX_COUNT*sizeof(points[0][0]));
  status = (char*)cvAlloc(MAX_COUNT);
}

LkTracking::~LkTracking() {

}

void
LkTracking::compute(const IplImage& img) {
  int	i;
  int	k;

  if (!image) {
    image = cvCreateImage(cvGetSize(&img), 8, 3);
    image->origin = img.origin;
    grey = cvCreateImage(cvGetSize(&img), 8, 1);
    prev_grey = cvCreateImage(cvGetSize(&img), 8, 1);
    pyramid = cvCreateImage(cvGetSize(&img), 8, 1);
    prev_pyramid = cvCreateImage(cvGetSize(&img), 8, 1);
  }
  cvCopy(&img, image, 0);
  cvCvtColor(image, grey, CV_BGR2GRAY);
  if (need_to_init) {
    IplImage* eig = cvCreateImage(cvGetSize(grey), 32, 1);
    IplImage* temp = cvCreateImage(cvGetSize(grey), 32, 1);
    double quality = 0.01;
    double min_distance = 10;

    count = MAX_COUNT;
    cvGoodFeaturesToTrack(grey, eig, temp, points[1], &count,
			  quality, min_distance, 0, 3, 0, 0.04);
    cvFindCornerSubPix(grey, points[1], count,
		       cvSize(win_size,win_size), cvSize(-1,-1),
		       cvTermCriteria(CV_TERMCRIT_ITER|CV_TERMCRIT_EPS,20,0.03));
    cvReleaseImage(&eig);
    cvReleaseImage(&temp);
    add_remove_pt = 0;
  }
  else if (count > 0) {
    cvCalcOpticalFlowPyrLK(prev_grey, grey, prev_pyramid, pyramid,
			   points[0], points[1], count, cvSize(win_size,win_size), 3, status, 0,
			   cvTermCriteria(CV_TERMCRIT_ITER|CV_TERMCRIT_EPS,20,0.03), flags);
    flags |= CV_LKFLOW_PYR_A_READY;
    result.clear();
    for (i = k = 0; i < count; i++) {
      for (CvPoints::iterator it = ptsToAdd.begin();
	   it != ptsToAdd.end();
	   ++it) {
	double dx = pt.x - points[1][i].x;
	double dy = pt.y - points[1][i].y;
	if (dx*dx + dy*dy <= 25)
	  ptsToAdd.erase(it);
      }
      if (!status[i])
	continue;
      points[1][k++] = points[1][i];
      result.push_back(cvPointFrom32f(points[1][i]));  
    }
    count = k;
  }
  BOOST_FOREACH(CvPoint it, ptsToAdd)
    if (count < MAX_COUNT) {
      points[1][count++] = cvPointTo32f(it);
      cvFindCornerSubPix(grey, points[1] + count - 1, 1,
			 cvSize(win_size,win_size), cvSize(-1,-1),
			 cvTermCriteria(CV_TERMCRIT_ITER|CV_TERMCRIT_EPS,20,0.03));
    }
  ptsToAdd.empty();
  CV_SWAP(prev_grey, grey, swap_temp);
  CV_SWAP(prev_pyramid, pyramid, swap_temp);
  CV_SWAP(points[0], points[1], swap_points);
  need_to_init = 0;
}

/**
 * \brief Replace the list of the points to track
 */
void
LkTracking::addPointsToTrack(CvPoints& cP) {
  ptsToAdd = cP;
}

void
LkTracking::emptyResult() {
  result.empty();
}

void
LkTracking::actOnKeyPressed(char& key) {
  switch (key) {
  case 'r':
    need_to_init = true;
    break;
  case 'c':
    count = 0;
    result.clear();
    break;
  default:
    ;
  }
}

void
LkTracking::writeOn(IplImage& img) const {
  BOOST_FOREACH(CvPoint i, result)
    cvCircle(&img, i, 3, CV_RGB(0,255,0), -1, 8,0);
}


CvPoints&
LkTracking::getResult() {
  return result;
}
