/*
 * Copyright (C) Gostai S.A.S., 2007.  All rights reserved.
 *
 * This software is provided "as is" without warranty of any kind,
 * either expressed or implied, including but not limited to the
 * implied warranties of fitness for a particular purpose.
 *
 * See the LICENSE file for more information.
 * For comments, bug reports and feedback: http://www.urbiforge.com
 */
#ifndef LKTRACKING_HH_
#define LKTRACKING_HH_

#include <iostream>
#include <cv.h>
#include <cxcore.h>
#include <highgui.h>
#include <math.h>
#include <vector>
#include <boost/foreach.hpp>
#include "visionbehavior.hpp"
#include "vision.hpp"

#define MAX_COUNT	200

/** \brief VisionBehavior 's implementation of 
 * the LK-Optical Flow (Lucas and Kanade) used to do
 * blob-tracking in an image.
 */
class		LkTracking : public VisionBehavior<CvPoints> {
private:
  CvPoints		result;
  IplImage*		image;
  IplImage*		grey;
  IplImage*		prev_grey;
  IplImage*		pyramid;
  IplImage*		prev_pyramid;
  IplImage*		swap_temp;
  int			win_size;
  CvPoint2D32f*		points[2];
  CvPoint2D32f*		swap_points;
  char*			status;
  int			count;
  int			need_to_init;
  int			night_mode;
  int			flags;
  int			add_remove_pt;
  CvPoint		pt;
  CvPoints		ptsToAdd;
public:
			LkTracking();
  virtual		~LkTracking();
  virtual void		writeOn(IplImage& img) const;
  virtual void		actOnKeyPressed(char& key);
  virtual CvPoints&	getResult();
  void			addPointsToTrack(CvPoints&);
private:
  virtual void		emptyResult();
  virtual void		compute(const IplImage& img);
};

#endif /* !LKTRACKING_HH_ */
