/*
 * Copyright (C) Gostai S.A.S., 2007.  All rights reserved.
 *
 * This software is provided "as is" without warranty of any kind,
 * either expressed or implied, including but not limited to the
 * implied warranties of fitness for a particular purpose.
 *
 * See the LICENSE file for more information.
 * For comments, bug reports and feedback: http://www.urbiforge.com
 */
#include <iostream>
#include <cxcore.h>
#include <highgui.h>
#include "vision.hpp"
#include "houghlines.hh"
#include "lktracking.hh"
#include "houghcircles.hh"
#include "squares.hh"

CvCapture*	getCameraInput(int argc, char* argv[]) {
  CvCapture*	camera = 0;

  if (argc > 1)
    camera = cvCreateFileCapture(argv[1]);
  else {
    camera = cvCreateCameraCapture(-1);
    if (!camera) {
      std::cerr << "No camera found" << std::endl;
      exit(1);
    }
    cvSetCaptureProperty(camera, CV_CAP_PROP_FRAME_WIDTH, 640);
    cvSetCaptureProperty(camera, CV_CAP_PROP_FRAME_HEIGHT, 480);
  }
  return camera;
}

void		initVision(Vision& vision) {
  vision.addBehavior<HoughLines>(new HoughLines(true));
  // vision.addBehavior<Squares>(new Squares());
  vision.addBehavior<LkTracking>(new LkTracking());
  vision.addBehavior<HoughCircles>(new HoughCircles());
  vision.setDelay(10);
}

void		welcome() {
  std::cout << "Welcome!" << std::endl;
  std::cout << "You can launch this program with ./vision: ";
  std::cout << "The default camera is used." << std::endl;
  std::cout << "You can launch it with ./vision my_video.avi: ";
  std::cout << "The program will use the video file." << std::endl;
  std::cout << "-------------------------------" << std::endl;
  std::cout << "Hot keys:" << std::endl;
  std::cout << "\tPress 'n' to switch night mode." << std::endl;
  std::cout << "\tPress 'r' to init the LK Optical " <<
    "Flow Tracker balls." << std::endl;
  std::cout << "\tPress 'c' to erase the LK Optical " <<
    "Flow Tracker balls." << std::endl;
  std::cout << "Have fun using it!" << std::endl;
  std::cout << "-----------" << std::endl;
}

int		main(int argc, char* argv[]) {
  CvCapture*	camera = 0;
  Vision	vision;

  welcome();
  initVision(vision);
  camera = getCameraInput(argc, argv);
  vision.workAndDisplay(*camera);
  cvReleaseCapture(&camera);
  return 0;
}
