/*
 * Copyright (C) Gostai S.A.S., 2007.  All rights reserved.
 *
 * This software is provided "as is" without warranty of any kind,
 * either expressed or implied, including but not limited to the
 * implied warranties of fitness for a particular purpose.
 *
 * See the LICENSE file for more information.
 * For comments, bug reports and feedback: http://www.urbiforge.com
 */
#ifndef VBKEYVISITOR_HPP_
#define VBKEYVISITOR_HPP_

#include <iostream>
#include <cv.h>
#include <cxcore.h>
#include <highgui.h>
#include <vector>
#include <string>
#include <boost/variant.hpp>
#include "visionbehavior.hpp"

/**
 * \brief Enum used to template the Variants visitors.
 * It changes the function called in the visitor.
 */
enum VBKeyEnum {KEYPRESSED};

/**
 * This type is usefull to call the Visitor with a char.
 * The visitor can't be called with a simple char, that is
 * the reason why I transform my simple char in a Variant<char>
 * before calling the visitor on it.
 */
typedef boost::variant<char>		VariantKeyArg;

/**
 * \brief A template for visiting Variants
 * with a key pressed (= a char)
 */
template<enum VBKeyEnum e>
class VBKeyVisitor : public boost::static_visitor<> {
public:
  template <class T>
  void	operator()(VisionBehavior<T>* vb,
		   char& c) const {
    switch (e) {
    case KEYPRESSED:
      vb->actOnKeyPressed(c);
      break;
    default:
      std::cerr << "Error: VBKeyVisitor called " <<
	"with unknown enum" << std::endl;
    }
  }

  template <class T>
  void	operator()(VisionBehavior<T>* vb) const {
    std::cerr << "Error: VBKeyVisitor called " <<
      "with bad enum" << std::endl;
  }
};

#endif /* !VBKEYVISITOR_HPP_ */
