/*
 * Copyright (C) Gostai S.A.S., 2007.  All rights reserved.
 *
 * This software is provided "as is" without warranty of any kind,
 * either expressed or implied, including but not limited to the
 * implied warranties of fitness for a particular purpose.
 *
 * See the LICENSE file for more information.
 * For comments, bug reports and feedback: http://www.urbiforge.com
 */
#ifndef VBVISITOR_HPP_
#define VBVISITOR_HPP_

#include <iostream>
#include <cv.h>
#include <cxcore.h>
#include <highgui.h>
#include <vector>
#include <string>
#include <boost/variant.hpp>
#include "visionbehavior.hpp"

/**
 * \brief Enum used to template the Variants visitors.
 * It changes the function called in the visitor.
 */
enum VBEnum {COMPUTE, WRITEON};

/**
 * This type is usefull to call the Visitor with an image.
 * The visitor can't be called with a simple image, that is
 * the reason why I transform my simple image in a Variant<Image>
 * before calling the visitor on it.
 */
typedef boost::variant<IplImage>	VariantImgArg;

/**
 * \brief A template for visiting Variants with an image
 */
template<enum VBEnum e>
class VBVisitor : public boost::static_visitor<> {
public:
  template <class T>
  void	operator()(VisionBehavior<T>* vb,
		   IplImage& src) const {
    std::cerr << "Error: VBVisitor(IplImage) called " <<
      "with unknown enum" << std::endl;
  }
};

/**
 * \brief Template specialized with the enum 'COMPUTE'.
 * It just means that the function called by the visitor
 * is 'doComputations()'.
 */
template<>
class VBVisitor<COMPUTE> : public boost::static_visitor<> {
public:
  template <class T>
  void	operator()(VisionBehavior<T>* vb,
		   IplImage& src) const {
    vb->doComputations(src);
  }
};

/**
 * \brief Template specialized with the enum 'WRITEON'.
 * It just means that the function called by the visitor
 * is 'writeOn()'.
 */
template<>
class VBVisitor<WRITEON> : public boost::static_visitor<> {
public:
  template <class T>
  void	operator()(VisionBehavior<T>* vb,
		   IplImage& src) const {
    vb->writeOn(src);
  }
};

#endif /* !VBVISITOR_HPP_ */
