/*
 * Copyright (C) Gostai S.A.S., 2007.  All rights reserved.
 *
 * This software is provided "as is" without warranty of any kind,
 * either expressed or implied, including but not limited to the
 * implied warranties of fitness for a particular purpose.
 *
 * See the LICENSE file for more information.
 * For comments, bug reports and feedback: http://www.urbiforge.com
 */
#ifndef VISIONBEHAVIOR_HH_
#define VISIONBEHAVIOR_HH_

#include <iostream>
#include <cxcore.h>
#include <vector>
#include <boost/variant.hpp>

#ifndef DEBUG
 #define DEBUG	false
#endif

/**
 *
 * \brief Interface for any vision computations.
 *
 * This interface is templated with the result of the computation.
 */
template <class T>
class VisionBehavior {
protected:
  static bool	debug;
  bool		enabled;
public:
  VisionBehavior();
  //! Destructor, used to free results
  virtual	~VisionBehavior() {};
  void		doComputations(const IplImage& img);
  //! This function draws the results of the computations on the picture. It means that 'writeOn()' is called more often than 'compute()'.
  virtual void	writeOn(IplImage& img) const = 0;
  //! This function is used to change the 'writeOn' behaviour
  virtual void	actOnKeyPressed(char& key) {key = key;};
  //! Returns the results of the computations
  virtual T&	getResult() = 0;
  void		switchOnOff();
private:
  virtual void	emptyResult() = 0;
  //! Fullfill this function which would be called according to the delay of the Vision class
  virtual void	compute(const IplImage& img) = 0;
};

template <class T>
bool VisionBehavior<T>::debug = DEBUG;

template <class T>
VisionBehavior<T>::VisionBehavior() {
  enabled = true;
}

template <class T>
void
VisionBehavior<T>::doComputations(const IplImage& img) {
  if (enabled)
    compute(img);
}

template <class T>
void
VisionBehavior<T>::switchOnOff() {
  if (enabled)
    emptyResult();
  enabled = !enabled;
}

#endif /* !VISIONBEHAVIOR_HH_ */
